package com.webssa;


import android.content.Intent;
import android.util.Log;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.broadcasting.impl.UploadGlobalNotesBroadcastReceiver;
import com.webssa.pda.sunmi.rest.PostNoteForShiftRestRequest;
import com.webssa.pda.sunmi.rest.helper.HttpClientProvider;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.NoteEntity;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.storage.dao.impl.LocalNoteDAO;
import com.webssa.pda.sunmi.storage.dao.impl.NoteDAO;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.LocalNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.NoteDbModel;
import com.webssa.pda.sunmi.util.DateNoTime;

import java.util.List;

import okhttp3.OkHttpClient;

import static com.webssa.pda.sunmi.service.UploadShiftNotesService.D;
import static com.webssa.pda.sunmi.service.UploadShiftNotesService.LOG_TAG;

public class Loader {
    public static boolean busy = false;

    public static void loadLocalNotes() {
        if (busy) {
            return;
        }
        busy = true;
        try (ShiftDAO dao = new ShiftDAO(new DbSelectionCache())) {
            LongId<ShiftRestEntity> shiftId = dao.getActiveShiftId(DateNoTime.today());
            if (shiftId != null) uploadAllLocalNotes(shiftId);
        }
        busy = false;
    }

    private static void uploadAllLocalNotes(LongId<ShiftRestEntity> shiftId) {
        try (LocalNoteDAO localDao = new LocalNoteDAO(); NoteDAO remoteDao = new NoteDAO()) {
            List<LocalNoteDbModel> notes = localDao.selectNotesForShiftOnly(shiftId);
            for (LocalNoteDbModel note : notes) {
                uploadLocalNote(shiftId, note, localDao, remoteDao);
            }
        }
    }

    private static void uploadLocalNote(LongId<ShiftRestEntity> shiftId, LocalNoteDbModel localNote, LocalNoteDAO localDao, NoteDAO remoteDao) {
        LongId<NoteEntity> remoteNoteId = null;
        localDao.beginTransaction();
        try {
            OkHttpClient httpClient = HttpClientProvider.getInstance().getApiHttpClient();
            PostNoteForShiftRestRequest request = PostNoteForShiftRestRequest.create(shiftId, localNote);
            boolean result = request.execute(httpClient);
            if (result) {
                NoteEntity remNote = request.getResultNote();
                localDao.deleteEntry(localNote);
                remoteDao.createOrUpdateInLocalDB(remNote);
                localDao.setTransactionSuccessful();
                remoteNoteId = remNote.id();
            } else if (D) {
                Log.d(LOG_TAG, "<--- Error sync note - " + request.getError());
            }
        } catch (Exception e) {
            if (D) Log.e(LOG_TAG, "<~~~ Exception sync note - " + e.getMessage());
        } finally {
            localDao.endTransaction();
        }

        if (D && (remoteNoteId != null)) {
            NoteDbModel note_db = remoteDao.selectEntryById(remoteNoteId);
            Log.d(LOG_TAG, "<--- Note has been uploaded - " + note_db);
        }
        Intent i = (new UploadGlobalNotesBroadcastReceiver(null)).getFullIntent();
        LocalBroadcastManager.getInstance(PDA_Application.getStaticAppContext()).sendBroadcast(i);
    }
}
