package com.webssa.library.crypto;

import android.util.Base64;

import java.security.GeneralSecurityException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

/**
 * Created by stanislav.perchenko on 24/08/2018.
 */

public class CryptoEngine {

    private final CryptoAlgorythmParams cryptoParams;
    private final byte[] aesKey;


    private final Cipher cipher;

    public CryptoEngine(CryptoAlgorythmParams cryptoParams, byte[] aesKey) {
        if (aesKey.length != 16) throw new IllegalArgumentException("Key must be 16 bytes long");
        this.cryptoParams = cryptoParams;
        this.aesKey = aesKey;
        try {
            cipher = Cipher.getInstance(cryptoParams.algorythm());
        } catch (NoSuchAlgorithmException | NoSuchPaddingException e ) {
            throw new RuntimeException(e);
        }
    }


    public synchronized byte[] decryptFromBase64(String closeBase64Text) throws GeneralSecurityException {
        byte[] encBytes = Base64.decode(closeBase64Text, Base64.DEFAULT);
        return decryptFromBytes(encBytes, 0, encBytes.length);
    }

    public synchronized byte[] decryptFromBytes(byte[] data, int offset, int length) throws GeneralSecurityException {
        if (cipher == null) {
            throw new NoSuchAlgorithmException("Encryption algorithm not supported on this device");
        }

        try {
            SecretKeySpec keySpec = new SecretKeySpec(aesKey, cryptoParams.keySpecAlgorythm());
            IvParameterSpec ivSpec = new IvParameterSpec(cryptoParams.initVector());
            cipher.init(Cipher.DECRYPT_MODE, keySpec, ivSpec);
        } catch (InvalidKeyException e) {
            throw new GeneralSecurityException("Bad decryption key format", e);
        } catch (InvalidAlgorithmParameterException e) {
            throw new GeneralSecurityException("Bad decryption algorythm parameter", e);
        }

        return cipher.doFinal(data, offset, length);
    }




    public synchronized String encryptToBase64(byte[] openData) throws GeneralSecurityException {
        if (cipher == null) {
            throw new NoSuchAlgorithmException("Encryption algorithm not supported on this device");
        }

        try {
            SecretKeySpec keySpec = new SecretKeySpec(aesKey, cryptoParams.keySpecAlgorythm());
            IvParameterSpec ivSpec = new IvParameterSpec(cryptoParams.initVector());
            cipher.init(Cipher.ENCRYPT_MODE, keySpec, ivSpec);
        } catch (InvalidKeyException e) {
            throw new GeneralSecurityException("Bad decryption key format", e);
        } catch (InvalidAlgorithmParameterException e) {
            throw new GeneralSecurityException("Bad decryption algorythm parameter", e);
        }

        byte[] encBytes = cipher.doFinal(openData);
        return Base64.encodeToString(encBytes, Base64.NO_WRAP);
    }

}
