package com.webssa.pda.sunmi;

import com.webssa.pda.sunmi.receipt.model.ReceiptEWCModel;
import com.webssa.pda.sunmi.receipt.model.ReceiptOrganizationModel;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

public final class GlobalProperties {

    private GlobalProperties() { }

    public static final boolean D = BuildConfig.DEBUG;// || BuildConfig.FLAVOR.startsWith(GlobalProperties.FLAVOR_NAME_DEV);

    /********************  Flavor names  **********************************************************/
    public static final String FLAVOR_NAME_DEV = "dev";
    public static final String FLAVOR_NAME_PROD = "prod";

    /********************  Database-related section  **********************************************/
    public static final String DB_NAME = "pda.db";
    public static final int DB_VERSION = 12;

    /**
     * This is to be used by a SimpleDateFormat instance
     */
    public static final String SERVER_PRIMARY_DATE_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss.SSSZ";     // milliseconds from 1 to 3 digits (required), zone - required (like +0500)
    public static final String[] SERVER_FALLBACK_DATE_TIME_FORMATS = new String[] {
            "yyyy-MM-dd'T'HH:mm:ssZ",                       // milliseconds are not allowed AND Zone is required
            "yyyy-MM-dd'T'HH:mm:ss.SSS",                    // milliseconds from 1 to MAX. Zone is allowed but ignored
            "yyyy-MM-dd'T'HH:mm:ss",                        // milliseconds and zone are allowed but ignored
    };

    /**
     * This is the Locale value using for formatting any String value within the application
     */
    public static final Locale APPLICATION_LOCALE = new Locale("en", "GB", "");

    /********************  HTTP client-related definitions  ***************************************/
    public static final boolean HTTP_UPSTREAM_USE_GZIP = true;
    public static final String HTTP_USER_AGENT_NAME = "PDA-app/1.0";

    /*************************  Picasso disc caching parameters  **********************************/
    public static final String PICASSO_CACHE_DIR_NAME = "my_picasso_cache";
    public static final float IMAGE_DISC_CACHE_ALLOWED_PART = 0.08f;
    public static final int MIN_IMAGE_DISK_CACHE_SIZE = 5 * 1024 * 1024; // 5MB
    public static final int MAX_IMAGE_DISK_CACHE_SIZE = 400 * 1024 * 1024; // 160MB
    public static final float MAX_IMAGE_MEM_CACHE_PART = 0.25f;


    public static final int SITE_PROXIMITY_DISTANCE_METERS = 300;

    public static final ReceiptOrganizationModel ORGANIZATION = ReceiptOrganizationModel.builder()
            .setOrganizationTitle("Choice textile Ltd")
            .setAddressLines(Arrays.asList("60 Coronation road", "Park Royal - London NW10 7PX"))
            .setPhone("020 8965 6699")
            .setFax("020 8965 6700")
            .setEmail("info@choicetextile.com")
            .setWebsite("www.choicetextile.com")
            .setWasteCarrierLicense("CBDU73882")
            .setVatRegNumber("777893451")
            .build();

    public static final List<ReceiptEWCModel> EWC_CODES = new ArrayList<>(4);
    static {
        EWC_CODES.add(ReceiptEWCModel.builder().setEwcCode("20 01 01").setEwcTitle("Paper").build());
        EWC_CODES.add(ReceiptEWCModel.builder().setEwcCode("20 01 10").setEwcTitle("Clothes").build());
        EWC_CODES.add(ReceiptEWCModel.builder().setEwcCode("20 01 11").setEwcTitle("Textiles").build());
        EWC_CODES.add(ReceiptEWCModel.builder().setEwcCode("20 02 03").setEwcTitle("Other non-\nbiodegradable").build());
    }
    public static final int RECEIPT_SIGNATURE_WIDTH_PX = 292;
    public static final int RECEIPT_SIGNATURE_STROKE_WIDTH = 2;

    public static final int COLLECT_ITEM_WEIGHT_MAX_DIGITS = 5;

    public static final int MAX_NUM_AD_HOC_VISITS = 5;

}
