package com.webssa.pda.sunmi.core;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.annotation.Nullable;
import androidx.lifecycle.LiveData;

import com.squareup.moshi.Moshi;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.rest.helper.ParserProvider;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * Created by stanislav.perchenko on 11/5/2019, 1:23 PM.
 */
public class EmergencyHolder {

    private static EmergencyHolder INSTANCE;

    public static EmergencyHolder getInstance() {
        if (INSTANCE == null) {
            synchronized (EmergencyHolder.class) {
                if (INSTANCE == null) {
                    INSTANCE = new EmergencyHolder(PDA_Application.getStaticAppContext());
                }
            }
        }
        return INSTANCE;
    }

    public interface OnEmergUpdateListener {
        void onUpdated(List<EmergencyCategoryEntity> items);
    }


    private final SharedPreferences sPref;
    private final Moshi parser;

    private final List<EmergencyCategoryEntity> mItems = new LinkedList();

    private EmergencyHolder(Context ctx) {
        sPref = ctx.getSharedPreferences(getClass().getSimpleName(), Context.MODE_PRIVATE);
        parser = ParserProvider.getApplicationMoshi();
        String json = sPref.getString("items", null);
        if (json == null) return;
        try {
            EmergencyCategoryEntity[] items = parser.adapter(EmergencyCategoryEntity[].class).fromJson(json);
            mItems.addAll(Arrays.asList(items));
            Collections.sort(mItems, (e1, e2) -> (e1.getOrder() - e2.getOrder()));
        } catch (IOException e) {
            System.err.print(e);
        }
    }


    void setData(Collection<EmergencyCategoryEntity> items) {
        synchronized (mItems) {
            if (mItems.equals(items)) return;
            mItems.clear();
            mItems.addAll(items);
            Collections.sort(mItems, (e1, e2) -> (e1.getOrder() - e2.getOrder()));
        }

        EmergencyCategoryEntity[] arr = items.toArray(new EmergencyCategoryEntity[items.size()]);
        String json = parser.adapter(EmergencyCategoryEntity[].class).toJson(arr);
        sPref.edit().putString("items", json).commit();

        List<OnEmergUpdateListener> obss;
        synchronized (mObservers) {
            obss = new ArrayList<>(mObservers.size());
            obss.addAll(mObservers);
        }

        List<EmergencyCategoryEntity> eee =  getEmergencies();
        for (OnEmergUpdateListener obs : obss) {
            obs.onUpdated(eee);
        }


    }

    public List<EmergencyCategoryEntity> getEmergencies() {
        synchronized (mItems) {
            EmergencyCategoryEntity[] items = mItems.toArray(new EmergencyCategoryEntity[mItems.size()]);
            return Arrays.asList(items);
        }
    }

    public LiveData<List<EmergencyCategoryEntity>> getEmergenciesLiveData() {
        return new EmergenciesLiveData();
    }

    @Nullable
    public EmergencyCategoryEntity getEmergencyById(@Nullable LongId<EmergencyCategoryEntity> emergId) {
        if (emergId != null) {
            synchronized (mItems) {
                for (EmergencyCategoryEntity item : mItems) {
                    if (item.id().equals(emergId)) return item;
                }
            }
        }
        return null;
    }



    /**********************************************************************************************/
    /*****************************  Data  update observer  ****************************************/
    /**********************************************************************************************/
    private final Set<OnEmergUpdateListener> mObservers = new HashSet<>();

    void requestUpdates(OnEmergUpdateListener l) {
        boolean upd;
        synchronized (mObservers) {
            upd = mObservers.add(l);
        }
        if (upd && !mItems.isEmpty()) {
            l.onUpdated(mItems);
        }
    }

    void removeUpdates(OnEmergUpdateListener l) {
        synchronized (mObservers) {
            mObservers.remove(l);
        }
    }



}
