package com.webssa.pda.sunmi.core;

import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.rest.ApplicationRestService;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;

import java.io.IOException;
import java.util.List;

import retrofit2.Response;

/**
 * Created by stanislav.perchenko on 11/5/2019, 2:37 PM.
 */
public class LoadEmergenciesService extends IntentService {

    public static final boolean D = GlobalProperties.D;
    public static final String LOG_TAG = "LOAD_GOODS_SERVICE";

    public static void launch(Context ctx) {
        Intent launcher = new Intent(ctx, LoadEmergenciesService.class);
        //launcher.putExtra(ARG_LAUNCH_REASON, LAUNCH_REASON_MANUALLY);
        ctx.startService(launcher);
    }

    public LoadEmergenciesService() {
        super(LoadEmergenciesService.class.getSimpleName());
    }

    @Override
    protected void onHandleIntent(@Nullable Intent intent) {
        if (D) Log.i(LOG_TAG, "---> Load Emergencies service has been started.");
        boolean isLoggedIn = SessionHolder.isLoggedIn();
        boolean isConnected= PDA_Application.getNetworkConnectionObserver().isConnected();
        if (!isLoggedIn || !isConnected) {
            if (D) Log.e(LOG_TAG, "<~~~ Cancel loading Emergencies because of not logged in or no Internet connection");
        } else {
            try {
                Response<List<EmergencyCategoryEntity>> resp = ApplicationRestService.getInstance().getAllEmergencies().execute();
                if (resp.isSuccessful()) {
                    List<EmergencyCategoryEntity> items = resp.body();
                    if (items != null) {
                        if (D) Log.d(LOG_TAG, String.format("<--- %d Emergency categories has been loaded from the backend", items.size()));
                        EmergencyHolder.getInstance().setData(items);
                    }
                }
            } catch (IOException e) {
                if (D) {
                    Log.e(LOG_TAG, "<~~~ Error loading Emergencies - "+e.getMessage(), e);
                } else {
                    System.err.println(e);
                }
            }
        }
    }
}
