package com.webssa.pda.sunmi.core;

import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.dataapi.ApiGetAllEWCCodes;
import com.webssa.pda.sunmi.dataapi.ApiGetAllGoodsWithPrices;

/**
 * Created by stanislav.perchenko on 10/18/2019.
 */
public class LoadGoodsService extends IntentService {
    public static final boolean D = GlobalProperties.D;
    public static final String LOG_TAG = "LOAD_GOODS_SERVICE";

    public static void launch(Context ctx) {
        Intent launcher = new Intent(ctx, LoadGoodsService.class);
        //launcher.putExtra(ARG_LAUNCH_REASON, LAUNCH_REASON_MANUALLY);
        ctx.startService(launcher);
    }

    public LoadGoodsService() {
        super(LoadGoodsService.class.getSimpleName());
    }

    @Override
    protected void onHandleIntent(@Nullable Intent intent) {
        if (D) Log.i(LOG_TAG, "---> Load Goods service has been started.");
        boolean isLoggedIn = SessionHolder.isLoggedIn();
        boolean isConnected= PDA_Application.getNetworkConnectionObserver().isConnected();
        if (!isLoggedIn || !isConnected) {
            if (D) Log.e(LOG_TAG, "<~~~ Cancel loading goods because of not logged in or no Internet connection");
        } else {
            loadEWCCodes();
            loadGoodsWithPrices();
        }
    }

    private void loadEWCCodes() {
        ApiGetAllEWCCodes api = new ApiGetAllEWCCodes(this);
        try {
            api.executeSynchronously();
            if (!D) {
                // Skip logging
            } else if(api.getResult() != null) {
                Log.d(LOG_TAG, String.format("<--- %d EWC codes has been loaded from the backend", api.getResult().size()));
            } else {
                Log.e(LOG_TAG, String.format("<~~~ Load EWC codes failed: %s - %s ", api.getError().getUserMessage(), api.getError().getInternalMessage()));
            }
        }catch (Exception e) {
            if (D) {
                Log.e(LOG_TAG, "<~~~ Error loading EWC codes - "+e.getMessage(), e);
            } else {
                System.err.println(e);
            }
        }
    }

    private void loadGoodsWithPrices() {
        ApiGetAllGoodsWithPrices api = new ApiGetAllGoodsWithPrices(this);
        try {
            api.executeSynchronously();
            if (!D) {
                // Skip logging
            } else if(api.getResult() != null) {
                Log.d(LOG_TAG, String.format("<--- %d good categories has been loaded from the backend", api.getResult().size()));
            } else {
                Log.e(LOG_TAG, String.format("<~~~ Load goods failed: %s - %s ", api.getError().getUserMessage(), api.getError().getInternalMessage()));
            }
        } catch (Exception e) {
            if (D) {
                Log.e(LOG_TAG, "<~~~ Error loading goods - "+e.getMessage(), e);
            } else {
                System.err.println(e);
            }
        }
    }
}
