package com.webssa.pda.sunmi.core;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.TextUtils;

import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.model.LocalVisitResult;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * Created by stanislav.perchenko on 12/11/2019, 10:50 AM.
 */
public final class LocalCompletedVisitsStorage {

    public static boolean saveLocalVisitResult(LocalVisitResult visitResult) {
        return getInstance()._saveLocalVisitResult(visitResult);
    }

    public static void removeVisitResult(LongId<VisitEntity> visitId) { //TODO Use this in the dedicated synchronization API !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        getInstance()._removeVisitResult(visitId);
    }

    @Nullable
    public static LocalVisitResult getLocalVisitResult(LongId<VisitEntity> visitId) { //TODO Use this in the dedicated synchronization API !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        return getInstance()._getLocalVisitResult(visitId);
    }

    /**********************************************************************************************/
    private static LocalCompletedVisitsStorage INSTANCE;

    private static LocalCompletedVisitsStorage getInstance() {
        if (INSTANCE == null) {
            synchronized (LocalCompletedVisitsStorage.class) {
                if (INSTANCE == null) {
                    INSTANCE = new LocalCompletedVisitsStorage(PDA_Application.getStaticAppContext());
                }
            }
        }
        return INSTANCE;
    }

    /**********************************************************************************************/
    private final Context context;
    private final SharedPreferences sPrefs;

    private LocalCompletedVisitsStorage(Context context) {
        this.context = context;
        sPrefs = context.getSharedPreferences(getClass().getSimpleName(), Context.MODE_PRIVATE);
    }

    private synchronized boolean _saveLocalVisitResult(LocalVisitResult visitResult) {
        try {
            sPrefs.edit().putString(visitResult.visitId().toString(), visitResult.toJsonObject().toString()).commit();
            return true;
        } catch (JSONException e) {
            return false;
        }
    }

    private synchronized void _removeVisitResult(LongId<VisitEntity> visitId) {
        sPrefs.edit().remove(visitId.toString()).commit();
    }

    @Nullable
    private synchronized LocalVisitResult _getLocalVisitResult(LongId<VisitEntity> visitId) {
        String txt = sPrefs.getString(visitId.toString(), null);
        if (TextUtils.isEmpty(txt)) {
            return null;
        } else {
            try {
                return LocalVisitResult.fromJson(new JSONObject(txt));
            } catch (JSONException e) {
                return null;
            }
        }
    }

}

