package com.webssa.pda.sunmi.core;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.TextUtils;

import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.model.UntiedLocalNote;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * Created by stanislav.perchenko on 12/12/2019, 12:53 PM.
 */
public final class LocalFailedVisitsStorage {

    public static boolean saveLocalVisitResult(LongId<VisitEntity> visitId, UntiedLocalNote note) {
        return getInstance()._saveLocalVisitResult(visitId, note);
    }

    public static void removeVisitResult(LongId<VisitEntity> visitId) { //TODO Use this in the dedicated synchronization API !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        getInstance()._removeVisitResult(visitId);
    }

    @Nullable
    public static UntiedLocalNote getLocalVisitResult(LongId<VisitEntity> visitId) { //TODO Use this in the dedicated synchronization API !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        return getInstance()._getLocalVisitResult(visitId);
    }

    /**********************************************************************************************/
    private static LocalFailedVisitsStorage INSTANCE;

    private static LocalFailedVisitsStorage getInstance() {
        if (INSTANCE == null) {
            synchronized (LocalFailedVisitsStorage.class) {
                if (INSTANCE == null) {
                    INSTANCE = new LocalFailedVisitsStorage(PDA_Application.getStaticAppContext());
                }
            }
        }
        return INSTANCE;
    }


    /**********************************************************************************************/
    private final Context context;
    private final SharedPreferences sPrefs;

    private LocalFailedVisitsStorage(Context context) {
        this.context = context;
        sPrefs = context.getSharedPreferences(getClass().getSimpleName(), Context.MODE_PRIVATE);
    }


    private synchronized boolean _saveLocalVisitResult(LongId<VisitEntity> visitId, UntiedLocalNote note) {
        try {
            sPrefs.edit().putString(visitId.toString(), note.toJsonObject().toString()).commit();
            return true;
        } catch (JSONException e) {
            return false;
        }
    }

    private synchronized void _removeVisitResult(LongId<VisitEntity> visitId) {
        sPrefs.edit().remove(visitId.toString()).commit();
    }

    @Nullable
    private synchronized UntiedLocalNote _getLocalVisitResult(LongId<VisitEntity> visitId) {
        String txt = sPrefs.getString(visitId.toString(), null);
        if (TextUtils.isEmpty(txt)) {
            return null;
        } else {
            try {
                return UntiedLocalNote.fromJsonObject(new JSONObject(txt));
            } catch (JSONException e) {
                return null;
            }
        }
    }
}
