package com.webssa.pda.sunmi.core;

import android.os.Looper;

import androidx.lifecycle.MutableLiveData;

/**
 * This is package-private implementation. Use this via SessionHolder
 *
 * Created by stanislav.perchenko on 10/28/2019, 2:40 PM.
 */
class SessionLiveData extends MutableLiveData<Session> {
    private final SessionHolder.OnSessionUpdateListener listener = new SessionHolder.OnSessionUpdateListener() {
        private Session mSession;

        @Override
        public void onSessionUpdated(Session s) {
            boolean needUpd;
            synchronized (this) {
                needUpd = !s.equals(mSession);
                mSession = s;
            }

            if (needUpd && (Thread.currentThread() == Looper.getMainLooper().getThread())) {
                SessionLiveData.this.setValue(s);
            } else if (needUpd) {
                SessionLiveData.this.postValue(s);
            }
        }
    };

    @Override
    protected void onActive() {
        SessionHolder.getInstance().requestUpdates(listener);
    }

    @Override
    protected void onInactive() {
        SessionHolder.getInstance().removeUpdates(listener);
    }

}
