package com.webssa.pda.sunmi.dataapi;

import android.content.Context;

import com.webssa.library.core.error.AppError;
import com.webssa.library.core.error.BadResponseError;
import com.webssa.pda.sunmi.rest.ApplicationRestService;
import com.webssa.pda.sunmi.rest.model.AcceptShiftResponse;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dao.impl.SitesDAO;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.ShiftDbModel;

import java.util.Set;

import retrofit2.Call;

/**
 * Created by stanislav.perchenko on 10/21/2019.
 */
public class ApiAcceptShift extends BaseRestApi<AcceptShiftResponse, ShiftDbModel> {

    public static ApiAcceptShift create(Context context, LongId<ShiftRestEntity> shiftId) {
        return new ApiAcceptShift(context, shiftId);
    }

    private final LongId<ShiftRestEntity> shiftId;

    // Use creator pattern
    private ApiAcceptShift(Context context, LongId<ShiftRestEntity> shiftId) {
        super(context);
        this.shiftId = shiftId;
    }

    @Override
    protected Call<AcceptShiftResponse> getRetrofitCall() {
        return ApplicationRestService.getInstance().acceptShift(shiftId);
    }

    @Override
    protected void setError(AppError error) {
        try { //TODO Remove after testing !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            Thread.sleep(800);
        } catch (InterruptedException e) {
            System.err.println(e);
        }

        super.setError(error);
    }

    @Override
    protected void continueWithResponse(AcceptShiftResponse resp) {

        try { //TODO Remove after testing !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            Thread.sleep(800);
        } catch (InterruptedException e) {
            System.err.println(e);
        }

        if (resp.getShift().id().equals(shiftId)) {
            continueWithSuccessResponse(resp.getShift(), resp.getRelatedVisits(), resp.getRelatedSites());
        } else {
            final String msg = String.format("Wrong Shift ID was returned - %d. Expected - %d", resp.getShift().id().getValue(), shiftId.getValue());
            setError(BadResponseError.Companion.create(getContext(), msg));
        }
    }

    private void continueWithSuccessResponse(ShiftRestEntity shift, Set<VisitEntity> visits, Set<SiteEntity> sites) {

        try (ShiftDAO shiftDao = new ShiftDAO(new DbSelectionCache());
             SitesDAO sitesDao = new SitesDAO();
             VisitDAO visitDao = new VisitDAO(null))
        {
            shiftDao.createOrUpdateInLocalDB(shift);
            for (SiteEntity site : sites) {
                sitesDao.createOrUpdateInLocalDB(site);
            }
            for (VisitEntity visit : visits) {
                visitDao.createOrUpdateInLocalDB(visit);
            }

            ShiftDbModel fullShift = shiftDao.selectEntryById(shift.id());
            setResult(fullShift);
        }
    }
}
