package com.webssa.pda.sunmi.dataapi;

import androidx.annotation.Nullable;

import com.webssa.library.core.error.ApiRestError;
import com.webssa.library.core.processor.AbstractExecutable;
import com.webssa.pda.sunmi.model.VisitCreationModel;
import com.webssa.pda.sunmi.rest.ApplicationRestService;
import com.webssa.pda.sunmi.rest.model.CreateAdHocVisitResponse;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.util.DateNoTime;

import java.io.IOException;

import retrofit2.Response;

/**
 * Created by stanislav.perchenko on 08.07.2020 at 13:45.
 */
public class ApiCreateAdHocVisit extends AbstractExecutable<VisitDbModel> {
    private final VisitCreationModel creationModel;
    private final boolean createLocalIfError;

    public static ApiCreateAdHocVisit create(VisitCreationModel creationModel, boolean createLocalIfError) {
        return new ApiCreateAdHocVisit(creationModel, createLocalIfError);
    }

    private ApiCreateAdHocVisit(VisitCreationModel creationModel, boolean createLocalIfError) {
        this.creationModel = creationModel;
        this.createLocalIfError = createLocalIfError;
    }

    @Override
    public void executeSynchronously() throws Exception {
        DbSelectionCache selCache = new DbSelectionCache();
        try (ShiftDAO shiftDao = new ShiftDAO(selCache);
             VisitDAO vDao = new VisitDAO(selCache))
        {

            VisitEntity createdVisit = createVisitRemote(creationModel);
            if (createdVisit != null) {
                vDao.createOrUpdateInLocalDB(createdVisit);
                VisitDbModel result = vDao.selectEntryById(createdVisit.id());
                ShiftRestEntity updShift = loadShift(creationModel.getShiftId());
                if (updShift != null) shiftDao.createOrUpdateInLocalDB(updShift);
                setResult(result);
            } else if (createLocalIfError) {
                VisitDbModel result = vDao.createAdHocVisitLocally(creationModel);
                setResult(result);
            } else {
                setError(ApiRestError.Companion.createNoPrefix("Visit not created"));
            }
        }
    }

    @Nullable
    private VisitEntity createVisitRemote(VisitCreationModel arg) {
        try {
            Response<CreateAdHocVisitResponse> resp = ApplicationRestService.getInstance().createAdHocVisit(arg).execute();
            return (resp.code() == 200 && resp.body() != null && resp.body().getVisit() != null)
                    ? resp.body().getVisit().copyWithAdHocFlag()
                    : null;
        } catch (IOException e) {
            return null;
        }
    }

    @Nullable
    private ShiftRestEntity loadShift(LongId<ShiftRestEntity> shiftId) {
        try {
            Response<ShiftRestEntity> resp = ApplicationRestService.getInstance().getShiftByID(shiftId).execute();
            return (resp.code() == 200 && resp.body() != null) ? resp.body() : null;
        } catch (IOException e) {
            return null;
        }
    }

}
