package com.webssa.pda.sunmi.dataapi;

import androidx.annotation.NonNull;

import com.webssa.library.core.processor.AbstractExecutable;
import com.webssa.pda.sunmi.model.UntiedLocalNote;
import com.webssa.pda.sunmi.rest.PostNoteForShiftRestRequest;
import com.webssa.pda.sunmi.rest.helper.HttpClientProvider;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.NoteEntity;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.storage.dao.impl.LocalNoteDAO;
import com.webssa.pda.sunmi.storage.dao.impl.NoteDAO;
import com.webssa.pda.sunmi.storage.dbmodel.BaseNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.LocalNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.NoteDbModel;

import java.util.Date;

import okhttp3.OkHttpClient;

/**
 * Created by stanislav.perchenko on 11/6/2019, 9:22 AM.
 */
public class ApiCreateNoteForShift extends AbstractExecutable<BaseNoteDbModel> {

    private final LongId<ShiftRestEntity> shiftId;
    private final UntiedLocalNote noteData;

    public ApiCreateNoteForShift(@NonNull LongId<ShiftRestEntity> shiftId, @NonNull UntiedLocalNote noteData) {
        assert (shiftId != null);
        assert (noteData != null);
        this.shiftId = shiftId;
        this.noteData = noteData;
    }

    @Override
    public void executeSynchronously() throws Exception {
        OkHttpClient httpClient = HttpClientProvider.getInstance().getApiHttpClient();
        PostNoteForShiftRestRequest request = PostNoteForShiftRestRequest.create(shiftId, noteData);
        boolean result = request.execute(httpClient);
        if (result) {
            NoteEntity note = request.getResultNote();
            try (NoteDAO dao = new NoteDAO()) {
                dao.createOrUpdateInLocalDB(note);
                NoteDbModel note_db = dao.selectEntryById(note.id());
                setResult(note_db);
            }
        } else {
            LocalNoteDbModel localNote = LocalNoteDbModel.builder()
                    .setId(null)
                    .setShiftId(shiftId)
                    .setOptVisitId(null)
                    .setCreatedAt(new Date())
                    .setNoteText(noteData.getNoteText())
                    .setOptEmergencyId((noteData.getOptEmergency() == null) ? null : noteData.getOptEmergency().id())
                    .setAttachedImages(noteData.getImportedImages())
                    .build();
            try (LocalNoteDAO dao = new LocalNoteDAO()) {
                LongId<LocalNoteDbModel> id = dao.createOrUpdateInLocalDB(localNote);
                localNote = localNote.withId(id);
            }
            setResult(localNote);
        }
    }
}
