package com.webssa.pda.sunmi.dataapi;

import android.content.Context;

import com.google.common.truth.Truth;
import com.webssa.library.core.error.BadResponseError;
import com.webssa.pda.sunmi.rest.ApplicationRestService;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.ShiftDbModel;

import retrofit2.Call;

/**
 * Created by stanislav.perchenko on 10/24/2019.
 */
public class ApiLoadShiftFromBackend extends BaseRestApi<ShiftRestEntity, ShiftDbModel> {

    public static ApiLoadShiftFromBackend create(Context context, LongId<ShiftRestEntity> shiftId) {
        return new ApiLoadShiftFromBackend(context, shiftId);
    }

    private final LongId<ShiftRestEntity> shiftId;

    private ApiLoadShiftFromBackend(Context context, LongId<ShiftRestEntity> shiftId) {
        super(context);
        Truth.assertThat(this.shiftId = shiftId).isNotNull();
    }

    @Override
    protected Call<ShiftRestEntity> getRetrofitCall() {
        return ApplicationRestService.getInstance().getShiftByID(shiftId);
    }

    @Override
    protected void continueWithResponse(ShiftRestEntity resp) {
        try { //TODO Remove after testing !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            Thread.sleep(800);
        } catch (InterruptedException e) {
            System.err.println(e);
        }

        if (resp.id().equals(shiftId)) {
            try (ShiftDAO dao = new ShiftDAO(new DbSelectionCache())) {
                dao.createOrUpdateInLocalDB(resp);
                ShiftDbModel fullShift = dao.selectEntryById(resp.id());
                setResult(fullShift);
            }
        } else {
            final String msg = String.format("Wrong Shift ID was returned - %d. Expected - %d", resp.id().getValue(), shiftId.getValue());
            setError(BadResponseError.Companion.create(getContext(), msg));
        }

    }
}
