package com.webssa.pda.sunmi.dataapi;

import android.content.Context;
import android.text.TextUtils;

import com.webssa.library.core.error.ApiRestError;
import com.webssa.library.core.error.LocalError;
import com.webssa.pda.sunmi.BuildConfig;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.core.SessionHolder;
import com.webssa.pda.sunmi.core.SingleVisitEditorStorage;
import com.webssa.pda.sunmi.rest.ApplicationRestService;
import com.webssa.pda.sunmi.rest.model.DriverLoginRequest;
import com.webssa.pda.sunmi.rest.model.LoginAnswer;
import com.webssa.pda.sunmi.rest.model.UserEntity;
import com.webssa.pda.sunmi.storage.dao.impl.CollectedGoodDAO;
import com.webssa.pda.sunmi.storage.dao.impl.LocalNoteDAO;
import com.webssa.pda.sunmi.storage.dao.impl.NoteDAO;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;

import retrofit2.Call;

public class ApiLogin extends BaseRestApi<LoginAnswer, UserEntity> {

    private final DriverLoginRequest credentials;
    private long tStart;

    public ApiLogin(Context context, DriverLoginRequest credentials) {
        super(context);
        this.credentials = credentials;
    }

    @Override
    protected Call<LoginAnswer> getRetrofitCall() {
        tStart = System.currentTimeMillis();
        return ApplicationRestService.getInstance().login(credentials);
    }


    protected void continueWithResponse(LoginAnswer respData) {
        int duration = BuildConfig.DEBUG ? 1600 : 800;
        int dt = (int)(System.currentTimeMillis() - tStart);
        if (dt < duration) {
            try {
                Thread.sleep(duration - dt);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        final Context c = getContext();
        if (!respData.isSuccess()) {
            setError(ApiRestError.Companion.createNoPrefix(TextUtils.isEmpty(respData.getErrorMessage()) ? "unknown reason." : respData.getErrorMessage()));
        } else if (SessionHolder.isLoggedIn()) {
            setError(LocalError.Companion.create(c, c.getString(R.string.err_already_logged_in)));
        } else {

            try (CollectedGoodDAO cgDao = new CollectedGoodDAO();
                 LocalNoteDAO lNoteDao = new LocalNoteDAO();
                 NoteDAO noteDao = new NoteDAO();
                 ShiftDAO shiftDao = new ShiftDAO(null);
                 VisitDAO visitDao = new VisitDAO(null)) {
                cgDao.removeAll();
                noteDao.removeAll();
                lNoteDao.removeAll();
                shiftDao.removeAll();
                visitDao.removeAll();
            }

            SingleVisitEditorStorage.getInstance().clear();

            SessionHolder.login(respData);
            setResult(respData.getUser());
        }
    }
}
