package com.webssa.pda.sunmi.dataapi;

import com.google.common.truth.Truth;
import com.webssa.library.core.error.AppError;
import com.webssa.library.core.processor.AbstractExecutable;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.ShiftDbModel;
import com.webssa.pda.sunmi.util.DateNoTime;

import org.jetbrains.annotations.Nullable;

/**
 * Created by stanislav.perchenko on 10/22/2019.
 */
public class ApiSelectSavedShiftForDate extends AbstractExecutable<ShiftDbModel> {

    public static ApiSelectSavedShiftForDate forDate(DateNoTime date) {
        return new ApiSelectSavedShiftForDate(date);
    }


    private final DateNoTime argDate;

    // Use Creator
    private ApiSelectSavedShiftForDate(DateNoTime argDate) {
        Truth.assertThat(argDate).isNotNull();
        this.argDate = argDate;
    }

    @Override
    public void executeSynchronously() throws Exception {
        try(ShiftDAO dao = new ShiftDAO(new DbSelectionCache())) {
            ShiftDbModel shift = dao.getShiftForDate(argDate);
            if (shift != null) {
                setResult(shift);
            } else {
                // This is a stub error entity to notify that no shift is found
                // It won't ctually be used.
                setError(new AppError() {
                    @Nullable
                    @Override
                    public String getInternalMessage() {
                        return "";
                    }

                    @Nullable
                    @Override
                    public String getUserMessage() {
                        return "";
                    }

                    @Nullable
                    @Override
                    public Exception getException() {
                        return null;
                    }
                });
            }
        }
    }
}
