package com.webssa.pda.sunmi.dataapi;

import android.content.Context;

import androidx.annotation.NonNull;

import com.webssa.library.core.error.LocalError;
import com.webssa.library.core.processor.AbstractExecutable;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;

/**
 * Created by stanislav.perchenko on 12/10/2019, 11:56 AM.
 */
public class ApiSelectVisitLocally extends AbstractExecutable<VisitDbModel> {

    private final Context ctx;
    private final LongId<VisitEntity> visitId;

    public ApiSelectVisitLocally(Context ctx, @NonNull LongId<VisitEntity> visitId) {
        this.ctx = ctx;
        assert (visitId != null);
        this.visitId = visitId;
    }

    @Override
    public void executeSynchronously() throws Exception {
        try(VisitDAO dao = new VisitDAO(new DbSelectionCache())) {
            VisitDbModel visit = dao.selectEntryById(visitId);
            if (visit == null) {
                setError(LocalError.Companion.create(ctx, "Visit not found - " + visitId));
            } else {
                setResult(visit);
            }
        }
    }
}
