package com.webssa.pda.sunmi.dataapi;

import android.content.Context;

import com.webssa.library.core.error.ApiRestError;
import com.webssa.library.core.error.IORestError;
import com.webssa.library.core.error.LocalError;
import com.webssa.library.core.processor.AbstractExecutable;
import com.webssa.pda.sunmi.rest.ApplicationRestService;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.StartVisitPutBody;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitStateEntity;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.util.DateNoTime;

import java.io.IOException;

import retrofit2.Response;

/**
 * Created by stanislav.perchenko on 11/8/2019, 5:37 PM.
 */
public class ApiStartVisit extends AbstractExecutable<VisitDbModel> {

    private final Context context;
    private final LongId<VisitEntity> visitId_1;

    public ApiStartVisit(Context context, LongId<VisitEntity> visitId) {
        assert (context != null);
        assert (visitId != null);
        this.context = context;
        this.visitId_1 = visitId;
    }

    @Override
    public void executeSynchronously() throws Exception {
        DbSelectionCache cache = new DbSelectionCache();
        try (ShiftDAO shiftDao = new ShiftDAO(cache); VisitDAO vDao = new VisitDAO(cache)) {
            VisitDbModel visit = vDao.selectEntryById(visitId_1);
            VisitState vs = visit.state();
            if (vs == null) {
                setError(LocalError.Companion.create(context, "Visit not found - "+visitId_1));
            } else if (vs != VisitState.NEW) {
                setError(LocalError.Companion.create(context, "Visit in wrong state - "+vs));
            } else {
                vDao.clear(visitId_1);



                if (visit.isAdHocVisit() && !visit.isRemoteCreated()) {
                    DateNoTime shiftDate = shiftDao.selectExecutionDateForShift(visit.shiftId());
                    ApiSyncVisitsBack apiSync = new ApiSyncVisitsBack(visit.shiftId());
                    VisitDbModel createdVisit = apiSync.createAdHocVisitRemote(visit, shiftDate);
                    if (createdVisit == null) {
                        //AdHoc visit was not created - start it locally
                        vDao.startVisitStateLocally(visit.id());
                        setResult(visit);
                        return;
                    } else {
                        // AdHoc visit was created - old one with fake ID was removed
                        visit = createdVisit;
                    }
                }

                boolean started = startVisitRemote(visit.id(), vDao);
                if (!started) {
                    vDao.startVisitStateLocally(visit.id());
                }
                visit = vDao.selectEntryById(visit.id());
                setResult(visit);
            }
        }
    }

    private boolean startVisitRemote(LongId<VisitEntity> visitId, VisitDAO dao) {
        try {
            Response<VisitStateEntity> response = ApplicationRestService.getInstance().startVisit(visitId,new StartVisitPutBody(null)).execute();
            if (response.code() == 200 && response.body() != null && visitId.equals(response.body().getId()) && (response.body().getState() == VisitState.IN_PROGRESS)) {
                VisitStateEntity new_state = response.body();
                dao.updateVisitStateFromRemote(new_state);
                return true;
            } else {
                return false;
            }
        } catch (IOException e) {
            return false;
        }
    }

}
