package com.webssa.pda.sunmi.dataapi;


import android.content.Context;

import com.squareup.moshi.JsonDataException;
import com.webssa.library.core.error.IORestError;
import com.webssa.library.core.error.ParseRestError;
import com.webssa.library.core.error.ServerRestError;
import com.webssa.library.core.processor.AbstractExecutable;

import java.io.IOException;

import retrofit2.Call;
import retrofit2.Response;

/**
 * Created by stanislav.perchenko on 10/18/2019.
 */
public abstract class BaseRestApi<TRESPONSE, TRESULT> extends AbstractExecutable<TRESULT> {

    protected abstract Call<TRESPONSE> getRetrofitCall();
    protected abstract void continueWithResponse(TRESPONSE respData);

    private final Context context;

    public BaseRestApi(Context context) {
        this.context = context;
    }

    public Context getContext() {
        return context;
    }

    @Override
    public void executeSynchronously() throws Exception {
        try {
            Call<TRESPONSE> call = getRetrofitCall();
            Response<TRESPONSE> response = call.execute();
            if (response.isSuccessful()) {
                TRESPONSE respData = response.body();
                if (respData == null) {
                    setError(ParseRestError.Companion.create(context, call.request().url().toString()));
                } else {
                    continueWithResponse(respData);
                }

            } else {
                setError(ServerRestError.Companion.create(context, response.code(), response.message()));
            }
        } catch (IOException e) {
            setError(IORestError.Companion.create(context, e));
        } catch (JsonDataException e) {
            setError(ParseRestError.Companion.create(context, e));
        }
    }
}
