package com.webssa.pda.sunmi.model;

import android.os.Parcel;
import android.os.Parcelable;

import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.PricedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by stanislav.perchenko on 11/14/2019, 7:59 PM.
 */
public final class LocalCollectedGoodItem implements Parcelable {

    private final LongId<SiteEntity> siteId;
    private final LongId<PricedGoodEntity> goodcategoryId;
    private int weightKg;

    public LocalCollectedGoodItem(LongId<SiteEntity> siteId, LongId<PricedGoodEntity> goodcategoryId, int weightKg) {
        assert (siteId != null);
        assert (goodcategoryId != null);
        assert (weightKg > 0);
        this.siteId = siteId;
        this.goodcategoryId = goodcategoryId;
        this.weightKg = weightKg;
    }

    public LongId<SiteEntity> getSiteId() {
        return siteId;
    }

    public LongId<PricedGoodEntity> getGoodcategoryId() {
        return goodcategoryId;
    }

    public int getWeightKg() {
        return weightKg;
    }

    public JSONObject toJsonObject() {
        try {
            JSONObject json = new JSONObject();
            json.put("site_id", siteId.getValue());
            json.put("good_cat_id", goodcategoryId.getValue());
            json.put("weight", weightKg);
            return json;
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
    }

    public String tojson() {
        return toJsonObject().toString();
    }

    public static LocalCollectedGoodItem fromJson(String json) throws JSONException {
        return fromJsonObject(new JSONObject(json));
    }

    public static LocalCollectedGoodItem fromJsonObject(JSONObject jo) throws JSONException {
        return new LocalCollectedGoodItem(
                new LongId<SiteEntity>(jo.getLong("site_id")),
                new LongId<PricedGoodEntity>(jo.getLong("good_cat_id")),
                jo.getInt("weight"));
    }

    public static List<LocalCollectedGoodItem> fromJsonArray(JSONArray jArr) throws JSONException {
        final int len = jArr.length();
        List<LocalCollectedGoodItem> items = new ArrayList<>(len);
        for (int i=0; i<len; i++) {
            items.add(fromJsonObject(jArr.getJSONObject(i)));
        }
        return items;
    }

    public static JSONArray toJsonArray(Collection<LocalCollectedGoodItem> items) {
        JSONArray jArr = new JSONArray();
        for (LocalCollectedGoodItem item : items) {
            jArr.put(item.toJsonObject());
        }
        return jArr;
    }

    public static String toJsonArrayString(Collection<LocalCollectedGoodItem> items) {
        return toJsonArray(items).toString();
    }

    /************************  Parcelable implementation  *****************************************/
    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        Object[] fieldArray = new Object[3];
        fieldArray[0] = siteId;
        fieldArray[0] = goodcategoryId;
        fieldArray[1] = weightKg;
        dest.writeArray(fieldArray);
    }

    protected LocalCollectedGoodItem(Parcel in) {
        Object[] fa = in.readArray(LocalCollectedGoodItem.class.getClassLoader());
        siteId = (LongId<SiteEntity>) fa[0];
        goodcategoryId = (LongId<PricedGoodEntity>) fa[1];
        weightKg = ((Integer) fa[2]).intValue();
    }

    public static final Creator<LocalCollectedGoodItem> CREATOR = new Creator<LocalCollectedGoodItem>() {
        public LocalCollectedGoodItem createFromParcel(Parcel in) {
            return new LocalCollectedGoodItem(in);
        }

        public LocalCollectedGoodItem[] newArray(int size) {
            return new LocalCollectedGoodItem[size];
        }
    };

}
