package com.webssa.pda.sunmi.model;

import android.os.Parcelable;

import androidx.annotation.Nullable;

import com.google.auto.value.AutoValue;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by stanislav.perchenko on 11/20/2019, 2:22 PM.
 */
@AutoValue
public abstract class LocalVisitResult implements Parcelable {

    public abstract LongId<VisitEntity> visitId();

    public abstract List<LocalCollectedGoodItem> collectedGoods();

    public abstract String driverName();

    public abstract String customerName();

    public abstract String customerSignature();

    public abstract boolean whiteSacks();
    public abstract int clearBags();

    @Nullable
    public abstract UntiedLocalNote optDriverNote();

    public LocalVisitResult withVisitId(LongId<VisitEntity> id) {
        return toBuilder().setVisitId(id).build();
    }

    abstract Builder toBuilder();

    public static Builder builder() {
        return new AutoValue_LocalVisitResult.Builder();
    }

    @AutoValue.Builder
    public abstract static class Builder {
        public abstract Builder setVisitId(LongId<VisitEntity> id);
        public abstract Builder setCollectedGoods(List<LocalCollectedGoodItem> collectedGoods);
        public abstract Builder setDriverName(String driverName);
        public abstract Builder setCustomerName(String customerName);
        public abstract Builder setCustomerSignature(String customerSignature);
        public abstract Builder setWhiteSacks(boolean value);
        public abstract Builder setClearBags(int value);
        public abstract Builder setOptDriverNote(@Nullable UntiedLocalNote optDriverNote);

        public abstract LocalVisitResult build();
    }

    public String toJson() {
        try {
            return toJsonObject().toString();
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
    }

    public JSONObject toJsonObject() throws JSONException {
        JSONObject jObj = new JSONObject();
        jObj.put("visitId", visitId().getValue());
        jObj.put("customerName", customerName());
        jObj.put("customerSignature", customerSignature());
        jObj.put("whiteSacks", whiteSacks());
        jObj.put("clearBags", clearBags());
        jObj.put("driverName", driverName());
        if (optDriverNote() != null) {
            jObj.put("optDriverNote", optDriverNote().toJsonObject());
        }

        JSONArray jGoods = new JSONArray();
        for (LocalCollectedGoodItem good : collectedGoods()) {
            jGoods.put(good.toJsonObject());
        }
        jObj.put("collectedGoods", jGoods);
        return jObj;
    }

    public static LocalVisitResult fromJson(JSONObject jo) throws JSONException {

        JSONArray jGoods = jo.getJSONArray("collectedGoods");
        List<LocalCollectedGoodItem> goods = new ArrayList<>(jGoods.length());
        for(int i=0; i<jGoods.length(); i++) {
            goods.add(LocalCollectedGoodItem.fromJsonObject(jGoods.getJSONObject(i)));
        }


        JSONObject jNote = jo.optJSONObject("optDriverNote");

        return builder()
                .setVisitId(LongId.Companion.valueOf(jo.getLong("visitId")))
                .setCollectedGoods(goods)
                .setDriverName(jo.getString("driverName"))
                .setCustomerName(jo.getString("customerName"))
                .setCustomerSignature(jo.getString("customerSignature"))
                .setWhiteSacks(jo.getBoolean("whiteSacks"))
                .setClearBags(jo.getInt("clearBags"))
                .setOptDriverNote((jNote == null) ? null : UntiedLocalNote.fromJsonObject(jNote))
                .build();
    }
}
