package com.webssa.pda.sunmi.rest;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.EnvironmentRelatedConstants;
import com.webssa.pda.sunmi.core.SettingsHolder;
import com.webssa.pda.sunmi.model.VisitCreationModel;
import com.webssa.pda.sunmi.rest.helper.DateQueryParameter;
import com.webssa.pda.sunmi.rest.helper.HttpClientProvider;
import com.webssa.pda.sunmi.rest.helper.ParserProvider;
import com.webssa.pda.sunmi.rest.model.AcceptShiftResponse;
import com.webssa.pda.sunmi.rest.model.CreateAdHocVisitResponse;
import com.webssa.pda.sunmi.rest.model.DriverLoginRequest;
import com.webssa.pda.sunmi.rest.model.FindNewShiftResponse;
import com.webssa.pda.sunmi.rest.model.LoginAnswer;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.StartVisitPutBody;
import com.webssa.pda.sunmi.rest.model.UserEntity;
import com.webssa.pda.sunmi.rest.model.VisitFailPostBody;
import com.webssa.pda.sunmi.rest.model.VisitResponse;
import com.webssa.pda.sunmi.rest.model.VisitResultPostBody;
import com.webssa.pda.sunmi.rest.model.impl.EWCCodeEntity;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.rest.model.impl.PricedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitStateEntity;

import java.util.List;

import retrofit2.Call;
import retrofit2.Retrofit;
import retrofit2.converter.moshi.MoshiConverterFactory;
import retrofit2.http.Body;
import retrofit2.http.GET;
import retrofit2.http.PUT;
import retrofit2.http.Path;
import retrofit2.http.Query;

public interface ApplicationRestService {

    ApplicationRestService INSTANCE_GOOD = new Retrofit.Builder()
            .baseUrl(EnvironmentRelatedConstants.BACKEND_API_URL)
            .client(HttpClientProvider.getInstance().getApiHttpClient())
            .addConverterFactory(MoshiConverterFactory.create(ParserProvider.getApplicationMoshi()))
            .build()
            .create(ApplicationRestService.class);

    ApplicationRestService INSTANCE_BAD = new Retrofit.Builder()
            .baseUrl("http://badapiserver.me/")
            .client(HttpClientProvider.getInstance().getApiHttpClient())
            .addConverterFactory(MoshiConverterFactory.create(ParserProvider.getApplicationMoshi()))
            .build()
            .create(ApplicationRestService.class);

    static ApplicationRestService getInstance() {
        return SettingsHolder.getInstance().getSettings().useFailNetworking() ? INSTANCE_BAD : INSTANCE_GOOD;
    }

    /**
     * API #1.1
     * @param body
     * @return
     */
    @NonNull
    @PUT("logindriver")
    Call<LoginAnswer> login(@Body @NonNull DriverLoginRequest body);

    /**
     * API #1.2
     * @return
     */
    @NonNull
    @GET("todaynewshift")
    Call<FindNewShiftResponse> findNewShiftForToday();

    /**
     * API #1.3
     * @return
     */
    @NonNull
    @GET("pricedgoodsv2")
    Call<List<PricedGoodEntity>> getAllGoodsWithPrices();

    /**
     * API #1.4
     * @param shiftId
     * @return
     */
    @NonNull
    @PUT("shift/{shift_id}/accept")
    Call<AcceptShiftResponse> acceptShift(@Path("shift_id") @NonNull LongId<ShiftRestEntity> shiftId);

    /**
     * API #1.5
     * @param shiftId
     * @return
     */
    @NonNull
    @PUT("shift/{shift_id}/reject")
    Call<ShiftRestEntity> rejectShift(@Path("shift_id") @NonNull LongId<ShiftRestEntity> shiftId);

    /**
     * API #1.6
     * @param shiftId
     * @return
     */
    @NonNull
    @GET("shift/{shift_id}")
    Call<ShiftRestEntity> getShiftByID(@Path("shift_id") @NonNull LongId<ShiftRestEntity> shiftId);


    /**
     * API #1.7
     * @param shiftId
     * @return
     */
    @NonNull
    @PUT("shift/{shift_id}/close")
    Call<ShiftRestEntity> closeShift(@Path("shift_id") @NonNull LongId<ShiftRestEntity> shiftId,
                                     @Query("close_time") @Nullable DateQueryParameter optCloseTime);

    /**
     * API #1.9
     * @param visitId
     * @return
     */
    @NonNull
    @PUT("visit/{visit_id}/start")
    Call<VisitStateEntity> startVisit(@Path("visit_id") @NonNull LongId<VisitEntity> visitId, @Body StartVisitPutBody optBody);

    /**
     * API #1.10
     * @param visitId
     * @return
     */
    @NonNull
    @PUT("visit/{visit_id}/fail")
    Call<VisitStateEntity> failVisit(@Path("visit_id") @NonNull LongId<VisitEntity> visitId,
                                     @Body @NonNull VisitFailPostBody failureDescription);

    /**
     * API #1.1
     * @param visitId
     * @return
     */
    @NonNull
    @PUT("visit/{visit_id}/complete")
    Call<VisitResponse> completeVisit(@Path("visit_id") @NonNull LongId<VisitEntity> visitId,
                                      @Body @NonNull VisitResultPostBody visitResult);

    /**
     * API #1.12
     * @return
     */
    @NonNull
    @GET("users/me")
    Call<UserEntity> getMyProfile();

    /**
     * API #1.13
     * @return
     */
    @NonNull
    @GET("emergencies")
    Call<List<EmergencyCategoryEntity>> getAllEmergencies();

    /**
     * API #1.14
     * @return
     */
    @NonNull
    @GET("ewccodes")
    Call<List<EWCCodeEntity>> getAllEWCCodes();

    @NonNull
    @GET("visit/{visit_id}/state")
    Call<VisitStateEntity> getVisitState(@Path("visit_id") @NonNull LongId<VisitEntity> visitId);


    @NonNull
    @GET("sites")
    Call<List<SiteEntity>> getAllSites();

    @NonNull
    @PUT("visit/add")
    Call<CreateAdHocVisitResponse> createAdHocVisit(@Body @NonNull VisitCreationModel visitResult);
}
