package com.webssa.pda.sunmi.rest;

import android.net.Uri;

import androidx.annotation.Nullable;

import com.webssa.library.attachment.model.CoverImportModel;
import com.webssa.library.core.error.AppError;
import com.webssa.library.core.error.IORestError;
import com.webssa.library.core.error.ParseRestError;
import com.webssa.library.core.error.ServerRestError;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.EnvironmentRelatedConstants;
import com.webssa.pda.sunmi.rest.helper.MyUrlBuilder;

import org.json.JSONArray;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * Created by stanislav.perchenko on 11/28/2019, 5:19 PM.
 */
public class UploadImagesRestRequest {

    public static UploadImagesRestRequest create(List<CoverImportModel> importedImages) {
        return new UploadImagesRestRequest(importedImages);
    }

    private final List<CoverImportModel> importedImages;

    private List<Uri> result;
    private AppError lastError;

    private UploadImagesRestRequest(List<CoverImportModel> importedImages) {
        this.importedImages = importedImages;
    }


    public synchronized boolean execute(OkHttpClient httpClient) {
        MyUrlBuilder urlBld = new MyUrlBuilder(EnvironmentRelatedConstants.BACKEND_API_URL, "/images/upload");

        Request request = new Request.Builder()
                .url(urlBld.build())
                .post(getBody())
                .build();

        //--- Execute request ---
        try {
            Response okHttpResponse = httpClient.newCall(request).execute();
            int httpCode = okHttpResponse.code();
            if (httpCode == 200) {
                try {
                    JSONArray jLinks = new JSONArray(okHttpResponse.body().string());
                    Uri[] links = new Uri[jLinks.length()];
                    for (int i = 0; i < links.length; i++) links[i] = Uri.parse(jLinks.getString(i));
                    result = Arrays.asList(links);
                    return true;
                } catch (Exception e) {
                    lastError = ParseRestError.Companion.create(PDA_Application.getStaticAppContext(), e);
                }
            } else {
                lastError = ServerRestError.Companion.create(PDA_Application.getStaticAppContext(), httpCode, okHttpResponse.message());
            }
        } catch (IOException e) {
            lastError = IORestError.Companion.create(PDA_Application.getStaticAppContext(), e);
        }
        return false;
    }

    private RequestBody getBody() {
        MultipartBody.Builder bodyBld = new MultipartBody.Builder()
                .setType(MultipartBody.FORM);

        for (CoverImportModel img : importedImages) {
            MediaType imgMediaType = MediaType.parse(img.mimeType());
            File imgFile = new File(img.localFile());
            bodyBld.addPart(MultipartBody.Part.createFormData("images", img.relativeHashName(),
                    RequestBody.create(imgMediaType, imgFile)));
        }
        return bodyBld.build();
    }


    @Nullable
    public List<Uri> getResult() {
        return result;
    }

    @Nullable
    public AppError getLastError() {
        return lastError;
    }
}
