package com.webssa.pda.sunmi.rest.jsonadapter;

import android.os.Build;

import androidx.annotation.NonNull;

import com.squareup.moshi.JsonAdapter;
import com.squareup.moshi.JsonDataException;
import com.squareup.moshi.JsonReader;
import com.squareup.moshi.JsonWriter;
import com.webssa.pda.sunmi.GlobalProperties;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class DateJsonAdapter extends JsonAdapter<Date> {

    public static final JsonAdapter.Factory FACTORY = (type, ___, __) -> (type == Date.class) ?
            new DateJsonAdapter() :
            null;

    private final DateFormat primaryFormat;
    private final DateFormat[] fallbackFormats;

    public DateJsonAdapter() {
        this.primaryFormat = new SimpleDateFormat(GlobalProperties.SERVER_PRIMARY_DATE_TIME_FORMAT, GlobalProperties.APPLICATION_LOCALE);
        this.fallbackFormats = new DateFormat[GlobalProperties.SERVER_FALLBACK_DATE_TIME_FORMATS.length];
        for (int i=0; i<fallbackFormats.length; i++) {
            this.fallbackFormats[i] = new SimpleDateFormat(GlobalProperties.SERVER_FALLBACK_DATE_TIME_FORMATS[i], GlobalProperties.APPLICATION_LOCALE);
        }
    }

    @Override
    public Date fromJson(JsonReader reader) throws IOException {
        Object value = reader.readJsonValue();
        return (value instanceof String) ? deserialize((String) value) : null;
    }

    public Date deserialize(String txtDate) throws JsonDataException {
        if (Build.VERSION.SDK_INT < 24) {
            txtDate = conv8601TimezoneTo822(txtDate);
        }


        ParseException lastError;
        try{
            return primaryFormat.parse(txtDate);
        } catch (ParseException e){
            lastError = null;
        }
        for (DateFormat parser : fallbackFormats) {
            try{
                return parser.parse(txtDate);
            } catch (ParseException e){
                lastError = null;
            }
        }
        throw new JsonDataException("error parse Date - "+txtDate, lastError);
    }

    private String conv8601TimezoneTo822(String dat) {
        final int len = dat.length();
        if ((len >= 7) && (dat.charAt(len-6) == '+' || dat.charAt(len-6) == '-') && (dat.charAt(len-3) == ':')) {
            return dat.substring(0, len-3) + dat.substring(len-2);
        } else {
            return dat;
        }
    }



    @Override
    public void toJson(JsonWriter writer, Date value) throws IOException {
        writer.value(value == null ? null : primaryFormat.format(value));
    }

    @NonNull
    @Override
    public String toString() {
        return "JsonAdapter(Date)";
    }
}