package com.webssa.pda.sunmi.rest.jsonadapter;

import android.location.Location;

import com.squareup.moshi.JsonAdapter;
import com.squareup.moshi.JsonReader;
import com.squareup.moshi.JsonWriter;

import java.io.IOException;

import javax.annotation.Nullable;

public class LocationJsonAdapter extends JsonAdapter<Location> {

    public static final JsonAdapter.Factory FACTORY = (type, ___, __) -> (type == Location.class) ? new LocationJsonAdapter() : null;

    private static final String FIELD_PROVIDER = "provider";
    private static final String FIELD_LATITUDE = "lat";
    private static final String FIELD_LONGITUDE = "lon";
    private static final String FIELD_ALTITUDE = "alt";
    private static final String FIELD_ACCURACY = "accuracy";
    private static final String FIELD_TIME = "timestamp";

    private final JsonReader.Options options = JsonReader.Options.of(FIELD_PROVIDER, FIELD_LATITUDE, FIELD_LONGITUDE, FIELD_ALTITUDE, FIELD_ACCURACY, FIELD_TIME);

    @Nullable
    @Override
    public Location fromJson(JsonReader reader) throws IOException {
        String provider = null;
        double lat = 0;
        double lon = 0;
        double alt = 0;
        float accu = 0;
        long time  = 0;

        reader.beginObject();
        while (reader.hasNext()) {
            switch (reader.selectName(options)) {
                case 0:
                    provider = reader.nextString();
                    break;
                case 1:
                    lat = reader.nextDouble();
                    break;
                case 2:
                    lon = reader.nextDouble();
                    break;
                case 3:
                    alt = reader.nextDouble();
                    break;
                case 4:
                    accu = (float) reader.nextDouble();
                    break;
                case 5:
                    time = reader.nextLong();
                    break;
            }
        }
        reader.endObject();
        Location loc = new Location(provider);
        loc.setLatitude(lat);
        loc.setLongitude(lon);
        loc.setAltitude(alt);
        loc.setAccuracy(accu);
        loc.setTime(time);
        return loc;
    }

    @Override
    public void toJson(JsonWriter writer, @Nullable Location value) throws IOException {
        if (value == null) {
            writer.value((String) null);
        } else {
            writer.beginObject();
            writer.name(FIELD_PROVIDER);
            writer.value(value.getProvider());
            writer.name(FIELD_LATITUDE);
            writer.value(value.getLatitude());
            writer.name(FIELD_LONGITUDE);
            writer.value(value.getLongitude());
            writer.name(FIELD_ALTITUDE);
            writer.value(value.getAltitude());
            writer.name(FIELD_ACCURACY);
            writer.value(value.getAccuracy());
            writer.name(FIELD_TIME);
            writer.value(value.getTime());
            writer.endObject();
        }
    }
}
