package com.webssa.pda.sunmi.rest.jsonadapter

import android.net.Uri
import android.text.TextUtils
import com.squareup.moshi.*
import com.webssa.pda.sunmi.rest.model.CollectedGoodUploadEntity
import com.webssa.pda.sunmi.rest.model.LongId
import com.webssa.pda.sunmi.rest.model.VisitResultPostBody
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity
import java.lang.reflect.Type
import java.util.*
import kotlin.String

/**
 * Created by stanislav.perchenko on 11/27/2019, 1:16 PM.
 */
class VisitResultPostBodyJsonAdapter(moshi: Moshi) : JsonAdapter<VisitResultPostBody>() {

    companion object {
        val FACTORY = object : JsonAdapter.Factory {
            override fun create(type: Type, annotations: Set<out Annotation>, m: Moshi): JsonAdapter<VisitResultPostBody>? {
                return if (type == VisitResultPostBody::class.java)
                    VisitResultPostBodyJsonAdapter(m)
                else
                    null
            }
        }

        val FFF = JsonAdapter.Factory { type, annots, m -> if (type == VisitResultPostBody::javaClass) VisitResultPostBodyJsonAdapter(m) else null }
    }

    private val options: JsonReader.Options =
            JsonReader.Options.of("visit_id", "delayed_complete_time",
                    "customer_name", "customer_signature", "note", "goods",
                    "clear_bags", "white_sacks")

    private val longIdOfVisitEntityAdapter: JsonAdapter<LongId<VisitEntity>> =
            moshi.adapter<LongId<VisitEntity>>(Types.newParameterizedType(LongId::class.java, VisitEntity::class.java), kotlin.collections.emptySet(), "visitId")

    private val nullableDateAdapter: JsonAdapter<Date?> =
            moshi.adapter<Date?>(Date::class.java, kotlin.collections.emptySet(), "delayedCompleteTime")

    private val stringAdapter: JsonAdapter<String> =
            moshi.adapter<String>(String::class.java, kotlin.collections.emptySet(), "customerName")

    private val intAdapter: JsonAdapter<Int> =
            moshi.adapter<Int>(Int::class.java, kotlin.collections.emptySet(), "")
    private val booleanAdapter: JsonAdapter<Boolean> =
            moshi.adapter<Boolean>(Boolean::class.java, kotlin.collections.emptySet(), "")

    private val nullableStringAdapter: JsonAdapter<String?> =
            moshi.adapter<String?>(String::class.java, kotlin.collections.emptySet(), "optNoteText")

    private val nullableListOfUriAdapter: JsonAdapter<List<Uri>?> =
            moshi.adapter<List<Uri>?>(Types.newParameterizedType(List::class.java, Uri::class.java), kotlin.collections.emptySet(), "noteImages")

    private val nullableListOfCollectedGoodUploadEntityAdapter: JsonAdapter<List<CollectedGoodUploadEntity>?> =
            moshi.adapter<List<CollectedGoodUploadEntity>?>(Types.newParameterizedType(List::class.java, CollectedGoodUploadEntity::class.java), kotlin.collections.emptySet(), "collectedGoods")

    override fun toString(): String = "GeneratedJsonAdapter(VisitResultPostBody)"

    override fun fromJson(reader: JsonReader): VisitResultPostBody {
        var visitId: LongId<VisitEntity>? = null
        var delayedComplTime: Date? = null
        var delayedComplTimeSet: Boolean = false
        var customerName: String? = null
        var customerSign: String? = null
        var clearBags: Int = 0
        var whiteSacks: Boolean = false
        var optNoteText: String? = null
        var optNoteTextSet: Boolean = false
        var noteImages: List<Uri>? = null
        var noteImagesSet: Boolean = false
        var collectedGoods: List<CollectedGoodUploadEntity>? = null
        var collectedGoodsSet: Boolean = false
        reader.beginObject()
        while (reader.hasNext()) {
            when (reader.selectName(options)) {
                0 -> visitId = longIdOfVisitEntityAdapter.fromJson(reader)
                        ?: throw JsonDataException("Non-null value 'visitId' was null at ${reader.path}")
                1 -> {
                    delayedComplTime = nullableDateAdapter.fromJson(reader)
                    delayedComplTimeSet = true
                }
                2 -> customerName = stringAdapter.fromJson(reader)
                        ?: throw JsonDataException("Non-null value 'customerName' was null at ${reader.path}")
                3 -> customerSign = stringAdapter.fromJson(reader)
                        ?: throw JsonDataException("Non-null value 'customerSign' was null at ${reader.path}")
                4 -> {
                    reader.beginObject()

                    for (i in 0 until 2) {
                        var name: String = reader.nextName()
                        if ("text".equals(name)) {
                            optNoteText = nullableStringAdapter.fromJson(reader)
                            optNoteTextSet = true
                        } else if ("images".equals(name)) {
                            noteImages = nullableListOfUriAdapter.fromJson(reader)
                            noteImagesSet = true
                        }
                    }

                    reader.endObject()
                }
                5 -> {
                    collectedGoods = nullableListOfCollectedGoodUploadEntityAdapter.fromJson(reader)
                    collectedGoodsSet = true
                }
                6 -> {
                    clearBags = intAdapter.fromJson(reader)
                            ?: throw JsonDataException("Non-null value was null at ${reader.path}")
                }
                7 -> {
                    whiteSacks = booleanAdapter.fromJson(reader)
                            ?: throw JsonDataException("Non-null value was null at ${reader.path}")
                }
                -1 -> {
                    // Unknown name, skip it.
                    reader.skipName()
                    reader.skipValue()
                }
            }
        }
        reader.endObject()
        var result = VisitResultPostBody(
                visitId = visitId ?: throw JsonDataException("Required property 'visitId' missing at ${reader.path}"),
                customerName = customerName
                        ?: throw JsonDataException("Required property 'customerName' missing at ${reader.path}"),
                customerSign = customerSign
                        ?: throw JsonDataException("Required property 'customerSign' missing at ${reader.path}"),
                whiteSacks = whiteSacks,
                clearBags = clearBags)
        result = result.copy(
                delayedCompleteTime = if (delayedComplTimeSet) delayedComplTime else result.delayedCompleteTime,
                optNoteText = if (optNoteTextSet) optNoteText else result.optNoteText,
                noteImages = if (noteImagesSet) noteImages else result.noteImages,
                collectedGoods = if (collectedGoodsSet) collectedGoods else result.collectedGoods)
        return result
    }

    override fun toJson(writer: JsonWriter, value: VisitResultPostBody?) {
        if (value == null) {
            throw NullPointerException("value was null! Wrap in .nullSafe() to write nullable values.")
        }
        writer.beginObject()
        writer.name("visit_id")
        longIdOfVisitEntityAdapter.toJson(writer, value.visitId)
        writer.name("delayed_complete_time")
        nullableDateAdapter.toJson(writer, value.delayedCompleteTime)
        writer.name("customer_name")
        stringAdapter.toJson(writer, value.customerName)
        writer.name("customer_signature")
        stringAdapter.toJson(writer, value.customerSign)

        writer.name("white_sacks")
        booleanAdapter.toJson(writer, value.whiteSacks)
        writer.name("clear_bags")
        intAdapter.toJson(writer, value.clearBags)

        if (!TextUtils.isEmpty(value.optNoteText)) {
            writer.name("note")
            writer.beginObject()
            writer.name("text")
            stringAdapter.toJson(writer, value.optNoteText)
            writer.name("opt_emergency_id").value(value.optEmergencyId)
            writer.name("images")
            nullableListOfUriAdapter.toJson(writer, value.noteImages)
            writer.endObject()
        }

        writer.name("goods")
        nullableListOfCollectedGoodUploadEntityAdapter.toJson(writer, value.collectedGoods)
        writer.endObject()
    }
}
