package com.webssa.pda.sunmi.rest.model.impl

import android.location.Location
import android.os.Parcel
import android.os.Parcelable
import com.squareup.moshi.Json
import com.squareup.moshi.JsonClass
import com.webssa.library.util.StringJoinerCompat
import com.webssa.pda.sunmi.rest.model.IdProvidingModel
import com.webssa.pda.sunmi.rest.model.LongId
import java.lang.IllegalStateException
import java.util.*

@JsonClass(generateAdapter = true)
data class SiteEntity(
        @Json(name = "site_id")
        val id: LongId<SiteEntity>,

        @Json(name = "shop_name")
        val siteName: String,

        @Json(name = "shop_code")
        val code: Int? = 0,

        @Json(name = "shop_code_name")
        val shopCodeText: String? = null,

        @Json(name = "address_line_1")
        val addressLine1: String,

        @Json(name = "address_line_2")
        val addressLine2: String? = null,

        @Json(name = "town")
        val town: String? = null,

        @Json(name = "city")
        val city: String? = null,

        @Json(name = "postcode")
        val postCode: String,

        @Json(name = "latitude")
        val latitude: Double,

        @Json(name = "longitude")
        val longitude: Double,

        @Json(name = "uuid")
        val uuid: String? = null,

        @Json(name = "payment_method")
        val paymentMethod: String
) : IdProvidingModel, Parcelable {

    override fun id(): LongId<SiteEntity> {
        return id
    }

    @Transient
    private lateinit var cachedLocation: Location

    fun location(): Location {
        if (! ::cachedLocation.isInitialized) {
            synchronized(this) {
                if (! ::cachedLocation.isInitialized) {
                    cachedLocation = Location("fused")
                    cachedLocation.latitude = latitude
                    cachedLocation.longitude = longitude
                }
            }
        }
        return cachedLocation
    }




    override fun describeContents() = 0

    private constructor(p: Parcel) : this(
            id = p.readParcelable(SiteEntity.javaClass.classLoader),
            siteName = p.readString()!!,
            code = p.readInt(),
            shopCodeText = p.readString(),
            addressLine1 = p.readString()!!,
            addressLine2 = p.readString(),
            town = p.readString(),
            city = p.readString(),
            postCode = p.readString()!!,
            latitude = p.readDouble(),
            longitude = p.readDouble(),
            uuid = p.readString(),
            paymentMethod = p.readString()
    )

    override fun writeToParcel(p: Parcel, flags: Int) {
        p.writeParcelable(id, flags)
        p.writeString(siteName)
        p.writeInt(code ?: 0)
        p.writeString(shopCodeText)
        p.writeString(addressLine1)
        p.writeString(addressLine2)
        p.writeString(town)
        p.writeString(city)
        p.writeString(postCode)
        p.writeDouble(latitude)
        p.writeDouble(longitude)
        p.writeString(uuid)
        p.writeString(paymentMethod)
    }

    companion object {
        @JvmField
        val CREATOR = object : Parcelable.Creator<SiteEntity> {
            override fun createFromParcel(parcel: Parcel) = SiteEntity(parcel)
            override fun newArray(size: Int) = arrayOfNulls<SiteEntity>(size)
        }
    }

    class Builder {
        private lateinit var id: LongId<SiteEntity>
        private lateinit var name: String
        private var code: Int = 0
        private var codeText: String? = null
        private lateinit var address1: String
        private var address2: String? = null
        private var town: String? = null
        private var city: String? = null
        private lateinit var postCode: String
        private var latitude: Double = 0.0
        private var longitud: Double = 0.0
        private lateinit var uuid: String
        private lateinit var payMethod: String

        fun setId(id: LongId<SiteEntity>) = apply { this.id = id }
        fun setShopName(name: String) = apply { this.name = name }
        fun setShopCode(code: Int) = apply { this.code = code }
        fun setShopCodeText(codeTxt: String?) = apply { this.codeText = codeTxt }
        fun setAddressLine1(address: String) = apply { this.address1 = address }
        fun setAddressLine2(address: String?) = apply { this.address2 = address }
        fun setTown(t: String?) = apply { this.town = t }
        fun setCity(c: String?) = apply { this.city = c }
        fun setPostCode(pc: String) = apply { this.postCode = pc }
        fun setLatitude(latitude: Double) = apply { this.latitude = latitude }
        fun setLongitude(longitude: Double) = apply { this.longitud = longitude }
        fun setUuid(uuid: String) = apply { this.uuid = uuid }
        fun setPayMethod(method: String) = apply { this.payMethod = method }

        fun build(): SiteEntity {
            val missed = StringJoinerCompat(", ", "", "")
            if (! ::id.isInitialized) missed.add("id")
            if (! ::name.isInitialized) missed.add("name")
            if (! ::address1.isInitialized) missed.add("address_line_1")
            if (! ::postCode.isInitialized) missed.add("postcode")
            if (! ::uuid.isInitialized) missed.add("uuid")

            if (missed.length() == 0) {
                return SiteEntity(id = id, siteName = name, code = code, shopCodeText = codeText, addressLine1 = address1, addressLine2 = address2, town = town, city = city, postCode = postCode, latitude = latitude, longitude = longitud, uuid = uuid, paymentMethod = payMethod)
            } else {
                throw IllegalStateException("SiteEntity.Builder - some fields are missed: $missed")
            }
        }
    }

}