package com.webssa.pda.sunmi.storage;


import android.content.ContentValues;
import android.database.Cursor;
import android.location.Location;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.squareup.moshi.Moshi;
import com.webssa.pda.sunmi.rest.helper.ParserProvider;
import com.webssa.pda.sunmi.rest.model.IdProvidingModel;
import com.webssa.pda.sunmi.storage.dbhelper.DatabaseManager;

import org.json.JSONArray;
import org.json.JSONException;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by stanislav.perchenko on 10/25/2016.
 */
public final class DbUtils {

    private DbUtils() {
        throw new IllegalStateException("No instances!");
    }

    private static final Moshi helperMoshi = ParserProvider.getApplicationMoshi();




    public static ContentValues putDateToContentValues(ContentValues inValues, String key, @Nullable Date date) {
        inValues.put(key, (date != null) ? date.getTime() : 0);
        return inValues;
    }

    public static synchronized ContentValues putLocationToContentValues(ContentValues inValues, String key, @Nullable Location locat) {
        inValues.put(key, (locat == null) ? "" : helperMoshi.adapter(Location.class).toJson(locat));
        return inValues;
    }

    public static synchronized ContentValues putURLToContentValues(ContentValues inValues, String key, @Nullable URL url) {
        if (url != null) inValues.put(key, url.toString());
        return inValues;
    }

    public static synchronized ContentValues putIdsToValuesAsJson(ContentValues inValues, String key, long... ids) {
        JSONArray jIds = new JSONArray();
        for (long id : ids) {
            jIds.put(id);
        }
        inValues.put(key, jIds.toString());
        return inValues;
    }

    public static synchronized ContentValues putIdsToValuesAsJson(ContentValues inValues, String key, Long... ids) {
        JSONArray jIds = new JSONArray();
        for (long id : ids) {
            jIds.put(id);
        }
        inValues.put(key, jIds.toString());
        return inValues;
    }

    public static synchronized ContentValues putIdsToValuesAsJson(ContentValues inValues, String key, Collection<Long> ids) {
        JSONArray jIds = new JSONArray();
        for (long id : ids) {
            jIds.put(id);
        }
        inValues.put(key, jIds.toString());
        return inValues;
    }



    public static synchronized Location optLocationFromCursor(Cursor c, String columnName) {
        String locationJson = c.getString(c.getColumnIndex(columnName));
        return TextUtils.isEmpty(locationJson) ? null : helperMoshi.adapter(Location.class).fromJsonValue(locationJson);
    }


    public static String buildWhereSectionIN(String column, long[] values) {
        StringBuilder sb = new StringBuilder("("+column+" IN (");
        for (int i=0; i<values.length; i++) {
            sb.append((i > 0) ? ", "+values[i] : ""+values[i]);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionIN(String column, Long[] values) {
        StringBuilder sb = new StringBuilder("("+column+" IN (");
        for (int i=0; i<values.length; i++) {
            sb.append((i > 0) ? ", "+values[i] : ""+values[i]);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionIN(String column, String[] values) {
        StringBuilder sb = new StringBuilder("("+column+" IN (");
        for (int i=0; i<values.length; i++) {
            String part = String.format(((i > 0) ? ", '%s'" : "'%s'"), values[i]);
            sb.append(part);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionIN(String column, List<String> values) {
        StringBuilder sb = new StringBuilder("("+column+" IN (");
        for (int i=0; i<values.size(); i++) {
            String part = String.format(((i > 0) ? ", '%s'" : "'%s'"), values.get(i));
            sb.append(part);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionIN(String column, Object[] values) {
        StringBuilder sb = new StringBuilder("("+column+" IN (");
        for (int i=0; i<values.length; i++) {
            String part = String.format(((i > 0) ? ", '%s'" : "'%s'"), values[i].toString());
            sb.append(part);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionNotIN(String column, long[] values) {
        StringBuilder sb = new StringBuilder("("+column+" NOT IN (");
        for (int i=0; i<values.length; i++) {
            sb.append((i > 0) ? ", "+values[i] : ""+values[i]);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionNotIN(String column, Long[] values) {
        StringBuilder sb = new StringBuilder("("+column+" NOT IN (");
        for (int i=0; i<values.length; i++) {
            sb.append((i > 0) ? ", "+values[i] : ""+values[i]);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionNotIN(String column, String[] values) {
        StringBuilder sb = new StringBuilder("("+column+" NOT IN (");
        for (int i=0; i<values.length; i++) {
            String part = String.format(((i > 0) ? ", '%s'" : "'%s'"), values[i]);
            sb.append(part);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionNotIN(String column, List<String> values) {
        StringBuilder sb = new StringBuilder("("+column+" NOT IN (");
        for (int i=0; i<values.size(); i++) {
            String part = String.format(((i > 0) ? ", '%s'" : "'%s'"), values.get(i));
            sb.append(part);
        }
        sb.append("))");
        return sb.toString();
    }

    public static String buildWhereSectionNotIN(String column, Object[] values) {
        StringBuilder sb = new StringBuilder("("+column+" NOT IN (");
        for (int i=0; i<values.length; i++) {
            String part = String.format(((i > 0) ? ", '%s'" : "'%s'"), values[i].toString());
            sb.append(part);
        }
        sb.append("))");
        return sb.toString();
    }




    public static <MOD extends IdProvidingModel> Collection<MOD> reorderDbModelByIds(@NonNull Map<Long, MOD> inModels, @NonNull long[] ids) {
        Collection<MOD> ret = new ArrayList<>(ids.length);
        for (long id : ids) {
            MOD item = inModels.get(id);
            if (item != null) ret.add(item);
        }
        return ret;
    }

    @Nullable
    public static URL optURLFromCursor(Cursor c, String columnName) {
        String icUrl = c.getString(c.getColumnIndex(columnName));
        try {
            return (icUrl == null) ? null : new URL(icUrl);
        } catch (MalformedURLException e) {
            return null;
        }
    }



    public static  long[] getIdArrayFromCursor(Cursor c, String columnName) {
        String txtData = "";
        try {
            JSONArray jArr = new JSONArray(txtData = c.getString(c.getColumnIndex(columnName)));
            long[] ids = new long[jArr.length()];
            for (int i = 0; i < jArr.length(); i++) {
                ids[i] = jArr.getLong(i);
            }
            return ids;
        } catch (JSONException e) {
            e.printStackTrace();
            Log.e(DatabaseManager.DB_LOG_TAG, "Runtime database error. Wrong data format. JSON array of type long expected in the field "+columnName+". Got: "+txtData);
            return new long[0];
        }
    }



    public static class LocalizationWhereClauseBuilder {
        private String parentIdColumn;
        private Long parentId;
        private String locale;

        public LocalizationWhereClauseBuilder setParentIdColumn(String parentIdColumn) {
            this.parentIdColumn = parentIdColumn;
            return this;
        }

        public LocalizationWhereClauseBuilder setParentId(long parentId) {
            this.parentId = parentId;
            return this;
        }

        public LocalizationWhereClauseBuilder setLocale(String locale) {
            this.locale = locale;
            return this;
        }

        public String build() {
            if ((parentIdColumn == null) || parentId == null || locale == null) {
                throw new IllegalStateException("Not all fields for localization clause were set");
            }
            return String.format("(%s = %d AND %s = '%s')", parentIdColumn, parentId, COLUMN_LOCALE, locale);
        }
    }

    public static final String COLUMN_LOCALE = "locale";
}
