package com.webssa.pda.sunmi.storage.dao.impl;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.CollectedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.PricedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dao.BaseDAO;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by stanislav.perchenko on 11/22/2019, 2:54 PM.
 */
public class CollectedGoodDAO extends BaseDAO<CollectedGoodEntity, CollectedGoodEntity> {

    public static final String TABLE_NAME = "Collectedgoods";

    public static final String COLUMN_ID = "_id";
    public static final String COLUMN_VISIT_ID = "visit_id";
    public static final String COLUMN_GOOD_CAT_ID = "goodcat_id";
    public static final String COLUMN_NAME = "name";
    public static final String COLUMN_WEIGHT = "weight";
    public static final String COLUMN_TOT_AMOUNT = "tot_amount";
    public static final String COLUMN_TOT_SCALE = "tot_scale";
    public static final String COLUMN_ORDER = "o_r_d_e_r";

    public static final String[] SELECTION_COLUMNS = {COLUMN_ID, COLUMN_VISIT_ID, COLUMN_GOOD_CAT_ID, COLUMN_NAME, COLUMN_WEIGHT, COLUMN_TOT_AMOUNT, COLUMN_TOT_SCALE, COLUMN_ORDER};

    private static final String[] CREATIONS = new String[]{
            //                                       id             visit_id    good_id     name     weight      amount      scale       order
            String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY, %s INTEGER, %s INTEGER, %s TEXT, %s INTEGER, %s INTEGER, %s INTEGER, %s INTEGER);",
                    TABLE_NAME,
                    COLUMN_ID,
                    COLUMN_VISIT_ID,
                    COLUMN_GOOD_CAT_ID,
                    COLUMN_NAME,
                    COLUMN_WEIGHT,
                    COLUMN_TOT_AMOUNT,
                    COLUMN_TOT_SCALE,
                    COLUMN_ORDER)
    };

    private static final String[] DELETIONS = new String[] {
            String.format("DROP TABLE IF EXISTS %s", TABLE_NAME)
    };

    public static void createTables(SQLiteDatabase db) {
        for (String cr : CREATIONS) db.execSQL(cr);
    }

    public static void dropTables(SQLiteDatabase db) {
        for (String del : DELETIONS) db.execSQL(del);
    }

    public CollectedGoodDAO() {
        super();
    }

    @Override
    protected String getTableName() {
        return TABLE_NAME;
    }

    @Override
    protected String[] getSelectionColumns() {
        return SELECTION_COLUMNS;
    }

    @Override
    protected String getIdColumnName() {
        return COLUMN_ID;
    }

    @Override
    public LongId<CollectedGoodEntity> createOrUpdateInLocalDB(CollectedGoodEntity entry) {
        ContentValues vvv = new ContentValues();
        vvv.put(COLUMN_VISIT_ID, entry.getVisitId().getValue());
        vvv.put(COLUMN_GOOD_CAT_ID, entry.getGoodcategoryId().getValue());
        vvv.put(COLUMN_NAME, entry.getGoodName());
        vvv.put(COLUMN_WEIGHT, entry.getCollectedWeight());
        vvv.put(COLUMN_TOT_AMOUNT, entry.getCollectedTotalAmount());
        vvv.put(COLUMN_TOT_SCALE, entry.getCollectedTotalScale());
        vvv.put(COLUMN_ORDER, entry.getOrder());

        String where = COLUMN_ID + " = " + entry.id().getValue();
        if (getWriteableDatabase().update(TABLE_NAME, vvv, where, null) == 0) {
            vvv.put(COLUMN_ID, entry.id().getValue());
            long id = getWriteableDatabase().insert(TABLE_NAME, null, vvv);
            return (id > 0) ? LongId.Companion.valueOf(id) : null;
        } else {
            return entry.id();
        }
    }

    @Override
    public int deleteEntry(CollectedGoodEntity entry) {
        return super.deleteEntryById(entry.id());
    }

    @Override
    public CollectedGoodEntity buildDbModel(Cursor c) {
        LongId<CollectedGoodEntity> id = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_ID)));
        LongId<VisitEntity> visitId = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_VISIT_ID)));
        LongId<PricedGoodEntity> goodId = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_GOOD_CAT_ID)));
        String name = c.getString(c.getColumnIndex(COLUMN_NAME));
        int weight = c.getInt(c.getColumnIndex(COLUMN_WEIGHT));
        int amount = c.getInt(c.getColumnIndex(COLUMN_TOT_AMOUNT));
        int scale = c.getInt(c.getColumnIndex(COLUMN_TOT_SCALE));
        int order = c.getInt(c.getColumnIndex(COLUMN_ORDER));
        return new CollectedGoodEntity(id, visitId, goodId, name, weight, amount, scale, order);
    }


    public List<CollectedGoodEntity> selectCollectedGoodsForVisit(LongId<VisitEntity> visitId) {
        String where = String.format("%s = %d", COLUMN_VISIT_ID, visitId.getValue());
        List<CollectedGoodEntity> data = selectEntriesForWhereAndSort(where, COLUMN_ORDER+" ASC", 0);
        return data;
    }

    public void clear(LongId<VisitEntity> visitId) {
        String where = String.format("%s = %d", COLUMN_VISIT_ID, visitId.getValue());
        deleteEntryByWhere(where);

    }
}
