package com.webssa.pda.sunmi.storage.dao.impl;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.EWCCodeEntity;
import com.webssa.pda.sunmi.storage.dao.BaseDAO;

public class EWCCodeDAO extends BaseDAO<EWCCodeEntity, EWCCodeEntity> {

    public static final String TABLE_NAME = "EWCCodes";

    public static final String COLUMN_ID = "_id";
    public static final String COLUMN_EWC_CODE = "ewc_code";
    public static final String COLUMN_EWC_TITLE = "ewc_title";

    public static final String[] SELECTION_COLUMNS = {COLUMN_ID, COLUMN_EWC_CODE, COLUMN_EWC_TITLE};

    private static final String[] CREATIONS = new String[]{
            String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY, %s TEXT, %s TEXT);",
                    TABLE_NAME,
                    COLUMN_ID,
                    COLUMN_EWC_CODE,
                    COLUMN_EWC_TITLE)
    };

    private static final String[] DELETIONS = new String[] {
            String.format("DROP TABLE IF EXISTS %s", TABLE_NAME)
    };

    public static void createTables(SQLiteDatabase db) {
        for (String cr : CREATIONS) db.execSQL(cr);
    }

    public static void dropTables(SQLiteDatabase db) {
        for (String del : DELETIONS) db.execSQL(del);
    }

    public EWCCodeDAO() {
        super();
    }

    @Override
    protected String getTableName() {
        return TABLE_NAME;
    }

    @Override
    protected String[] getSelectionColumns() {
        return SELECTION_COLUMNS;
    }

    @Override
    protected String getIdColumnName() {
        return COLUMN_ID;
    }

    @Override
    public LongId<EWCCodeEntity> createOrUpdateInLocalDB(EWCCodeEntity entry) {
        ContentValues vvv = new ContentValues();
        vvv.put(COLUMN_EWC_CODE, entry.getCode());
        vvv.put(COLUMN_EWC_TITLE, entry.getTitle());

        String where = COLUMN_ID + " = " + entry.id().getValue();
        if (getWriteableDatabase().update(TABLE_NAME, vvv, where, null) == 0) {
            vvv.put(COLUMN_ID, entry.id().getValue());
            long id = getWriteableDatabase().insert(TABLE_NAME, null, vvv);
            return (id > 0) ? LongId.Companion.valueOf(id) : null;
        } else {
            return entry.id();
        }
    }

    @Override
    public int deleteEntry(EWCCodeEntity entry) {
        return super.deleteEntryById(entry.id());
    }

    @Override
    public EWCCodeEntity buildDbModel(Cursor c) {
        LongId<EWCCodeEntity> id = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_ID)));
        return new EWCCodeEntity.Builder()
                .setId(id)
                .setCode(c.getString(c.getColumnIndex(COLUMN_EWC_CODE)))
                .setTitle(c.getString(c.getColumnIndex(COLUMN_EWC_TITLE)))
                .build();
    }
}
