package com.webssa.pda.sunmi.storage.dao.impl;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.text.TextUtils;

import com.webssa.library.attachment.model.CoverImportModel;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dao.BaseDAO;
import com.webssa.pda.sunmi.storage.dbmodel.impl.LocalNoteDbModel;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * Created by stanislav.perchenko on 11/5/2019, 8:10 PM.
 */
public class LocalNoteDAO extends BaseDAO<LocalNoteDbModel, LocalNoteDbModel> {

    public static final String TABLE_NAME = "Localnotes";

    public static final String COLUMN_ID = "_id";
    public static final String COLUMN_SHIFT_ID = "shift_id";
    public static final String COLUMN_VISIT_ID = "visit_id";
    public static final String COLUMN_CREATED_AT = "created_at";
    public static final String COLUMN_TEXT = "text";
    public static final String COLUMN_EMERG_ID = "emerg_id";
    public static final String COLUMN_IMAGES = "images";


    public static final String[] SELECTION_COLUMNS = {COLUMN_ID, COLUMN_SHIFT_ID, COLUMN_VISIT_ID, COLUMN_CREATED_AT, COLUMN_TEXT, COLUMN_EMERG_ID, COLUMN_IMAGES};

    private static final String[] CREATIONS = new String[]{
            String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY, %s INTEGER, %s INTEGER, %s INTEGER, %s TEXT, %s INTEGER, %s TEXT);",
                    TABLE_NAME,
                    COLUMN_ID,
                    COLUMN_SHIFT_ID,
                    COLUMN_VISIT_ID,
                    COLUMN_CREATED_AT,
                    COLUMN_TEXT,
                    COLUMN_EMERG_ID,
                    COLUMN_IMAGES)
    };

    private static final String[] DELETIONS = new String[]{
            String.format("DROP TABLE IF EXISTS %s", TABLE_NAME)
    };

    public static void createTables(SQLiteDatabase db) {
        for (String cr : CREATIONS) db.execSQL(cr);
    }

    public static void dropTables(SQLiteDatabase db) {
        for (String del : DELETIONS) db.execSQL(del);
    }

    public LocalNoteDAO() {
        super();
    }

    @Override
    protected String getTableName() {
        return TABLE_NAME;
    }

    @Override
    protected String[] getSelectionColumns() {
        return SELECTION_COLUMNS;
    }

    @Override
    protected String getIdColumnName() {
        return COLUMN_ID;
    }

    @Override
    public LongId<LocalNoteDbModel> createOrUpdateInLocalDB(LocalNoteDbModel entry) {
        ContentValues vvv = new ContentValues();
        vvv.put(COLUMN_SHIFT_ID, entry.shiftId().getValue());
        vvv.put(COLUMN_VISIT_ID, (entry.optVisitId() == null) ? 0 : entry.optVisitId().getValue());
        vvv.put(COLUMN_CREATED_AT, entry.createdAt().getTime());
        vvv.put(COLUMN_TEXT, entry.noteText());
        vvv.put(COLUMN_IMAGES, getImageJson(entry.attachedImages()));
        if (entry.optEmergencyId() != null) vvv.put(COLUMN_EMERG_ID, entry.optEmergencyId().getValue());

        if (entry.id() != null) {
            String where = COLUMN_ID + " = " + entry.id().getValue();
            int n_upd = getWriteableDatabase().update(TABLE_NAME, vvv, where, null);
            if (n_upd > 0) {
                return entry.id();
            } else {
                vvv.put(COLUMN_ID, entry.id().getValue());
            }
        }
        long id = getWriteableDatabase().insert(TABLE_NAME, null, vvv);
        return (id > 0) ? LongId.Companion.valueOf(id) : null;
    }

    private String getImageJson(List<CoverImportModel> images) {
        StringBuilder sb = new StringBuilder();
        for (CoverImportModel img : images) {
            if (sb.length() > 0) sb.append(';');
            sb.append(img.toJson());
        }
        return sb.toString();
    }

    @Override
    public int deleteEntry(LocalNoteDbModel entry) {
        return super.deleteEntryById(entry.id());
    }

    @Override
    public LocalNoteDbModel buildDbModel(Cursor c) {
        LongId<LocalNoteDbModel> id = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_ID)));
        LongId<ShiftRestEntity> shiftId = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_SHIFT_ID)));
        LongId<VisitEntity> optVisitId = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_VISIT_ID)));
        LongId<EmergencyCategoryEntity> optEmergencyId = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_EMERG_ID)));

        List<CoverImportModel> images = new ArrayList<>();
        String siii = c.getString(c.getColumnIndex(COLUMN_IMAGES));
        String[] sss = siii.split(";");
        if (sss.length > 0) {
            for (int i = 0; i < sss.length; i++) {
                if (!TextUtils.isEmpty(sss[i])) {
                    images.add(CoverImportModel.fromJson(sss[i]));
                }
            }
        } else {
            images = Arrays.asList();
        }

        return LocalNoteDbModel.builder()
                .setId(id)
                .setShiftId(shiftId)
                .setOptVisitId(optVisitId)
                .setCreatedAt(new Date(c.getLong(c.getColumnIndex(COLUMN_CREATED_AT))))
                .setNoteText(c.getString(c.getColumnIndex(COLUMN_TEXT)))
                .setOptEmergencyId(optEmergencyId)
                .setAttachedImages(images)
                .build();
    }

    public List<LocalNoteDbModel> selectNotesForShiftOnly(LongId<ShiftRestEntity> shiftId) {
        String where = String.format("%s = %d AND %s < 1", COLUMN_SHIFT_ID, shiftId.getValue(), COLUMN_VISIT_ID);
        return selectEntriesForWhereAndSort(where, COLUMN_CREATED_AT + " ASC", 0);
    }
}
