package com.webssa.pda.sunmi.storage.dao.impl;

import android.annotation.SuppressLint;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.storage.dao.BaseDAO;
import com.webssa.pda.sunmi.util.TimeOfDayHMS;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by stanislav.perchenko on 10/21/2019.
 */
public class SitesDAO extends BaseDAO<SiteEntity, SiteEntity> {
    public static final String TABLE_NAME = "Sites";

    public static final String COLUMN_ID = "_id";
    public static final String COLUMN_NAME = "name";
    public static final String COLUMN_CODE = "code";
    public static final String COLUMN_CODE_TEXT = "code_text";
    public static final String COLUMN_ADDRESS_1 = "address_1";
    public static final String COLUMN_ADDRESS_2 = "address_2";
    public static final String COLUMN_TOWN = "town";
    public static final String COLUMN_CITY = "city";
    public static final String COLUMN_ZIP = "zip";
    public static final String COLUMN_LAT = "lat";
    public static final String COLUMN_LON = "lom";
    public static final String COLUMN_UUID = "uuid";
    public static final String COLUMN_PAY_METHOD = "pay_method";

    public static final String[] SELECTION_COLUMNS = new String[]{COLUMN_ID, COLUMN_NAME, COLUMN_CODE, COLUMN_CODE_TEXT, COLUMN_ADDRESS_1, COLUMN_ADDRESS_2, COLUMN_TOWN, COLUMN_CITY, COLUMN_ZIP, COLUMN_LAT, COLUMN_LON, COLUMN_UUID, COLUMN_PAY_METHOD};

    private static final String[] CREATIONS = new String[]{
            String.format("CREATE TABLE %s (%s INTEGER PRIMARY KEY, %s TEXT, %s INTEGER, %s TEXT, %s TEXT, %s TEXT, %s TEXT, %s TEXT, %s TEXT, %s REAL, %s REAL, %s TEXT, %s TEXT);",
                    TABLE_NAME,
                    COLUMN_ID,
                    COLUMN_NAME,
                    COLUMN_CODE,
                    COLUMN_CODE_TEXT,
                    COLUMN_ADDRESS_1, COLUMN_ADDRESS_2, COLUMN_TOWN, COLUMN_CITY, COLUMN_ZIP,
                    COLUMN_LAT,
                    COLUMN_LON,
                    COLUMN_UUID,
                    COLUMN_PAY_METHOD)
    };

    private static final String[] DELETIONS = new String[] {
            String.format("DROP TABLE IF EXISTS %s", TABLE_NAME)
    };

    public static void createTables(SQLiteDatabase db) {
        for (String cr : CREATIONS) db.execSQL(cr);
    }

    public static void dropTables(SQLiteDatabase db) {
        for (String del : DELETIONS) db.execSQL(del);
    }

    public SitesDAO() {
        super();
    }

    @Override
    protected String getTableName() {
        return TABLE_NAME;
    }

    @Override
    protected String[] getSelectionColumns() {
        return SELECTION_COLUMNS;
    }

    @Override
    protected String getIdColumnName() {
        return COLUMN_ID;
    }

    @Override
    public LongId<SiteEntity> createOrUpdateInLocalDB(SiteEntity entry) {
        ContentValues vvv = new ContentValues();

        vvv.put(COLUMN_NAME, entry.getSiteName());
        vvv.put(COLUMN_CODE, entry.getCode());
        vvv.put(COLUMN_CODE_TEXT, entry.getShopCodeText());
        vvv.put(COLUMN_ADDRESS_1, entry.getAddressLine1());
        vvv.put(COLUMN_ADDRESS_2, entry.getAddressLine2());
        vvv.put(COLUMN_TOWN, entry.getTown());
        vvv.put(COLUMN_CITY, entry.getCity());
        vvv.put(COLUMN_ZIP, entry.getPostCode());
        vvv.put(COLUMN_LAT, entry.getLatitude());
        vvv.put(COLUMN_LON, entry.getLongitude());
        vvv.put(COLUMN_UUID, entry.getUuid());
        vvv.put(COLUMN_PAY_METHOD, entry.getPaymentMethod());

        String where = COLUMN_ID + " = " + entry.id().getValue();
        if (getWriteableDatabase().update(TABLE_NAME, vvv, where, null) == 0) {
            vvv.put(COLUMN_ID, entry.id().getValue());
            long id = getWriteableDatabase().insert(TABLE_NAME, null, vvv);
            return (id > 0) ? LongId.Companion.valueOf(id) : null;
        } else {
            return entry.id();
        }
    }

    @Override
    public int deleteEntry(SiteEntity entry) {
        return super.deleteEntryById(entry.id());
    }

    @Override
    public SiteEntity buildDbModel(Cursor c) {
        LongId<SiteEntity> id = LongId.Companion.valueOf(c.getLong(c.getColumnIndex(COLUMN_ID)));
        return new SiteEntity.Builder()
                .setId(id)
                .setShopName(c.getString(c.getColumnIndex(COLUMN_NAME)))
                .setShopCode(c.getInt(c.getColumnIndex(COLUMN_CODE)))
                .setShopCodeText(c.getString(c.getColumnIndex(COLUMN_CODE_TEXT)))
                .setAddressLine1(c.getString(c.getColumnIndex(COLUMN_ADDRESS_1)))
                .setAddressLine2(c.getString(c.getColumnIndex(COLUMN_ADDRESS_2)))
                .setTown(c.getString(c.getColumnIndex(COLUMN_TOWN)))
                .setCity(c.getString(c.getColumnIndex(COLUMN_CITY)))
                .setPostCode(c.getString(c.getColumnIndex(COLUMN_ZIP)))
                .setLatitude(c.getDouble(c.getColumnIndex(COLUMN_LAT)))
                .setLongitude(c.getDouble(c.getColumnIndex(COLUMN_LON)))
                .setUuid(c.getString(c.getColumnIndex(COLUMN_UUID)))
                .setPayMethod(c.getString(c.getColumnIndex(COLUMN_PAY_METHOD)))
                .build();
    }

    public List<SiteEntity> selectSitesNotInShift(LongId<ShiftRestEntity> shiftId) {
        @SuppressLint("DefaultLocale")
        String query = String.format("SELECT * FROM %1$s WHERE (%2$s NOT IN ( SELECT %4$s from %5$s as V WHERE V.%6$s = %7$d )) ORDER BY %3$s ASC",
                TABLE_NAME,
                COLUMN_ID,
                COLUMN_NAME,
                VisitDAO.COLUMN_SITE_ID,
                VisitDAO.TABLE_NAME,
                VisitDAO.COLUMN_SHIFT_ID,
                shiftId.getValue());
        Cursor c = getWriteableDatabase().rawQuery(query, new String[]{});
        List<SiteEntity> result = new ArrayList<>();
        if (c.moveToFirst()) {
            do {
                SiteEntity site = buildDbModel(c);
                result.add(site);
            } while (c.moveToNext());
        }
        return result;
    }
}
