package com.webssa.pda.sunmi.storage.dbhelper;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;


import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.storage.dao.impl.CollectedGoodDAO;
import com.webssa.pda.sunmi.storage.dao.impl.EWCCodeDAO;
import com.webssa.pda.sunmi.storage.dao.impl.LocalNoteDAO;
import com.webssa.pda.sunmi.storage.dao.impl.NoteDAO;
import com.webssa.pda.sunmi.storage.dao.impl.PricedGoodDAO;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dao.impl.SitesDAO;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by stanislav.perchenko on 10/25/2016.
 */
public class DatabaseManager {
    public static final String DB_LOG_TAG = "ShaRecoveryDB";

    private static DatabaseManager manager;

    public static synchronized void initializeInstance(Context context) {
        if (manager == null) {
            manager = new DatabaseManager(context);
        }
    }

    public static synchronized DatabaseManager getInstance() {
        if (manager == null) {
            throw new IllegalStateException(DatabaseManager.class.getSimpleName() + " is not initialized, call initializeInstance(...) method first.");
        }
        return manager;
    }


    private AtomicInteger openCounter = new AtomicInteger();
    private DbOpenHelper dbOpenHelper;
    private SQLiteDatabase writeableDatabase;


    private DatabaseManager(Context context) {
        dbOpenHelper = new DbOpenHelper(context);
    }


    public synchronized SQLiteDatabase openDatabase() {
        if(openCounter.incrementAndGet() == 1) {
            // Opening new database
            writeableDatabase = dbOpenHelper.getWritableDatabase();
        }
        return writeableDatabase;
    }

    public synchronized void closeDatabase() {
        if(openCounter.decrementAndGet() == 0) {
            // Closing database
            writeableDatabase.close();

        }
    }

    public synchronized void reCreateDatabase() {
        SQLiteDatabase db = openDatabase();
        try {
            dbOpenHelper.onUpgrade(db, db.getVersion(), db.getVersion());
        } finally {
            closeDatabase();
        }
    }










    /**********************************************************************************************/
    /************************   The SQLiteOpenHelper implementation   *****************************/
    /**********************************************************************************************/

    private static class DbOpenHelper extends SQLiteOpenHelper {

        public DbOpenHelper(Context context) {
            super(context, GlobalProperties.DB_NAME, null, GlobalProperties.DB_VERSION);
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
            db.beginTransaction();
            try {
                PricedGoodDAO.createTables(db);
                SitesDAO.createTables(db);
                VisitDAO.createTables(db);
                ShiftDAO.createTables(db);
                LocalNoteDAO.createTables(db);
                NoteDAO.createTables(db);
                EWCCodeDAO.createTables(db);
                CollectedGoodDAO.createTables(db);
                /*UserDAO.createTables(db);
                ImageDAO.createTables(db);
                ConfigDAO.createTables(db);
                InterestDAO.createTables(db);
                InterestLevelDAO.createTables(db);
                EventDAO.createTables(db);
                PostDAO.createTables(db);
                AttachmentDAO.createTables(db);
                DeletedEntitiesDBAdapter.createTables(db);
                FeedDAO.createTables(db);
                LocalPostCommentDAO.createTables(db);
                GenderDAO.createTables(db);
                AddictionDAO.createTables(db);
                CategoryDAO.createTables(db);
                PushDAO.createTables(db);
                ToLoadEntityDbAdapter.createTables(db);
                ArticleDAO.createTables(db);
                UserIconsDBAdapter.createTables(db);
                UnreadChatMessageDbAdapter.createTables(db);
                ChallengeDAO.createTables(db);
                CairyProgramStepDAO.createTables(db);*/

                db.setTransactionSuccessful();
            } finally {
                db.endTransaction();
            }

        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int i, int i1) {
            db.beginTransaction();
            try {
                PricedGoodDAO.dropTables(db);
                PricedGoodDAO.createTables(db);
                SitesDAO.dropTables(db);
                SitesDAO.createTables(db);
                VisitDAO.dropTables(db);
                VisitDAO.createTables(db);
                ShiftDAO.dropTables(db);
                ShiftDAO.createTables(db);
                LocalNoteDAO.dropTables(db);
                LocalNoteDAO.createTables(db);
                NoteDAO.dropTables(db);
                NoteDAO.createTables(db);
                EWCCodeDAO.dropTables(db);
                EWCCodeDAO.createTables(db);
                CollectedGoodDAO.dropTables(db);
                CollectedGoodDAO.createTables(db);
                /*ImageDAO.dropTables(db);
                ImageDAO.createTables(db);
                UserDAO.dropTables(db);
                UserDAO.createTables(db);
                ConfigDAO.dropTables(db);
                ConfigDAO.createTables(db);
                InterestDAO.dropTables(db);
                InterestDAO.createTables(db);
                InterestLevelDAO.dropTables(db);
                InterestLevelDAO.createTables(db);
                EventDAO.dropTables(db);
                EventDAO.createTables(db);
                PostDAO.dropTables(db);
                PostDAO.createTables(db);
                AttachmentDAO.dropTables(db);
                AttachmentDAO.createTables(db);
                DeletedEntitiesDBAdapter.dropTables(db);
                DeletedEntitiesDBAdapter.createTables(db);
                FeedDAO.dropTables(db);
                FeedDAO.createTables(db);
                LocalPostCommentDAO.dropTables(db);
                LocalPostCommentDAO.createTables(db);
                GenderDAO.dropTables(db);
                GenderDAO.createTables(db);
                AddictionDAO.dropTables(db);
                AddictionDAO.createTables(db);
                CategoryDAO.dropTables(db);
                CategoryDAO.createTables(db);
                PushDAO.dropTables(db);
                PushDAO.createTables(db);
                ToLoadEntityDbAdapter.dropTables(db);
                ToLoadEntityDbAdapter.createTables(db);
                ArticleDAO.dropTables(db);
                ArticleDAO.createTables(db);
                UserIconsDBAdapter.dropTables(db);
                UserIconsDBAdapter.createTables(db);
                UnreadChatMessageDbAdapter.dropTables(db);
                UnreadChatMessageDbAdapter.createTables(db);
                ChallengeDAO.dropTables(db);
                ChallengeDAO.createTables(db);
                CairyProgramStepDAO.dropTables(db);
                CairyProgramStepDAO.createTables(db);*/

                db.setTransactionSuccessful();
            } finally {
                db.endTransaction();
            }
        }
    }
}
