package com.webssa.pda.sunmi.storage.dbhelper;

import android.util.Log;

import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.rest.model.IdProvidingModel;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.EWCCodeEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.util.StringUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by stanislav.perchenko on 10/18/2019.
 */
public class DbSelectionCache {
    private static final boolean D = GlobalProperties.D;
    private static final String TAG = "DbSelectionCache";

    Map<String, Map<Long, ? extends IdProvidingModel>> cache;
    private int nTry, nHit;

    public DbSelectionCache() {
        cache = new HashMap<>();
        if (D) Log.i(TAG, "=========================   DB cache created   =============================");
    }

    public void printCacheUsingStatistics() {
        float efficiency = nHit*100f / nTry;
        if (D) Log.i(TAG, String.format("========  Cache using statistics:  nTry=%d, nHit=%d, efficiency=%.2f%%  ===========", nTry, nHit, efficiency));
        if (D) Log.e(TAG, "~~~~~~~~~~~~~~~~~~~~~~~~~  Cache closed  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
    }

    private Map<Long, ? extends IdProvidingModel> getTypedCache(String classSimpleName) {
        String name = fixAutoValueClassName(classSimpleName);

        Map<Long, ? extends IdProvidingModel> typedCache = cache.get(name);
        if (typedCache == null) {
            typedCache = new HashMap<>();
            cache.put(name, typedCache);
        }
        return typedCache;
    }

    private static final String AUTO_VALUE_PREFIX = "AutoValue_";
    private String fixAutoValueClassName(String inName) {
        return (inName.contains(AUTO_VALUE_PREFIX)) ? StringUtils.removeFirstSubstring(inName, AUTO_VALUE_PREFIX) : inName;
    }

    public <MOD extends IdProvidingModel> void putDbModel(MOD model) {
        if (model.id() != null) {
            if (D) Log.d(TAG, String.format("Catch instance: type=%s, id=%d", model.getClass().getSimpleName(), model.id().getValue()));
            Map<Long, MOD> typedCache = (Map<Long, MOD>) getTypedCache(model.getClass().getName());
            typedCache.put(model.id().getValue(), model);
        }
    }

    public SiteEntity getSite(LongId<SiteEntity> siteId) {
        return (SiteEntity) updateStatistics(getTypedCache(SiteEntity.class.getName()).get(siteId.getValue()), SiteEntity.class, siteId);
    }

    public EWCCodeEntity getEwcCodeEntity(LongId<EWCCodeEntity> ewcId) {
        return (EWCCodeEntity) updateStatistics(getTypedCache(EWCCodeEntity.class.getName()).get(ewcId.getValue()), EWCCodeEntity.class, ewcId);
    }

    private <MOD extends IdProvidingModel> MOD updateStatistics(MOD model, Class<? extends IdProvidingModel> cls, LongId<?> itemId) {
        nTry ++;
        if (model != null) {
            nHit ++;
            if (D) Log.e(TAG, String.format("Cache HIT: type=%s, id=%s", cls.getSimpleName(), itemId));
        } else {
            if (D) Log.e(TAG, String.format("Cache missing: type=%s, id=%s", cls.getSimpleName(), itemId));
        }
        return model;
    }

}
