package com.webssa.pda.sunmi.storage.dbmodel.impl;

import android.os.Parcelable;

import androidx.annotation.Nullable;

import com.google.auto.value.AutoValue;
import com.webssa.pda.sunmi.rest.model.IdProvidingModel;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.ShiftState;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.storage.dbmodel.ScheduledModel;
import com.webssa.pda.sunmi.util.DateNoTime;
import com.webssa.pda.sunmi.util.DurationHMS;
import com.webssa.pda.sunmi.util.TimeOfDayHMS;

import java.util.Date;
import java.util.List;

/**
 * Created by stanislav.perchenko on 10/17/2019.
 */
@AutoValue
public abstract class ShiftDbModel implements IdProvidingModel, ScheduledModel, Parcelable {

    public abstract LongId<ShiftRestEntity> id();

    public abstract String driverId();

    public abstract ShiftState state();

    public abstract DateNoTime executionDate();

    @Nullable
    public abstract TimeOfDayHMS scheduledTimeStart();

    @Nullable
    public abstract DurationHMS scheduledDuration();

    public abstract int nTotalVisitsBackend();

    public abstract int nCompletedVisits();

    public abstract int nFailedVisits();

    @Nullable
    public abstract Date actualTimeStart();

    @Nullable
    public abstract Date actualTimeClosed();

    public abstract List<VisitDbModel> visits();

    public abstract List<NoteDbModel> remoteNotes();

    public abstract List<LocalNoteDbModel> localNotes();

    public static Builder builder() {
        return new AutoValue_ShiftDbModel.Builder();
    }

    @AutoValue.Builder
    public abstract static class Builder {
        public abstract Builder setId(LongId<ShiftRestEntity> id);
        public abstract Builder setDriverId(String driverId);
        public abstract Builder setState(ShiftState state);
        public abstract Builder setExecutionDate(DateNoTime executionDate);
        public abstract Builder setScheduledTimeStart(@Nullable TimeOfDayHMS scheduledTimeStart);
        public abstract Builder setScheduledDuration(@Nullable DurationHMS scheduledDuration);
        public abstract Builder setNTotalVisitsBackend(int nTotalVisitsBackend);
        public abstract Builder setNCompletedVisits(int nCompletedVisits);
        public abstract Builder setNFailedVisits(int nFailedVisits);
        public abstract Builder setActualTimeStart(@Nullable Date actualTimeStart);
        public abstract Builder setActualTimeClosed(@Nullable Date actualTimeClosed);
        public abstract Builder setVisits(List<VisitDbModel> visits);
        public abstract Builder setRemoteNotes(List<NoteDbModel> remoteNotes);
        public abstract Builder setLocalNotes(List<LocalNoteDbModel> localNotes);

        public abstract ShiftDbModel build();
    }

    private Date sDate;

    @Override
    public Date scheduledDate() {
        if (sDate == null) {
            synchronized (this) {
                if (sDate == null) {
                    sDate = executionDate().getDate();
                }
            }
        }
        return sDate;
    }

    @Override
    public int scheduledTimeMillis() {
        return 0;
    }
}
