package com.webssa.pda.sunmi.storage.dbmodel.impl;

import android.os.Parcelable;

import androidx.annotation.Nullable;

import com.google.auto.value.AutoValue;
import com.webssa.pda.sunmi.rest.model.IdProvidingModel;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.rest.model.impl.CollectedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dbmodel.ScheduledModel;
import com.webssa.pda.sunmi.util.TimeOfDayHMS;

import java.util.Date;
import java.util.List;

/**
 * Created by stanislav.perchenko on 10/17/2019.
 */
@AutoValue
public abstract class VisitDbModel implements IdProvidingModel, ScheduledModel, Parcelable {
    public abstract LongId<VisitEntity> id();

    public abstract VisitState state();

    public abstract LongId<ShiftRestEntity> shiftId();

    public abstract SiteEntity site();

    public abstract int order();

    @Nullable
    public abstract TimeOfDayHMS suggestedTime();

    @Nullable
    public abstract Date actualStartTime();

    @Nullable
    public abstract Date actualCompleteTime();

    @Nullable
    public abstract String customerName();

    @Nullable
    public abstract String customerSignature();

    @Nullable
    public abstract Integer totalPayedAmount();

    @Nullable
    public abstract Integer totalPayedScale();

    @Nullable
    public abstract NoteDbModel optNote();

    @Nullable
    public abstract Boolean whiteSacks();
    @Nullable
    public abstract Integer clearBags();

    public abstract boolean isSynced();

    public abstract boolean isAdHocVisit();

    public abstract boolean isRemoteCreated();


    public abstract List<CollectedGoodEntity> collectedGoods();

    public VisitDbModel withId(LongId<VisitEntity> id) {
        return toBuilder().setId(id).build();
    }

    abstract Builder toBuilder();

    public static Builder builder() {
        return new AutoValue_VisitDbModel.Builder();
    }

    @AutoValue.Builder
    public abstract static class Builder {
        public abstract Builder setId(LongId<VisitEntity> id);
        public abstract Builder setState(VisitState state);
        public abstract Builder setShiftId(LongId<ShiftRestEntity> shiftId);
        public abstract Builder setSite(SiteEntity site);
        public abstract Builder setOrder(int order);
        public abstract Builder setSuggestedTime(@Nullable TimeOfDayHMS suggestedTime);
        public abstract Builder setActualStartTime(@Nullable Date actualStartTime);
        public abstract Builder setActualCompleteTime(@Nullable Date actualCompleteTime);
        public abstract Builder setCustomerName(@Nullable String customerName);
        public abstract Builder setCustomerSignature(@Nullable String customerSignature);
        public abstract Builder setTotalPayedAmount(@Nullable Integer totalPayedAmount);
        public abstract Builder setTotalPayedScale(@Nullable Integer totalPayedScale);
        public abstract Builder setOptNote(NoteDbModel optNote);
        public abstract Builder setWhiteSacks(@Nullable Boolean whiteSacks);
        public abstract Builder setClearBags(@Nullable Integer clearBags);
        public abstract Builder setIsSynced(boolean isSynced);
        public abstract Builder setIsAdHocVisit(boolean isAdHocVisit);
        public abstract Builder setIsRemoteCreated(boolean isRemoteCreated);


        public abstract Builder setCollectedGoods(List<CollectedGoodEntity> collectedGoods);

        public abstract VisitDbModel build();
    }

    @Override
    public Date scheduledDate() {
        return null;
    }

    private Integer timeMillis;

    @Override
    public int scheduledTimeMillis() {
        if (timeMillis == null) {
            synchronized (this) {
                if (timeMillis == null) {
                    timeMillis = (suggestedTime() == null) ? 0 : 1000 * (suggestedTime().hours()*3600 + suggestedTime().minutes()*60 + suggestedTime().seconds());
                }
            }
        }
        return timeMillis.intValue();
    }
}
