package com.webssa.pda.sunmi.ui.activity;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ListView;

import androidx.annotation.Nullable;

import com.alperez.widget.picker.AlpNumberWheelPicker;
import com.webssa.library.widget.SlidingViewFlipper;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.model.VisitCreationModel;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dao.impl.SitesDAO;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.ui.adapter.SitesSingleChoiceAdapter;
import com.webssa.pda.sunmi.ui.annotation.Layout;
import com.webssa.pda.sunmi.util.DateNoTime;
import com.webssa.pda.sunmi.util.TimeOfDayHMS;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by stanislav.perchenko on 07.07.2020 at 15:07.
 */
@Layout(R.layout.activity_create_ad_hoc_visit)
public class AdHocVisitCreateActivity extends BaseToolbarActivity {

    public static final String ARG_SHIFT_ID = "shift_id";

    public static final String RESULT_VISIT_CREATION = "visit_creation";

    @Nullable
    @Override
    protected Integer getToolbarResId() {
        return R.id.toolbar;
    }

    @Nullable
    @Override
    protected String getScreenTitle() {
        return getString(R.string.screen_create_ad_hoc_visit);
    }

    @Nullable
    @Override
    protected String getScreenSubTitle() {
        return null;
    }

    private LongId<ShiftRestEntity> argShiftId;
    private Thread loaderThread;

    private SlidingViewFlipper vFlipper;
    private View vActionNext;
    private View vActionNextOverlay;
    private Button vActionCreate;
    private AlpNumberWheelPicker vTimePicker;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        argShiftId = getIntent().getParcelableExtra(ARG_SHIFT_ID);
        if (argShiftId == null) throw new RuntimeException("The Shift ID argument must be provided");

        vFlipper = findViewById(R.id.flipper);
        vActionNext = findViewById(R.id.action_next);
        vActionNextOverlay = findViewById(R.id.action_next_overlay);
        vActionCreate = findViewById(R.id.action_create);
        vTimePicker = findViewById(R.id.time_picker);

        vActionNextOverlay.setVisibility(View.VISIBLE);
        vActionNext.setEnabled(false);
        vActionNext.setOnClickListener(v -> vFlipper.showNext(1));
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        loaderThread = new Thread( () -> {
            DbSelectionCache cache = new DbSelectionCache();
            try ( SitesDAO sitesDao = new SitesDAO();
                  ShiftDAO shiftDao = new ShiftDAO(cache);
                  VisitDAO visDao = new VisitDAO(cache)) {
                List<SiteEntity> sites = sitesDao.selectSitesNotInShift(argShiftId);
                DateNoTime date = shiftDao.selectExecutionDateForShift(argShiftId);
                TimeOfDayHMS time = TimeOfDayHMS.builder().setHours(8).setMinutes(0).setSeconds(0).build(); //visDao.selectLatestTimeOfVisit(argShiftId);
                runOnUiThread(() -> populateData(sites, date, time));
            }
        }, "initial_data_loader");
        loaderThread.start();
    }

    private void populateData(List<SiteEntity> sites, @Nullable DateNoTime executionDate, @Nullable TimeOfDayHMS timeStart) {
        if (isDestroyed()) return;

        final SitesSingleChoiceAdapter adapter = new SitesSingleChoiceAdapter(this, sites);
        adapter.setOnSelectionChangedListener(site -> {
            vActionNext.setEnabled(true);
            vActionNextOverlay.setVisibility(View.GONE);
        });
        ((ListView) findViewById(R.id.list_sites)).setAdapter(adapter);


        int[] hhmm = new int[2];
        hhmm[0] = timeStart.hours();
        hhmm[1] = timeStart.minutes();
        if (hhmm[1] < 15) hhmm[1] = 15;
        else if (hhmm[1] < 30) hhmm[1] = 30;
        else if (hhmm[1] < 45) hhmm[1] = 45;
        else {
            hhmm[0] = hhmm[0] + 1;
            hhmm[1] = 0;
        }

        final List<TimeOfDayHMS> timeItems = new ArrayList<>(40);
        while(hhmm[0] < 23) {
            incrementHHMM(hhmm);
            TimeOfDayHMS t = TimeOfDayHMS.builder()
                    .setHours(hhmm[0])
                    .setMinutes(hhmm[1])
                    .setSeconds(0)
                    .build();
            timeItems.add(t);
        }
        vTimePicker.setMinValue(0);
        vTimePicker.setMaxValue(timeItems.size() - 1);
        vTimePicker.setDisplayValueFormatter(value -> (value > 0) ? timeItems.get(value).toStringHM() : "   ");
        vTimePicker.setSelection(0);
        vActionCreate.setEnabled(false);
        vTimePicker.setOnValueChangedListener((picker, oldVal, newVal) -> vActionCreate.setEnabled(newVal > 0));
        vActionCreate.setOnClickListener(v -> finishWithResult(adapter.getSelectedSite(), executionDate, timeItems.get(vTimePicker.getCurrentSelection())));
    }

    private void incrementHHMM(int[] hhmm) {
        if (hhmm[1] >= 45) {
            hhmm[0] = hhmm[0] + 1;
            hhmm[1] = 0;
        } else {
            hhmm[1] = hhmm[1] + 15;
        }
    }


    @Override
    public void onBackPressed() {
        if (vFlipper.getDisplayedChild() > 0) {
            vFlipper.showPrevious();
        } else {
            setResult(RESULT_CANCELED);
            super.onBackPressed();
        }
    }

    private void finishWithResult(SiteEntity site, @Nullable DateNoTime executionDate, TimeOfDayHMS suggestTime) {
        VisitCreationModel vcm = new VisitCreationModel(argShiftId, site.id(), executionDate, suggestTime);
        Intent intent = new Intent();
        intent.putExtra(RESULT_VISIT_CREATION, vcm);
        setResult(RESULT_OK, intent);
        finish();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            setResult(RESULT_CANCELED);
            finish();
            return true;
        } else {
            return super.onOptionsItemSelected(item);
        }
    }
}
