package com.webssa.pda.sunmi.ui.activity;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Rect;
import android.os.Bundle;

import com.webssa.library.attachment.crop.CropAvatarImageFragment;
import com.webssa.library.attachment.model.CoverImportModel;
import com.webssa.library.attachment.presenter.AttachAvatarPresenter;
import com.webssa.library.attachment.presenter.AttachAvatarView;
import com.webssa.pda.sunmi.R;

import java.io.File;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;

/**
 * Created by Alex H. on 1/4/2017.
 */

public class AttachAvatarActivity extends AppCompatActivity implements AttachAvatarView, CropAvatarImageFragment.OnCropAvatarImageListener{

    public static final String ARG_ATTACHMENT_LIVE = "attachment_live";

    public static final String RESULT_COVERIMAGE = "coverimage";

    private AttachAvatarPresenter presenter;
    private CropAvatarImageFragment cropFragment;

    private ProgressDialog progressDialog;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_attach_avatar);

        presenter = new AttachAvatarPresenter(this, getAttachmentLiveArgument());
        presenter.initializeView();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String permissions[], @NonNull int[] grantResults) {
        boolean consumed = presenter.onRequestPermissionsResult(requestCode, grantResults);
        if (!consumed) {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        boolean consumed = presenter.onActivityResult(requestCode, resultCode, data);
        if (!consumed) {
            super.onActivityResult(requestCode, resultCode, data);
        }
    }




    /**************************  View interface implementation  ***********************************/

    @Override
    public void close() {
        finish();
    }

    @Override
    public Context getContext() {
        return this;
    }

    @Override
    public Activity getActivity() {
        return this;
    }

    @Override
    public void requestPermissionsForPresenter(@NonNull String[] permissions, int requestCode) {
        ActivityCompat.requestPermissions(this, permissions, requestCode);
    }

    @Override
    public void showProgressWithMessage(String message) {
        if (progressDialog == null) {
            progressDialog = new ProgressDialog(this);
            progressDialog.setIndeterminate(true);
            progressDialog.setCancelable(false);
            progressDialog.setCanceledOnTouchOutside(false);
            progressDialog.setMessage(message);
        } else if (progressDialog.isShowing()) {
            progressDialog.dismiss();
            progressDialog.setMessage(message);
        }
        progressDialog.show();
    }

    @Override
    public void dismissProgressWithMessage() {
        if ((progressDialog != null) && progressDialog.isShowing()) {
            progressDialog.dismiss();
        }
    }

    @Override
    public void onAttachmentImported(File coverImage) {
        if (cropFragment == null) {
            cropFragment = CropAvatarImageFragment.newInstance(coverImage.getAbsolutePath());
            getSupportFragmentManager().beginTransaction().add(R.id.attach_avatar_content, cropFragment, "crop_fragment").commit();
        } else {
            cropFragment.setNewImage(coverImage);
        }
    }

    @Override
    public void finishWithResult(CoverImportModel cover) {
        Intent data = new Intent().putExtra(RESULT_COVERIMAGE, cover);
        setResult(RESULT_OK, data);
        finish();
    }



    /***********************  Cropping fragment interface implementation  *************************/
    @Override
    public void onTryAgain() {
        cropFragment.setNewImage(null);
        presenter.clearAndRecycle();
        presenter.initializeView();
    }

    @Override
    public void onCancel() {
        presenter.clearAndRecycle();
        finish();
    }

    @Override
    public void onImageCropped(File imageFile, Rect selectedRegion) {
        presenter.prepareCoverImageAndFinish(imageFile, selectedRegion);
    }


    private boolean getAttachmentLiveArgument() {
        try {
            return getIntent().getExtras().getBoolean(ARG_ATTACHMENT_LIVE, true);
        } catch (NullPointerException e) {
            return true;
        }
    }
}
