package com.webssa.pda.sunmi.ui.activity;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;

import com.alperez.sunmi.pos.receiptengine.template.PublicUtils;
import com.webssa.library.attachment.model.CoverImportModel;
import com.webssa.library.mypicasso.Callback;
import com.webssa.library.mypicasso.NetworkPolicy;
import com.webssa.library.widget.SignatureView;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.core.EmergencyHolder;
import com.webssa.pda.sunmi.core.LocalCompletedVisitsStorage;
import com.webssa.pda.sunmi.core.SessionHolder;
import com.webssa.pda.sunmi.model.LocalCollectedGoodItem;
import com.webssa.pda.sunmi.model.LocalVisitResult;
import com.webssa.pda.sunmi.model.UntiedLocalNote;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.rest.model.impl.CollectedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.rest.model.impl.PricedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dao.impl.PricedGoodDAO;
import com.webssa.pda.sunmi.storage.dao.impl.VisitDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.NoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.PricedGoodDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.ui.annotation.Layout;
import com.webssa.pda.sunmi.ui.dialog.PrintReceiptDialogFragment;
import com.webssa.pda.sunmi.util.DataUtils;
import com.webssa.pda.sunmi.util.Toaster;
import com.webssa.pda.sunmi.util.ViewUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by stanislav.perchenko on 11/10/2019, 7:51 PM.
 */
@Layout(R.layout.activity_visit_details)
public class CompletedVisitDetailsActivity extends BaseToolbarActivity implements PrintReceiptDialogFragment.OnCompleteListener {
    public static final String ARG_VISIT_ID = "visit_id";
    LongId<VisitEntity> argVisitId;
    private ViewGroup vGroupSite;
    private ViewGroup vGroupVisitTime;
    private ViewGroup vGroupCollectedGoods;
    private ViewGroup vGroupCustomer;
    private ViewGroup vGroupDriverNote;
    private TextView vTxtResultStatus;
    private TextView vTxtCustomerName;
    private SignatureView vCautomerSignature;

    @Nullable
    @Override
    protected Integer getToolbarResId() {
        return R.id.toolbar;
    }

    @Nullable
    @Override
    protected String getScreenTitle() {
        return getString(R.string.screen_title_job_details);
    }

    @Nullable
    @Override
    protected String getScreenSubTitle() {
        return null;
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        argVisitId = getIntent().getParcelableExtra(ARG_VISIT_ID);
        if (argVisitId == null) {
            finish();
            return;
        }
        super.onCreate(savedInstanceState);
        vGroupSite = findViewById(R.id.group_site);
        vGroupVisitTime = findViewById(R.id.group_visit_time);
        vGroupCollectedGoods = findViewById(R.id.group_collected_goods);
        vGroupCustomer = findViewById(R.id.group_customer);
        vGroupDriverNote = findViewById(R.id.group_driver_note);
        vTxtResultStatus = vGroupVisitTime.findViewById(R.id.txt_result_status);
        vTxtCustomerName = findViewById(R.id.txt_customer_name);
        vCautomerSignature = findViewById(R.id.customer_signature);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_visit_details, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        menu.findItem(R.id.menu_item_print_receipt).setVisible((mVisit != null) && (mVisit.state() == VisitState.COMPLETED));
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
            return true;
        } else if (item.getItemId() == R.id.menu_item_print_receipt) {
            if (mVisit != null && (SessionHolder.getSession() != null) && (SessionHolder.getSession().getUser() != null)) {

                if (mVisit.isSynced()) {
                    List<LocalCollectedGoodItem> localGoods = new ArrayList<>(mVisit.collectedGoods().size());
                    for (CollectedGoodEntity cGood : mVisit.collectedGoods())
                        localGoods.add(new LocalCollectedGoodItem(mVisit.site().id(), cGood.getGoodcategoryId(), cGood.getCollectedWeight()));
                    LocalVisitResult visitResult = LocalVisitResult.builder()
                            .setVisitId(mVisit.id())
                            .setCollectedGoods(localGoods)
                            .setDriverName(SessionHolder.getSession().getUser().getFullName())
                            .setCustomerName(mVisit.customerName())
                            .setCustomerSignature(mVisit.customerSignature())
                            .setWhiteSacks(mVisit.whiteSacks())
                            .setClearBags(mVisit.clearBags())
                            .setOptDriverNote(null)
                            .build();
                    PrintReceiptDialogFragment.newInstance(visitResult).show(getSupportFragmentManager(), "print-dialog");
                } else {
                    LocalVisitResult localVisit = LocalCompletedVisitsStorage.getLocalVisitResult(mVisit.id());
                    if (localVisit != null) {
                        List<LocalCollectedGoodItem> localGoods = localVisit.collectedGoods();
                        LocalVisitResult visitResult = LocalVisitResult.builder()
                                .setVisitId(mVisit.id())
                                .setCollectedGoods(localGoods)
                                .setDriverName(SessionHolder.getSession().getUser().getFullName())
                                .setCustomerName(localVisit.customerName())
                                .setCustomerSignature(localVisit.customerSignature())
                                .setWhiteSacks(localVisit.whiteSacks())
                                .setClearBags(localVisit.clearBags())
                                .setOptDriverNote(null)
                                .build();
                        PrintReceiptDialogFragment.newInstance(visitResult).show(getSupportFragmentManager(), "print-dialog");
                    }
                }
            }
            return true;
        } else {
            return super.onOptionsItemSelected(item);
        }
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        vGroupSite.setVisibility(View.GONE);
        vGroupVisitTime.setVisibility(View.GONE);
        vGroupCollectedGoods.setVisibility(View.GONE);
        vGroupCustomer.setVisibility(View.GONE);
        vGroupDriverNote.setVisibility(View.GONE);
        new Thread(() -> {
            try (VisitDAO dao = new VisitDAO(new DbSelectionCache())) {
                VisitDbModel visit = dao.selectEntryById(argVisitId);
                if (visit.state() == VisitState.NEW) {
                    List<String> images = Arrays.asList("https://www.thesun.co.uk/wp-content/uploads/2019/09/NINTCHDBPICT000520597899.jpg?w=160", "https://www.thesun.co.uk/wp-content/uploads/2019/09/NINTCHDBPICT000520598320.jpg?w=160", "https://www.monsterenergy.com/media/uploads_image/2017/10/23/1600/800/53df2c3ebdc07979fe4c2db44d9837bc.jpg?mod=e68f19bee4addd346e1da15eee27852b");
                    NoteDbModel note = NoteDbModel.builder()
                            .setId(LongId.Companion.valueOf(549))
                            .setShiftId(visit.shiftId())
                            .setOptVisitId(visit.id())
                            .setSequenceNumber(1)
                            .setAuthorId(SessionHolder.getSession().getUser().getUserId())
                            .setCreatedAt(new Date())
                            .setNoteText("UIsdfhsd fhsdfoasdhfo aishfaio hao sdfhaiopdfhaipdh fpiasdhf iahsf paidhsf piadhfipahfi phaipf haiofh asugfu oasgfuoagd fuoagsfo adi gfioa dhfpaidhfp hadpf dghg")
                            .setOptEmergencyId(LongId.Companion.valueOf(2))
                            .setAttachedImages(images)
                            .build();
                    List<CollectedGoodEntity> collectedGoods = new ArrayList<>();
                    collectedGoods.add(new CollectedGoodEntity(LongId.Companion.valueOf(1001), visit.id(), LongId.Companion.valueOf(1), "Books", 23, 59041, 3, 1));
                    collectedGoods.add(new CollectedGoodEntity(LongId.Companion.valueOf(1002), visit.id(), LongId.Companion.valueOf(7), "Electronics", 8, 38400, 2, 2));
                    collectedGoods.add(new CollectedGoodEntity(LongId.Companion.valueOf(1003), visit.id(), LongId.Companion.valueOf(6), "Household Items", 73, 39055, 2, 3));
                    final VisitDbModel newVisit = VisitDbModel.builder()
                            .setId(visit.id())
                            .setState(VisitState.COMPLETED)
                            .setShiftId(visit.shiftId())
                            .setSite(visit.site())
                            .setOrder(visit.order())
                            .setSuggestedTime(visit.suggestedTime())
                            .setActualStartTime(new Date()).setActualCompleteTime(new Date(System.currentTimeMillis() + 35 * 60 * 1000))
                            .setCustomerName("Salli May")
                            .setCustomerSignature("840;480+3488,2208;3392,2250;3333,2270;3254,2356;3154,2479;3086,2581;2988,2729;2929,2849;2845,3000;2786,3137;2702,3312;2653,3431;2595,3604;2545,3743;2476,3937;2436,4092;2369,4312;2328,4471;2273,4687;2234,4857;2190,5104;2152,5255;2095,5479;2066,5615;2011,5812;1982,5948;1952,6125;1932,6210;1904,6354;1885,6457;1869,6625;1859,6728;1845,6875;1835,6977;1833,7104;1833,7189;1833,7312;1833,7395;1843,7447;1845,7500;1869,7520;1880,7562;1904,7583;1940,7604;1969,7604;2035,7604;2102,7587;2202,7562;2290,7528;2416,7458;2505,7388;2630,7291;2729,7205;2869,7083;2966,6998;3095,6854;3204,6749;3345,6625;3451,6523;3476,6500;3607,6375;3714,6255;3738,6229;3869,6083;3966,5963;3988,5937;4107,5791;4195,5670;4321,5500;4399,5379;4500,5208;4578,5070;4678,4875;4745,4757;4833,4562;4902,4424;4988,4229;5047,4091;5107,3937;5145,3835;5202,3708;5231,3640;5273,3541;5293,3474;5309,3395;5318,3346;5333,3291;5321,3250;5311,3208;5268,3225;5226,3250;5167,3284;5071,3354;4994,3438;4880,3562;4792,3699;4773,3729;4666,3895;4568,4066;4547,4104;4440,4291;4352,4461;4333,4500;4226,4708;4157,4862;4047,5104;3977,5262;3892,5458;3844,5610;3773,5812;3735,5947;3678,6125;3639,6261;3607,6437;3588,6554;3571,6708;3561,6810;3547,6958;3547,7044;3547,7166;3547,7218;3559,7270;3559,7322;3571,7395;3581,7447;3595,7520;3614,7572;3630,7625;3660,7659;3702,7708;3741,7725;3797,7750;3855,7733;3964,7708;4080,7640;4107,7625;4261,7520;4405,7403;4440,7375;4619,7208;4764,7055;4797,7020;4952,6812;5086,6628;5119,6583;5273,6375;5387,6208;5416,6166;5547,5958;5633,5807;5654,5770;5761,5604;5848,5469;5869,5437;5964,5270;6049,5154;6071,5125;6178,4979;6254,4862;6273,4833;6369,4708;6444,4608;6547,4458;6615,4356;6702,4229;6749,4146;6809,4020;6847,3937;6892,3812;6911,3746;6940,3645;6959,3580;6988,3479;6997,3428;7011,3375;7011,3312;7002,3270;6978,3287;6952,3312;6913,3329;6857,3395;6798,3464;6714,3583;6629,3698;6607,3729;6523,3875;6446,3992;6345,4187;6268,4338;6250,4375;6166,4541;6099,4675;6011,4895;5953,5066;5869,5291;5829,5445;5773,5645;5735,5779;5690,5958;5661,6076;5630,6229;5611,6331;5583,6479;5573,6562;5559,6687;5559,6770;5559,6838;5559,6916;5559,6982;5559,7062;5569,7129;5571,7208;5580,7275;5583,7354;5592,7404;5607,7479;5619,7562;5642,7625;5666,7645;5690,7666;5714,7687;5738,7708;5767,7708;5809,7729;5847,7729;5916,7729;5973,7712;6071,7666;6148,7633;6250,7583;6336,7532;6452,7458;6529,7390;6654,7270;6731,7186;6845,7062;6932,6944;6952,6916;7047,6791;7133,6657;7154,6625;7250,6458;7346,6289;7369,6250;7464,6062;7541,5911;7559,5875;7654,5687;7740,5504;7761,5458;7857,5250;7943,5098;7964,5062;8071,4854;8168,4683;8190,4645;8285,4437;8392,4250;8416,4208;8523,3979;8611,3826;8630,3791;8726,3625;8794,3522;8880,3375;8929,3307;8976,3250;9004,3216;9035,3187+1642,6437;1642,6270;1652,6113;1678,5916;1719,5718;1785,5479;1844,5273;1857,5229;1940,4979;2029,4771;2047,4729;2154,4500;2245,4306;2261,4270;2380,4062;2480,3887;2500,3854;2619,3666;2718,3527;2857,3354;2943,3236;2964,3208;3071,3104;3160,3017;3285,2937;3374,2903;3488,2875;3566,2857;3678,2875;3718,2875;3785,2895;3853,2929;3964,3000;4041,3067;4142,3166;4221,3270;4321,3395;4409,3516;4523,3687;4591,3806;4702,3979;4778,4078;4880,4208;4957,4291;5059,4395;5136,4463;5250,4520;5327,4554;5440,4562;5507,4562;5619,4541;5745,4490;5773,4479;5916,4395;6071,4276;6107,4250;6273,4125;6417,4007;6452,3979;6619,3854;6751,3721;6785,3687;6952,3520;7067,3403;7095,3375;7238,3208;7335,3054;7357,3020;7476,2875;7534,2773;7630,2625;7699,2505;7785,2375;7843,2274;7928,2145;7995,2045;8071,1916;8110,1865;8154,1791;8172,1760;8154,1791")
                            .setTotalPayedAmount(45232)
                            .setTotalPayedScale(2)
                            .setOptNote(note)
                            .setCollectedGoods(collectedGoods)
                            .setIsSynced(true)
                            .build();
                    visit = newVisit;
                }
                final VisitDbModel passVisit = visit;
                runOnUiThread(() -> onVisitLoaded(passVisit));
            }
        }, "visit-loader").start();
    }

    private void onVisitLoaded(@Nullable VisitDbModel visit) {
        if (isDestroyed()) {
            return;
        } else if (visit == null || (visit.state() == VisitState.NEW) || (visit.state() == VisitState.IN_PROGRESS)) {
            finish();
            return;
        } else {
            mVisit = visit;
            invalidateOptionsMenu();
        }
        vGroupSite.setVisibility(View.VISIBLE);
        SiteEntity site = visit.site();
        final String resolvedShopCode = TextUtils.isEmpty(site.getShopCodeText()) ? (""+site.getCode()) : site.getShopCodeText();

        ((TextView) vGroupSite.findViewById(R.id.txt_site_name)).setText(site.getSiteName());
        ((TextView) vGroupSite.findViewById(R.id.txt_site_code)).setText(resolvedShopCode);
        ((TextView) vGroupSite.findViewById(R.id.txt_site_address)).setText(DataUtils.getDiaplayAddress(site));
        ((TextView) vGroupSite.findViewById(R.id.txt_site_location)).setText(String.format("%.5f, %.5f", site.getLatitude(), site.getLongitude()));
        vGroupVisitTime.setVisibility(View.VISIBLE);

        ((TextView) vGroupVisitTime.findViewById(R.id.txt_visit_suggest_time)).setText((visit.suggestedTime() == null) ? "-- : --" : visit.suggestedTime().toStringHM());
        ((TextView) vGroupVisitTime.findViewById(R.id.txt_visit_actual_time)).setText(String.format("%1$tH:%1$tM - %2$tH:%2$tM", visit.actualStartTime(), visit.actualCompleteTime()));
        int ss = (int) ((visit.actualCompleteTime().getTime() - visit.actualStartTime().getTime()) / 1000);
        int hh = ss / 3600;
        int mm = (ss - hh * 3600) / 60;
        ((TextView) vGroupVisitTime.findViewById(R.id.txt_visit_actual_duration)).setText((hh > 0) ? String.format("%dh %02dmin", hh, mm) : mm + "min");
        if (visit.state() == VisitState.COMPLETED) {
            vTxtResultStatus.setTextColor(ViewUtils.getColorFromResourcesCompat(getResources(), R.color.app_color_green, null));
            vTxtResultStatus.setText(R.string.job_complete_status_ok);
            vGroupCollectedGoods.setVisibility(View.VISIBLE);
            new Thread(() -> loadGoodCategories(visit), "good-categories-loader").start();
            vGroupCustomer.setVisibility(View.VISIBLE);
            if (visit.isSynced()) {
                vTxtCustomerName.setText(visit.customerName());
                vCautomerSignature.setSignature(visit.customerSignature());
            } else {
                LocalVisitResult localVisit = LocalCompletedVisitsStorage.getLocalVisitResult(visit.id());
                if (localVisit != null) {
                    vTxtCustomerName.setText(localVisit.customerName());
                    vCautomerSignature.setSignature(localVisit.customerSignature());
                }
            }
        } else if (visit.state() == VisitState.FAILED) {
            vTxtResultStatus.setTextColor(ViewUtils.getColorFromResourcesCompat(getResources(), R.color.app_color_red, null));
            vTxtResultStatus.setText(R.string.job_complete_status_failed);
        }

        boolean whiteSacks = false;
        int clearBags = 0;
        if (visit.isSynced()) {
            whiteSacks = visit.whiteSacks() != null && visit.whiteSacks();
            clearBags = visit.clearBags() != null ? visit.clearBags() : 0;
            NoteDbModel note = visit.optNote();
            if (note != null) {
                vGroupDriverNote.setVisibility(View.VISIBLE);
                populateNote(vGroupDriverNote, note.noteText(),
                        note.optEmergencyId(), note.attachedImages(), null);
            }
        } else {
            LocalVisitResult localVisit = LocalCompletedVisitsStorage.getLocalVisitResult(visit.id());
            if (localVisit != null) {
                whiteSacks = localVisit.whiteSacks();
                clearBags = localVisit.clearBags();
                UntiedLocalNote note = localVisit.optDriverNote();
                if (note != null) {
                    vGroupDriverNote.setVisibility(View.VISIBLE);
                    populateNote(vGroupDriverNote, note.getNoteText(),
                            note.getOptEmergency() != null ? note.getOptEmergency().getId() : null, null,
                            note.getImportedImages());
                }
            }
        }

        findViewById(R.id.white_sacks).setVisibility(whiteSacks ? View.VISIBLE : View.GONE);
        findViewById(R.id.clear_bags).setVisibility(clearBags > 0 ? View.VISIBLE : View.GONE);
        ((CheckBox) findViewById(R.id.clear_bags)).setText(String.format("%s: %s",
                getString(R.string.clear_bags), clearBags));
    }

    VisitDbModel mVisit;

    private void loadGoodCategories(final VisitDbModel visit) {
        try (PricedGoodDAO dao = new PricedGoodDAO(new DbSelectionCache())) {
            List<LongId<PricedGoodEntity>> ids = new ArrayList<>();
            Integer totalPayedAmount = 0;
            Integer totalPayedScale = 0;
            List<CollectedGoodEntity> collectedGoods = new ArrayList<>();
            String total = "";
            if (visit.isSynced()) {
                totalPayedAmount = visit.totalPayedAmount();
                totalPayedScale = visit.totalPayedScale();
                collectedGoods = visit.collectedGoods();
                for (CollectedGoodEntity good : collectedGoods) {
                    ids.add(good.getGoodcategoryId());
                }
                total = PublicUtils.formatPriceAsCurrency(totalPayedAmount,
                        totalPayedScale, GlobalProperties.APPLICATION_LOCALE);
            } else {
                LocalVisitResult localVisit = LocalCompletedVisitsStorage.getLocalVisitResult(visit.id());
                if (localVisit != null) {
                    int i = 0;
                    for (LocalCollectedGoodItem good : localVisit.collectedGoods()) {
                        ids.add(good.getGoodcategoryId());
                    }
                    List<PricedGoodDbModel> categories = dao.selectGoodsByIdsForSite(visit.site().id(), ids);
                    final Map<LongId<PricedGoodEntity>, PricedGoodDbModel> goods = new HashMap<>(categories.size());
                    for (PricedGoodDbModel pg : categories) {
                        goods.put(pg.id(), pg);
                    }
                    int total_amount = 0, total_scale = 0;
                    for (LocalCollectedGoodItem good : localVisit.collectedGoods()) {
                        PricedGoodDbModel goodCat = goods.get(good.getGoodcategoryId());

                        collectedGoods.add(new CollectedGoodEntity(
                                LongId.Companion.valueOf(1001 + i),
                                visit.id(),
                                good.getGoodcategoryId(),
                                goodCat.categoryName(), good.getWeightKg(),
                                goodCat.priceValue(),
                                goodCat.priceScale(),
                                i));
                        i++;
                        if (total_scale == 0) {
                            total_scale = goodCat.priceScale();
                            total_amount = goodCat.priceValue() * good.getWeightKg();
                        } else if (goodCat.priceScale() < total_scale) {
                            int dScale = total_scale - goodCat.priceScale();
                            int itemPrice = goodCat.priceValue() * (int) Math.pow(10, dScale);
                            total_amount += itemPrice * good.getWeightKg();
                        } else if (goodCat.priceScale() > total_scale) {
                            int dScale = goodCat.priceScale() - total_scale;
                            total_amount *= (int) Math.pow(10, dScale);
                            total_scale += dScale;
                            total_amount += goodCat.priceValue() * good.getWeightKg();
                        } else {// goodCat.priceScale() == total_scale
                            total_amount += goodCat.priceValue() * good.getWeightKg();
                        }
                    }

                    String txtTotal = PublicUtils.formatPriceAsCurrency(total_amount,
                            total_scale, GlobalProperties.APPLICATION_LOCALE);
                    total = txtTotal;
                }
            }
            List<PricedGoodDbModel> categories = dao.selectGoodsByIdsForSite(visit.site().id(), ids);
            final Map<LongId<PricedGoodEntity>, PricedGoodDbModel> mappp = new HashMap<>(categories.size());
            for (PricedGoodDbModel pg : categories) {
                mappp.put(pg.id(), pg);
            }
            List<CollectedGoodEntity> finalCollectedGoods = collectedGoods;
            String finalTotal = total;
            runOnUiThread(() -> populateGoods(finalTotal, mappp, finalCollectedGoods));

        }
    }

    private void populateGoods(String total,
                               Map<LongId<PricedGoodEntity>, PricedGoodDbModel> goodCategories,
                               List<CollectedGoodEntity> collectedGoods) {
        LinearLayout vContainer = vGroupCollectedGoods.findViewById(R.id.container_collected_goods);
        vContainer.removeAllViews();
        reorderCollectedGoods(collectedGoods, goodCategories);
        LayoutInflater inflater = getLayoutInflater();
        for (CollectedGoodEntity good : collectedGoods) {
            String name = good.getGoodName();
            int weight = good.getCollectedWeight();
            LongId<PricedGoodEntity> categoryId = good.getGoodcategoryId();
            if (vContainer.getChildCount() > 0) {
                View divider = new View(this);
                divider.setBackgroundColor(ViewUtils.getColorFromResourcesCompat(getResources(), R.color.text_gray, null));
                vContainer.addView(divider, ViewGroup.LayoutParams.MATCH_PARENT, 1);
            }
            inflater.inflate(R.layout.layout_good_item_details, vContainer, true);
            ViewGroup vChild = (ViewGroup) vContainer.getChildAt(vContainer.getChildCount() - 1);
            ((TextView) vChild.findViewById(R.id.txt_good_name)).setText(name);
            PricedGoodDbModel goodCat = goodCategories.get(categoryId);
            String txt_amount = (goodCat != null && goodCat.unitOfMeasure() != null) ?
                    getResources().getString(R.string.collected_amount,
                            weight, goodCat.unitOfMeasure()) : "" + weight;
            ((TextView) vChild.findViewById(R.id.txt_amount_weight)).setText(txt_amount);
            if (goodCat != null && goodCat.ewcCode() != null && goodCat.isHasPrice()) {
                String txtTotal = PublicUtils.formatPriceAsCurrency(goodCat.priceValue() *
                        weight, goodCat.priceScale(), GlobalProperties.APPLICATION_LOCALE);
                ((TextView) vChild.findViewById(R.id.txt_amount_money)).setText(txtTotal);
                String txtPrice = PublicUtils.formatPriceAsCurrency(goodCat.priceValue(), goodCat.priceScale(), GlobalProperties.APPLICATION_LOCALE);
                String txt_ewc_price = (goodCat.unitOfMeasure() == null)
                        ? String.format("%s \u2022 %s", goodCat.ewcCode().getCode(), txtPrice)
                        : String.format("%s \u2022 %s/%s", goodCat.ewcCode().getCode(), txtPrice, goodCat.unitOfMeasure());
                ((TextView) vChild.findViewById(R.id.txt_ewc_and_price)).setText(txt_ewc_price);
            } else {
                vChild.findViewById(R.id.txt_amount_money).setVisibility(View.GONE);
                vChild.findViewById(R.id.txt_ewc_and_price).setVisibility(View.GONE);
            }
        }
        TextView vTotal = vGroupCollectedGoods.findViewById(R.id.txt_total_payed);
        vTotal.setText(total);
    }

    private void reorderCollectedGoods(List<CollectedGoodEntity> src, Map<LongId<PricedGoodEntity>, PricedGoodDbModel> goodCategories) {
        Map<Long, Integer> mapOrders = new HashMap<>();
        for (Map.Entry<LongId<PricedGoodEntity>, PricedGoodDbModel> entr : goodCategories.entrySet()) {
            mapOrders.put(entr.getKey().getValue(), entr.getValue().categoryOrder());
        }
        Collections.sort(src, (ent1, ent2) -> {
            try {
                final int ord1 = mapOrders.get(ent1.getGoodcategoryId().getValue());
                final int ord2 = mapOrders.get(ent2.getGoodcategoryId().getValue());
                return ord1 - ord2; //TODO Validate this !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            } catch (Exception e) {
                return 0; // This should not happen by design;
            }
        });
    }

    private void populateNote(ViewGroup vGroup, String text,
                              LongId<EmergencyCategoryEntity> eId,
                              List<String> images,
                              List<CoverImportModel> localImages) {
        ((TextView) vGroup.findViewById(R.id.txt_driver_note)).setText(text);
        EmergencyCategoryEntity emerg = (eId == null) ? null : EmergencyHolder.getInstance()
                .getEmergencyById(eId);
        if (emerg == null) {
            vGroup.findViewById(R.id.emergency_group).setVisibility(View.GONE);
        } else {
            vGroup.findViewById(R.id.emergency_group).setVisibility(View.VISIBLE);
            ((TextView) vGroup.findViewById(R.id.txt_emergency)).setText(emerg.getTitle());
        }
        if ((images == null || images.size() == 0) &&
                (localImages == null || localImages.size() == 0)) {
            vGroup.findViewById(R.id.note_images_group).setVisibility(View.GONE);
        } else {
            vGroup.findViewById(R.id.note_images_group).setVisibility(View.VISIBLE);
            ImageView[] vIcons = new ImageView[5];
            vIcons[0] = findViewById(R.id.note_icon_1);
            vIcons[1] = findViewById(R.id.note_icon_2);
            vIcons[2] = findViewById(R.id.note_icon_3);
            vIcons[3] = findViewById(R.id.note_icon_4);
            vIcons[4] = findViewById(R.id.note_icon_5);
            for (int i = 0; i < vIcons.length; i++) {
                if (images != null) {
                    if (i < images.size()) {
                        int finalI = i;
                        PDA_Application.getPicasso().load(images.get(i))
                                .into(vIcons[i], new Callback() {
                                    @Override
                                    public void onSuccess() {
                                    }

                                    @Override
                                    public void onError() {
                                        PDA_Application.getPicasso().load(images.get(finalI))
                                                .networkPolicy(NetworkPolicy.OFFLINE)
                                                .into(vIcons[finalI]);
                                    }
                                });
                    } else {
                        vIcons[i].setImageDrawable(null);
                    }
                } else {
                    if (i < localImages.size()) {
                        File f = new File(localImages.get(i).localFile());
                        PDA_Application.getPicasso().load(f)
                                .into(vIcons[i]);
                    } else {
                        vIcons[i].setImageDrawable(null);
                    }
                }
            }
        }
    }

    @Override
    public void onPrintDone() {
        //Not implemented
    }

    @Override
    public void onPrintError() {
        Toaster.toastToUser(R.string.err_print_failed, Toast.LENGTH_LONG);
    }
}