package com.webssa.pda.sunmi.ui.activity;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.GridLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.webssa.library.attachment.model.CoverImportModel;
import com.webssa.library.core.error.AppError;
import com.webssa.library.core.processor.model.Callback;
import com.webssa.library.core.processor.model.Disposable;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.broadcasting.BroadcastController;
import com.webssa.pda.sunmi.dataapi.ApiCreateNoteForShift;
import com.webssa.pda.sunmi.model.UntiedLocalNote;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.storage.dbmodel.BaseNoteDbModel;
import com.webssa.pda.sunmi.ui.annotation.Layout;
import com.webssa.pda.sunmi.ui.dialog.SelectEmergencyCategoryDialog;
import com.webssa.pda.sunmi.util.ViewUtils;

import java.io.File;
import java.util.LinkedList;

@Layout(R.layout.activity_create_note)
public class NoteCreateActivity extends BaseToolbarActivity {


    public static final String ARG_DO_NOT_CREATE_NOTE = "do_not_create_note";
    public static final String ARG_SHIFT_ID = "shift_id";
    public static final String ARG_MAX_IMAGES = "max_images";
    public static final String ARG_NOTE_TO_EDIT = "to_edit_note";

    public static final String RESULT_NOTE = "note_result";



    private boolean argDoNotCreateNote;
    private LongId<ShiftRestEntity> argShiftId;
    private int MAX_IMAGES;

    private GridLayout vGrid;
    private EditText vEdtText;
    private Button vActionPost;


    @Nullable
    private EmergencyCategoryEntity mEmergCategory;
    private final LinkedList<View> mImgItems = new LinkedList<>();




    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        Intent launcher = getIntent();
        argDoNotCreateNote = launcher.getBooleanExtra(ARG_DO_NOT_CREATE_NOTE, false);
        if (!argDoNotCreateNote) {
            argShiftId = launcher.getParcelableExtra(ARG_SHIFT_ID);
            if (argShiftId == null) {
                throw new IllegalStateException("Shift ID is not provided");
            }
        }
        MAX_IMAGES = launcher.getIntExtra(ARG_MAX_IMAGES, 5);

        super.onCreate(savedInstanceState);

        vGrid = findViewById(R.id.grid);
        vEdtText = findViewById(R.id.edt_note);
        vActionPost = findViewById(R.id.action_post);
        vActionPost.setEnabled(false);
        vActionPost.setTag(argDoNotCreateNote ? R.string.action_save_note : R.string.action_post_note);
        vActionPost.setOnClickListener(v -> {
            if (argDoNotCreateNote) {
                onSaveNoteAndReturn();
            } else {
                onPostNote();
            }
        });
        vEdtText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                //Not implemented
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                //Not implemented
            }

            @Override
            public void afterTextChanged(Editable s) {
                vActionPost.setEnabled(!TextUtils.isEmpty(s));
            }
        });
        findViewById(R.id.action_select_emergency).setOnClickListener(v -> startSelectEmergDialog());

        if(argDoNotCreateNote && getIntent().hasExtra(ARG_NOTE_TO_EDIT)) {
            UntiedLocalNote note = getIntent().getParcelableExtra(ARG_NOTE_TO_EDIT);
            vEdtText.setText(note.getNoteText());
            mEmergCategory = note.getOptEmergency();
            populateEmergencyToUI(mEmergCategory);
            for(CoverImportModel img : note.getImportedImages()) {
                mImgItems.add(buildGridItem(img));
            }
        }

        mImgItems.add(buildAddImageActionItem());
        if (mImgItems.size() > MAX_IMAGES) {
            mImgItems.getLast().setVisibility(View.INVISIBLE);
        }
        updateGrid();
    }

    @Nullable
    @Override
    protected Integer getToolbarResId() {
        return R.id.toolbar;
    }

    @Nullable
    @Override
    protected String getScreenTitle() {
        return getString(R.string.screen_title_create_note);
    }

    @Nullable
    @Override
    protected String getScreenSubTitle() {
        return null;
    }

    private View buildGridItem(CoverImportModel model) {
        View item = getLayoutInflater().inflate(R.layout.layout_note_editor_image_item, vGrid, false);
        item.setTag(model);
        item.setOnClickListener(mImgItemClicker);
        ImageView iv = (ImageView) ((FrameLayout) item).getChildAt(0);
        PDA_Application.getPicasso().load(new File(model.localFile())).into(iv);
        return item;
    }

    private View buildAddImageActionItem() {
        View item = getLayoutInflater().inflate(R.layout.layout_add_image, vGrid, false);
        item.setOnClickListener(this::onAddImage);
        return item;
    }

    private void updateGrid() {
        vGrid.removeAllViews();
        int row=0, col=0;
        for (View v : mImgItems) {

            GridLayout.Spec rowSpec = GridLayout.spec(row, 1);
            GridLayout.Spec colSpec = GridLayout.spec(col, 1);
            GridLayout.LayoutParams glp = new GridLayout.LayoutParams(rowSpec, colSpec);
            vGrid.addView(v, glp);

            if (col == 2) {
                col = 0;
                row ++;
            } else {
                col ++;
            }
        }


    }

    private final View.OnClickListener mImgItemClicker = v -> {
        new AlertDialog.Builder(this).setMessage(R.string.msg_remove_image).setPositiveButton(android.R.string.yes, (dialog, which) -> {
            mImgItems.remove(v);
            if (mImgItems.size() <= MAX_IMAGES) {
                mImgItems.getLast().setVisibility(View.VISIBLE);
            }
            updateGrid();
        }).setNegativeButton(android.R.string.no, null).show();
    };

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onBackPressed() {
        if (!TextUtils.isEmpty(vEdtText.getText()) || mImgItems.size() > 1) {
            new AlertDialog.Builder(this)
                    .setMessage(R.string.msg_loose_unsaved_data)
                    .setPositiveButton(android.R.string.yes, (dialog, which) -> {
                        setResult(RESULT_CANCELED);
                        super.onBackPressed();
                    })
                    .setNegativeButton(android.R.string.no, null)
                    .show();
        } else {
            super.onBackPressed();
        }
    }

    /****************************  Add icon image  ************************************************/
    private static final int REQUEST_SELECT_AVATAR = 101;

    private void onAddImage(View v) {
        String[] options = { getString(R.string.action_take_picture), getString(R.string.action_pick_from_gallery) };
        new AlertDialog.Builder(this)
                .setTitle(R.string.dialog_title_attach_image)
                .setItems(options, (dialogInterface, index) -> {
                    switch (index) {
                        case 0:
                            startAttachAvatarActivity(true);
                            break;
                        case 1:
                            startAttachAvatarActivity(false);
                            break;
                    }
                    dialogInterface.dismiss();
                }).show();
    }

    public void startAttachAvatarActivity(boolean live) {
        Intent intent = new Intent(this, AttachAvatarActivity.class);
        intent.putExtra(AttachAvatarActivity.ARG_ATTACHMENT_LIVE, live);
        startActivityForResult(intent, REQUEST_SELECT_AVATAR);
    }


    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_SELECT_AVATAR) {
            if (resultCode != RESULT_CANCELED && (data != null)) {
                CoverImportModel importedIcon = data.getParcelableExtra(AttachAvatarActivity.RESULT_COVERIMAGE);
                mImgItems.add(mImgItems.size()-1, buildGridItem(importedIcon));
                if (mImgItems.size() > MAX_IMAGES) {
                    mImgItems.getLast().setVisibility(View.INVISIBLE);
                }
                updateGrid();
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data);
        }
    }

    /**********************************************************************************************/
    private void startSelectEmergDialog() {
        LongId<EmergencyCategoryEntity> sel_id = (mEmergCategory == null) ? null : mEmergCategory.id();
        SelectEmergencyCategoryDialog.newInstance(sel_id, (@Nullable EmergencyCategoryEntity emerg) -> populateEmergencyToUI(mEmergCategory = emerg))
                .show(getSupportFragmentManager(), "emergencies-dialog");
    }

    private void populateEmergencyToUI(EmergencyCategoryEntity emerg) {
        TextView tv = findViewById(R.id.txt_emerg);
        if (emerg == null) {
            tv.setText(R.string.item_no_emergency);
            tv.setTextColor(ViewUtils.getColorFromResourcesCompat(getResources(), R.color.text_light_gray, null));
        } else {
            tv.setText(emerg);
            tv.setTextColor(ViewUtils.getColorFromResourcesCompat(getResources(), R.color.text_dark, null));
        }
    }

    /**********************************************************************************************/
    private void onSaveNoteAndReturn() {
        UntiedLocalNote.Builder bld = UntiedLocalNote.builder()
                .setNoteText(vEdtText.getText().toString())
                .setEmergency(mEmergCategory);
        for (View v : mImgItems) {
            if (v.getTag() instanceof CoverImportModel) bld.addImage((CoverImportModel) v.getTag());
        }
        Intent result = new Intent();
        result.putExtra(RESULT_NOTE, bld.build());
        setResult(RESULT_OK, result);
        finish();
    }

    private void onPostNote() {
        if (argShiftId == null) throw new IllegalStateException("No valid Shift ID");
        UntiedLocalNote.Builder bld = UntiedLocalNote.builder()
                .setNoteText(vEdtText.getText().toString())
                .setEmergency(mEmergCategory);
        for (View v : mImgItems) {
            if (v.getTag() instanceof CoverImportModel) bld.addImage((CoverImportModel) v.getTag());
        }

        ApiCreateNoteForShift api = new ApiCreateNoteForShift(argShiftId, bld.build());
        api.on(PDA_Application.getDataProvidingApiExecutor()).executeAsync(new Callback<BaseNoteDbModel>() {
            @Override
            public void onSubscribe(@NonNull Disposable d) {
                showPostDialog();
            }

            @Override
            public void onSuccess(@Nullable BaseNoteDbModel result) {
                BroadcastController.broadcastNoteCreated();
                setResult(RESULT_OK);
                finish();
            }

            @Override
            public void onError(@NonNull AppError reason) {
                //TODO Toast this !!!!!!!!!!!!!!!!!!!
            }

            @Override
            public void onComplete(@NonNull Disposable d) {
                dismissPostDialog();
            }
        });
    }

    private ProgressDialog vProgress;

    private void showPostDialog() {
        if (vProgress == null) {
            vProgress = new ProgressDialog(this);
            vProgress.setCancelable(false);
            vProgress.setCanceledOnTouchOutside(false);
            vProgress.setIndeterminate(true);
            vProgress.setMessage(getString(R.string.msg_creating_note));
            vProgress.show();
        }
    }

    private void dismissPostDialog() {
        if (vProgress != null) {
            vProgress.dismiss();
            vProgress = null;
        }
    }
}
