package com.webssa.pda.sunmi.ui.activity;

import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.webssa.library.attachment.model.CoverImportModel;
import com.webssa.library.mypicasso.Callback;
import com.webssa.library.mypicasso.NetworkPolicy;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.core.EmergencyHolder;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.storage.dbmodel.BaseNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.LocalNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.NoteDbModel;
import com.webssa.pda.sunmi.ui.annotation.Layout;

import java.io.File;
import java.util.List;

/**
 * Created by stanislav.perchenko on 11/13/2019, 12:27 PM.
 */
@Layout(R.layout.activity_note_details)
public class NoteDetailsActivity extends BaseToolbarActivity {
    public static final String ARG_NOTE = "note";

    private TextView vTxtTimeCreated;
    private TextView vTxtEmergency;
    private TextView vTxtNoteText;
    private ViewGroup[] vImgIcons = new ViewGroup[5];
    private View vNotSyncedMark;


    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        BaseNoteDbModel note = getIntent().getParcelableExtra(ARG_NOTE);
        if (note == null) {
            throw new IllegalStateException("The 'note' argument is not provided");
        }

        super.onCreate(savedInstanceState);
        vTxtTimeCreated = findViewById(R.id.txt_time_created);
        vTxtEmergency = findViewById(R.id.txt_emerg);
        vTxtNoteText = findViewById(R.id.txt_note_text);
        vImgIcons[0] = findViewById(R.id.icon_container_1);
        vImgIcons[1] = findViewById(R.id.icon_container_2);
        vImgIcons[2] = findViewById(R.id.icon_container_3);
        vImgIcons[3] = findViewById(R.id.icon_container_4);
        vImgIcons[4] = findViewById(R.id.icon_container_5);
        vNotSyncedMark = findViewById(R.id.mark_not_synced);


        vNotSyncedMark.setVisibility(note.isRemoteNote() ? View.INVISIBLE : View.VISIBLE);
        if (note instanceof LocalNoteDbModel) {
            setNoteInternal((LocalNoteDbModel) note);
        } else if (note instanceof NoteDbModel) {
            setNoteInternal((NoteDbModel) note);
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
            return true;
        } else {
            return super.onOptionsItemSelected(item);
        }
    }

    @Nullable
    @Override
    protected Integer getToolbarResId() {
        return R.id.toolbar;
    }

    @Nullable
    @Override
    protected String getScreenTitle() {
        return getString(R.string.screen_title_note_details);
    }

    @Nullable
    @Override
    protected String getScreenSubTitle() {
        return null;
    }

    private void setNoteInternal(LocalNoteDbModel note) {
        vTxtTimeCreated.setText(String.format("%1$td-%1$tm-%1$tY  %1$tR", note.createdAt()));

        EmergencyCategoryEntity emerg = EmergencyHolder.getInstance().getEmergencyById(note.optEmergencyId());
        if (emerg == null) {
            vTxtEmergency.setVisibility(View.INVISIBLE);
        } else {
            vTxtEmergency.setVisibility(View.VISIBLE);
            vTxtEmergency.setText(emerg.getTitle());
        }
        vTxtNoteText.setText(note.noteText());
        List<CoverImportModel> images = note.attachedImages();
        for (int i = 0; i < vImgIcons.length; i++) {
            if (images.size() > i) {
                vImgIcons[i].setVisibility(View.VISIBLE);
                PDA_Application.getPicasso().load(new File(images.get(i).localFile())).into((ImageView) vImgIcons[i].getChildAt(0));
            }
        }
    }

    private void setNoteInternal(NoteDbModel note) {
        vTxtTimeCreated.setText(String.format("%1$td-%1$tm-%1$tY  %1$tR", note.createdAt()));

        EmergencyCategoryEntity emerg = EmergencyHolder.getInstance().getEmergencyById(note.optEmergencyId());
        if (emerg == null) {
            vTxtEmergency.setVisibility(View.INVISIBLE);
        } else {
            vTxtEmergency.setVisibility(View.VISIBLE);
            vTxtEmergency.setText(emerg.getTitle());
        }
        vTxtNoteText.setText(note.noteText());

        List<String> images = note.attachedImages();
        for (int i = 0; i < vImgIcons.length; i++) {
            if (images.size() > i) {
                vImgIcons[i].setVisibility(View.VISIBLE);
                int finalI = i;
                PDA_Application.getPicasso().load(images.get(i))
                        .into((ImageView) vImgIcons[i].getChildAt(0), new Callback() {
                            @Override
                            public void onSuccess() {
                            }

                            @Override
                            public void onError() {
                                PDA_Application.getPicasso().load(images.get(finalI))
                                        .networkPolicy(NetworkPolicy.OFFLINE)
                                        .into((ImageView) vImgIcons[finalI].getChildAt(0));
                            }
                        });
            }
        }
    }
}
