package com.webssa.pda.sunmi.ui.activity;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.hardware.Camera;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.view.MenuItemCompat;
import androidx.fragment.app.DialogFragment;

import com.google.zxing.BarcodeFormat;
import com.webssa.library.core.error.AppError;
import com.webssa.library.widget.SlidingViewFlipper;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.core.SingleVisitEditorStorage;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.ui.dialog.CameraSelectorDialogFragment;
import com.webssa.pda.sunmi.ui.presenter.ScannerPresenter;
import com.webssa.pda.sunmi.ui.widget.ScanVisitResultView;
import com.webssa.pda.sunmi.util.Toaster;

import org.jetbrains.annotations.NotNull;

import java.util.Arrays;

import me.dm7.barcodescanner.zxing.ZXingScannerView;

/**
 * Created by stanislav.perchenko on 11/7/2019, 8:39 AM.
 */
public class QrScannerActivity extends AppCompatActivity implements ScannerActivityView, CameraSelectorDialogFragment.CameraSelectorDialogListener {

    public static final String ARG_SHIFT_ID = "shift_id";

    public static final String RESULT_VISIT_ID = "visit_id";
    public static final String RESULT_VISIT_STATE = "visit_state";

    private ZXingScannerView mScannerView;
    private View vOverlayScanComplete;
    private SlidingViewFlipper vFlipper;

    private boolean mFlash = false;
    private boolean mAutoFocus = true;
    private int mCameraId = Camera.CameraInfo.CAMERA_FACING_BACK;

    private boolean previewPaused;
    private boolean activityResumed = false;
    private boolean cameraResumed = false;

    private ScannerPresenter presenter;

    private boolean isOptMenuDisabled;

    @Override
    protected void onCreate(@Nullable Bundle savedState) {
        super.onCreate(savedState);
        restoreFromSavedState(savedState);
        setContentView(R.layout.activity_qr_scanner);
        mScannerView= (ZXingScannerView) findViewById(R.id.scanner_view);
        vFlipper = findViewById(R.id.flipper);
        vOverlayScanComplete = findViewById(R.id.overlay_scan_complete);
        vOverlayScanComplete.setVisibility(View.GONE);

        setupToolbar();
        getSupportActionBar().setTitle(getString(R.string.screen_title_scan));

        presenter = new ScannerPresenter(this, getIntent().getParcelableExtra(ARG_SHIFT_ID));
        presenter.initializeView();

        mScannerView.setFormats(Arrays.asList(BarcodeFormat.AZTEC, BarcodeFormat.QR_CODE));
        mScannerView.setResultHandler(presenter);
    }

    public void setupToolbar() {
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        final ActionBar ab = getSupportActionBar();
        if(ab != null) {
            ab.setDisplayHomeAsUpEnabled(true);
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        activityResumed = true;
        startCameraIfAllowed();
        mScannerView.setFlash(mFlash);
        mScannerView.setAutoFocus(mAutoFocus);
    }

    @Override
    public void onPause() {
        super.onPause();
        activityResumed = false;
        stopCameraIfResumed();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        presenter.release();
    }


    private void stopCameraIfResumed() {
        if (cameraResumed) {
            mScannerView.stopCamera();
            cameraResumed = false;
        }
    }

    private void startCameraIfAllowed() {
        if (!cameraResumed && activityResumed && !previewPaused) {
            mScannerView.startCamera(mCameraId);
            cameraResumed = true;
        }
    }




    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putBoolean(FLASH_STATE, mFlash);
        outState.putBoolean(AUTO_FOCUS_STATE, mAutoFocus);
        outState.putInt(CAMERA_ID, mCameraId);
    }

    private static final String FLASH_STATE = "FLASH_STATE";
    private static final String AUTO_FOCUS_STATE = "AUTO_FOCUS_STATE";
    private static final String CAMERA_ID = "CAMERA_ID";

    private void restoreFromSavedState(@Nullable Bundle savedState) {
        if(savedState != null) {
            mFlash = savedState.getBoolean(FLASH_STATE, false);
            mAutoFocus = savedState.getBoolean(AUTO_FOCUS_STATE, true);
            mCameraId = savedState.getInt(CAMERA_ID, -1);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuItem menuItem;

        if(mFlash) {
            menuItem = menu.add(Menu.NONE, R.id.menu_flash, 0, R.string.flash_on);
        } else {
            menuItem = menu.add(Menu.NONE, R.id.menu_flash, 0, R.string.flash_off);
        }
        MenuItemCompat.setShowAsAction(menuItem, MenuItem.SHOW_AS_ACTION_NEVER);

        if(mAutoFocus) {
            menuItem = menu.add(Menu.NONE, R.id.menu_auto_focus, 0, R.string.auto_focus_on);
        } else {
            menuItem = menu.add(Menu.NONE, R.id.menu_auto_focus, 0, R.string.auto_focus_off);
        }
        MenuItemCompat.setShowAsAction(menuItem, MenuItem.SHOW_AS_ACTION_NEVER);

        menuItem = menu.add(Menu.NONE, R.id.menu_camera_selector, 0, R.string.select_camera);
        MenuItemCompat.setShowAsAction(menuItem, MenuItem.SHOW_AS_ACTION_NEVER);

        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        return super.onPrepareOptionsMenu(menu) && (vFlipper.getDisplayedChild() == 0) && !isOptMenuDisabled;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle presses on the action bar items
        switch (item.getItemId()) {
            case android.R.id.home:
                close();
                return true;
            case R.id.menu_flash:
                mFlash = !mFlash;
                if(mFlash) {
                    item.setTitle(R.string.flash_on);
                } else {
                    item.setTitle(R.string.flash_off);
                }
                mScannerView.setFlash(mFlash);
                return true;
            case R.id.menu_auto_focus:
                mAutoFocus = !mAutoFocus;
                if(mAutoFocus) {
                    item.setTitle(R.string.auto_focus_on);
                } else {
                    item.setTitle(R.string.auto_focus_off);
                }
                mScannerView.setAutoFocus(mAutoFocus);
                return true;
            case R.id.menu_camera_selector:
                pausePreview();
                DialogFragment cFragment = CameraSelectorDialogFragment.newInstance(this, mCameraId);
                cFragment.show(getSupportFragmentManager(), "camera_selector");
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onCameraSelected(int cameraId) {
        mCameraId = cameraId;
        resumePreview();
        mScannerView.setFlash(mFlash);
        mScannerView.setAutoFocus(mAutoFocus);
    }

    @Override
    public void onSelectionCanceled() {
        resumePreview();
    }


    /**********************************************************************************************/

    @Override
    public void onScanned() {
        vOverlayScanComplete.setVisibility(View.VISIBLE);
    }


    @Override
    public void showVisitNotFound() {
        vOverlayScanComplete.setVisibility(View.GONE);
        new AlertDialog.Builder(this)
                .setMessage(R.string.msg_wrong_code)
                .setCancelable(false)
                .setPositiveButton(android.R.string.ok, (dialog, which) -> close())
                .setOnCancelListener(dialog -> close())
                .show();
    }

    @Override
    public void onVisitFound(VisitDbModel visit) {
        if ((visit.state() == VisitState.NEW) && (SingleVisitEditorStorage.getInstance().inEditVisitId() != null) && !SingleVisitEditorStorage.getInstance().inEditVisitId().equals(visit.id())) {
            isOptMenuDisabled = true;
            invalidateOptionsMenu();
            Toaster.showSnackbar((View) vFlipper.getParent(), R.string.msg_already_edit_another_job, R.string.action_close, v -> close());
        } else {
            vOverlayScanComplete.setVisibility(View.GONE);
            vFlipper.showNext();
            ((Toolbar) findViewById(R.id.toolbar)).setBackgroundColor(getResources().getColor(R.color.colorPrimary, null));
            invalidateOptionsMenu();

            ScanVisitResultView resultView = findViewById(R.id.scan_result);
            resultView.setVisit(visit);
            resultView.setOnOpenVisitActionListener(this::completeActivityWithVisit);
            resultView.setOnStartVisitActionListener(presenter::startVisit);
        }
    }

    @Override
    public void pausePreview() {
        previewPaused = true;
        stopCameraIfResumed();
    }

    @Override
    public void resumePreview() {
        previewPaused = false;
        startCameraIfAllowed();
    }


    private ProgressDialog dialogProgress;
    private int nProgressCount;

    @Override
    public void onStartVisitStartApi() {
        if (dialogProgress == null) {
            dialogProgress = new ProgressDialog(this);
            dialogProgress.setIndeterminate(true);
            dialogProgress.setMessage(getString(R.string.msg_starting_job));
            dialogProgress.setCancelable(false);
            dialogProgress.setCanceledOnTouchOutside(false);
        }

        if (!dialogProgress.isShowing()) {
            dialogProgress.show();
            nProgressCount = 1;
        } else {
            nProgressCount ++;
        }
    }

    @Override
    public void onEndVisitStartApi() {
        if ((nProgressCount > 0) && (dialogProgress != null) && dialogProgress.isShowing()) {
            nProgressCount --;
            if (nProgressCount == 0) {
                dialogProgress.dismiss();
            }
        }
    }

    @Override
    public void onVisitStarted(VisitDbModel visit) {
        completeActivityWithVisit(visit);
    }

    @Override
    public void onStartVisitFailed(AppError reason) {
        new AlertDialog.Builder(this)
                .setTitle(R.string.err_start_job)
                .setMessage(reason.getUserMessage())
                .setCancelable(false)
                .setPositiveButton(android.R.string.ok, (dialog, which) -> close())
                .setOnCancelListener(dialog -> close())
                .show();
    }

    @NotNull
    @Override
    public Context getContext() {
        return this;
    }

    @Override
    public void close() {
        setResult(RESULT_CANCELED);
        finish();
    }


    private void completeActivityWithVisit(VisitDbModel visit) {
        Intent data = new Intent();
        data.putExtra(RESULT_VISIT_ID, visit.id());
        data.putExtra(RESULT_VISIT_STATE, visit.state().name());
        setResult(RESULT_OK, data);
        finish();
    }
}
