package com.webssa.pda.sunmi.ui.activity;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.google.android.material.navigation.NavigationView;
import com.webssa.library.core.error.AppError;
import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.broadcasting.impl.NoteCreatedBroadcastReceiver;
import com.webssa.pda.sunmi.broadcasting.impl.UploadGlobalNotesBroadcastReceiver;
import com.webssa.pda.sunmi.broadcasting.impl.VisitStateChangedBroadcastReceiver;
import com.webssa.pda.sunmi.core.LoadEmergenciesService;
import com.webssa.pda.sunmi.core.LoadGoodsService;
import com.webssa.pda.sunmi.core.Session;
import com.webssa.pda.sunmi.core.SessionHolder;
import com.webssa.pda.sunmi.model.VisitCreationModel;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.ShiftState;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dbmodel.BaseNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.ScheduledModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.ShiftDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.ui.adapter.ShiftAdapter;
import com.webssa.pda.sunmi.ui.presenter.ShiftActivityPresenter;
import com.webssa.pda.sunmi.util.DurationHMS;
import com.webssa.pda.sunmi.util.Toaster;

import org.jetbrains.annotations.NotNull;

import java.util.Date;
import java.util.Objects;

import static com.webssa.pda.sunmi.GlobalProperties.D;


public class ShiftActivity extends AppCompatActivity/*LocationTrackingActivity*/ implements ShiftActivityView {

    public static final String LOG_TAG = "SHIFT_ACTIVITY";

    public static final String ARG_START_FROM_LOGIN = "start_from_login";

    public static final int REQUEST_SCAN_QR = 101;
    public static final int REQUEST_CAMERA_PERMISSION = 102;
    public static final int REQUEST_CREATE_AD_HOC_VISIT = 103;

    private View vSyncOverlayProgress;
    private DrawerLayout mDrawerLayout;
    private ActionBarDrawerToggle mDrawerToggle;
    private NavigationView vDrawer;

    private ImageView vImgMyProfile;
    private TextView vTxtDriverName;


    private ViewGroup vPageProgressGroup;
    private ViewGroup vTimeErrorGroup;
    private ViewGroup vSearchErrorGroup;
    private ViewGroup vNotFoundGroup;
    private ViewGroup vNewShiftGroup;
    private SwipeRefreshLayout vContentRefresher;

    private ShiftAdapter mShiftAdapter;

    private ShiftActivityPresenter mPresenter;

    private Session mSession;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (getIntent().getBooleanExtra(ARG_START_FROM_LOGIN, false)) {
            //todo do it before job!
            LoadGoodsService.launch(getApplicationContext());
            LoadEmergenciesService.launch(getApplicationContext());
        }

        setContentView(R.layout.activity_shift);
        vSyncOverlayProgress = findViewById(R.id.sync_overlay_progress);
        vSyncOverlayProgress.setOnClickListener(v -> {
        });
        vSyncOverlayProgress.setVisibility(View.GONE);
        mDrawerLayout = findViewById(R.id.layout_drawer);
        vDrawer = findViewById(R.id.left_drawer);
        vImgMyProfile = vDrawer.getHeaderView(0).findViewById(R.id.icon_my_profile);
        vTxtDriverName = vDrawer.getHeaderView(0).findViewById(R.id.txt_driver_name);
        vTimeErrorGroup = findViewById(R.id.group_time_error);
        vPageProgressGroup = findViewById(R.id.group_page_progress);
        vSearchErrorGroup = findViewById(R.id.group_connection_problem);
        vSearchErrorGroup.findViewById(R.id.action_search_again).setOnClickListener(this::startSearchShiftDataFromScratch);
        vNotFoundGroup = findViewById(R.id.group_not_found);
        vNotFoundGroup.findViewById(R.id.action_search_again).setOnClickListener(this::startSearchShiftDataFromScratch);
        vNewShiftGroup = findViewById(R.id.group_new_shift);
        vContentRefresher = findViewById(R.id.group_visits);
        vContentRefresher.setEnabled(false);

        vDrawer.setNavigationItemSelectedListener(menuItem -> {
            final int id = menuItem.getItemId();
            switch (id) {
                case R.id.menu_item_create_note:
                    if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null)) {
                        tryStartActivityMakeNote(mShiftAdapter.getShiftId());
                    }
                    break;
                case R.id.menu_item_scan_qr:
                    if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null)) {
                        startActivityScanner(mShiftAdapter.getShiftId());
                    }
                    break;
                case R.id.menu_item_settings:
                    startActivity(new Intent(this, SettingsActivity.class));
                    break;
                    //This was excluded!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                /*case R.id.menu_item_location_test:
                    startActivity(new Intent(this, LocationTestActivity.class));
                    break;*/
                case R.id.menu_item_upload_job:
                    mPresenter.syncVisitsBack(mShiftAdapter.getShiftId());
                    break;
                case R.id.menu_item_logout:
                    askForLogout();
                    break;
                case R.id.menu_item_document_shift_start:
                    openDocument((mSession != null) ? mSession.getDocumentShiftStart() : null, getResources().getString(R.string.action_shift_start));
                    break;
                case R.id.menu_item_document_shift_end:
                    openDocument((mSession != null) ? mSession.getDocumentShiftEnd() : null, getResources().getString(R.string.action_shift_end));
                    break;
                case R.id.menu_item_document_diesel_purchase:
                    openDocument((mSession != null) ? mSession.getDocumentDieselPurchase() : null, getResources().getString(R.string.action_diesel_purchase));
                    break;
            }
            mDrawerLayout.closeDrawer(GravityCompat.START);
            return true;
        });

        setupToolbar();

        mPresenter = new ShiftActivityPresenter(this);
        mPresenter.initializeView();

        startSearchShiftDataFromScratch(null);
        SessionHolder.getInstance().updateMyProfileAsync();
    }

    private void openDocument(@Nullable Uri link, String docName) {
        if (link != null) {
            Intent launcher = new Intent(Intent.ACTION_VIEW);
            launcher.setData(link);
            String title = "Open "+docName;
            Intent chooser = Intent.createChooser(launcher, title);
            startActivity(chooser);
        }
    }

    private void setupToolbar() {
        final int textColor = getColor(R.color.text_white);
        Toolbar vTb = findViewById(R.id.toolbar);
        vTb.setTitleTextColor(textColor);
        setSupportActionBar(vTb);
        ActionBar ab = getSupportActionBar();
        ab.setTitle(R.string.screen_title_shift);
        ab.setHomeButtonEnabled(true);
        ab.setDisplayHomeAsUpEnabled(true);
        mDrawerToggle = new ActionBarDrawerToggle(this, mDrawerLayout, 0, 0);
        mDrawerToggle.setDrawerIndicatorEnabled(true);
        mDrawerToggle.getDrawerArrowDrawable().setSpinEnabled(false);
        mDrawerToggle.getDrawerArrowDrawable().setColor(textColor);
        mDrawerLayout.addDrawerListener(mDrawerToggle);
    }

    private void startSearchShiftDataFromScratch(@Nullable View v) {
        if (mPresenter.loadTodayShift()) {
            showPageProgress();
        }
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);


        if (mDrawerToggle != null) {
            mDrawerToggle.syncState();
        }

        SessionHolder.getInstance().getSessionLiveData().observe(this, session -> {
            PDA_Application.getPicasso().load(session.getUser().getPhotoUrl()).into(vImgMyProfile);
            vTxtDriverName.setText(session.getUser().getFullName());
            mSession = session;
            invalidateOptionsMenu();
        });
        notesReceiver.register(this);
        visitReceiver.register(this);
        uploadGlobalNotes.register(this);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        super.onCreateOptionsMenu(menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        final boolean hasAdapter = (mShiftAdapter != null);
        menu.findItem(R.id.menu_item_scan_qr).setVisible(hasAdapter && (mShiftAdapter.getShiftId() != null));
        Menu drawerMenu = vDrawer.getMenu();
        drawerMenu.findItem(R.id.menu_item_document_shift_start).setVisible((mSession != null) && (mSession.getDocumentShiftStart() != null));
        drawerMenu.findItem(R.id.menu_item_document_shift_end).setVisible((mSession != null) && (mSession.getDocumentShiftEnd() != null));
        drawerMenu.findItem(R.id.menu_item_document_diesel_purchase).setVisible((mSession != null) && (mSession.getDocumentDieselPurchase() != null));
        drawerMenu.findItem(R.id.menu_item_upload_job).setVisible(hasAdapter && mShiftAdapter.isHasUnsyncedData());
        super.onPrepareOptionsMenu(menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                if (mDrawerLayout.isDrawerOpen(GravityCompat.START)) {
                    mDrawerLayout.closeDrawer(GravityCompat.START);
                } else {
                    mDrawerLayout.openDrawer(GravityCompat.START);
                }
                return true;
            case R.id.menu_item_scan_qr:
                if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null)) {
                    startActivityScanner(mShiftAdapter.getShiftId());
                }
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onBackPressed() {
        if (mDrawerLayout.isDrawerOpen(GravityCompat.START)) {
            mDrawerLayout.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mPresenter.release();
        notesReceiver.unRegister(this);
        visitReceiver.unRegister(this);
        uploadGlobalNotes.unRegister(this);
    }

    private final NoteCreatedBroadcastReceiver notesReceiver = new NoteCreatedBroadcastReceiver(() -> {
        if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null)) {
            mPresenter.reloadAcceptedShiftLocally(mShiftAdapter.getShiftId());
        }
    });

    private final VisitStateChangedBroadcastReceiver visitReceiver = new VisitStateChangedBroadcastReceiver(visit -> {
        if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null) && (mShiftAdapter.getShiftId().equals(visit.shiftId()))) {
            mPresenter.reloadAcceptedShiftLocally(mShiftAdapter.getShiftId());
            mDrawerLayout.postDelayed(() -> mPresenter.reSyncSavedShift(mShiftAdapter.getShiftId()), 600);
        }
    });

    private final UploadGlobalNotesBroadcastReceiver uploadGlobalNotes = new UploadGlobalNotesBroadcastReceiver(visit -> {
        if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null)) {
            try {
                mPresenter.reSyncSavedShift(mShiftAdapter.getShiftId());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    });

    private void hideAll() {
        vPageProgressGroup.setVisibility(View.GONE);
        vTimeErrorGroup.setVisibility(View.GONE);
        vSearchErrorGroup.setVisibility(View.GONE);
        vNotFoundGroup.setVisibility(View.GONE);
        vNewShiftGroup.setVisibility(View.GONE);
        vContentRefresher.setVisibility(View.GONE);
    }

    private void showPageProgress() {
        hideAll();
        vPageProgressGroup.setVisibility(View.VISIBLE);
    }

    private void showTimeError(int dtSeconds) {
        hideAll();
        vTimeErrorGroup.setVisibility(View.VISIBLE);

        DurationHMS hms = DurationHMS.fromSeconds(Math.abs(dtSeconds));
        final String message = (dtSeconds > 0)
                ? getString(R.string.msg_wrong_local_time_ahead, hms.toString())
                : getString(R.string.msg_wrong_local_time_behind, hms.toString());

        ((TextView) vTimeErrorGroup.findViewById(R.id.txt_error_wrong_time)).setText(message);

        vTimeErrorGroup.findViewById(R.id.action_open_time_setup).setOnClickListener(v -> {
            startActivity(new Intent(Settings.ACTION_DATE_SETTINGS));
            finish();
        });
    }

    @SuppressLint("SetTextI18n")
    private void showNewShiftFound(@NonNull final ShiftRestEntity shift) {
        vNewShiftGroup.setVisibility(View.VISIBLE);
        ((TextView) vNewShiftGroup.findViewById(R.id.txt_num_visits)).setText(String.format("%02d", shift.getNVisits(), GlobalProperties.APPLICATION_LOCALE));
        ((TextView) vNewShiftGroup.findViewById(R.id.txt_exec_date)).setText(shift.getExecutionDate().toStringDMY());
        ((TextView) vNewShiftGroup.findViewById(R.id.txt_time_start)).setText(Objects.requireNonNull(shift.getScheduledTimeStart()).toStringHM());
        ((TextView) vNewShiftGroup.findViewById(R.id.txt_shift_duration)).setText(String.format("%dh %02dmin", shift.getScheduledDuration().hours(), shift.getScheduledDuration().minutes(), GlobalProperties.APPLICATION_LOCALE));
        vNewShiftGroup.findViewById(R.id.action_accept).setOnClickListener(v -> {
            mPresenter.loadAllSites();
            mPresenter.acceptShift(shift.id());
            showModalProgress();
        });
        vNewShiftGroup.findViewById(R.id.action_reject).setOnClickListener(v -> {
            mPresenter.rejectShift(shift.id());
            showModalProgress();
        });
    }

    private ProgressDialog progressDialog;

    private void showModalProgress() {
        if (progressDialog == null) {
            progressDialog = new ProgressDialog(this);
            progressDialog.setIndeterminate(true);
            progressDialog.setCancelable(false);
            progressDialog.setCanceledOnTouchOutside(false);
            progressDialog.setMessage("Updating...");
        }
        progressDialog.show();
    }

    private void dismissModalProgress() {
        if (progressDialog != null) {
            progressDialog.dismiss();
            progressDialog = null;
        }
    }

    private void tryStartActivityMakeNote(LongId<ShiftRestEntity> shiftId) {
        if (mShiftAdapter.getShiftState() == ShiftState.ACCEPTED) {
            Intent i = new Intent(this, NoteCreateActivity.class);
            i.putExtra(NoteCreateActivity.ARG_MAX_IMAGES, 5);
            i.putExtra(NoteCreateActivity.ARG_SHIFT_ID, shiftId);
            i.putExtra(NoteCreateActivity.ARG_DO_NOT_CREATE_NOTE, false);
            startActivity(i);
        } else {
            Toaster.toastToUser(R.string.msg_err_note_on_closed_shift, Toast.LENGTH_LONG);
        }
    }

    private void startActivityScanner(LongId<ShiftRestEntity> shiftId) {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) == PackageManager.PERMISSION_GRANTED) {
            Intent i = new Intent(this, QrScannerActivity.class);
            i.putExtra(QrScannerActivity.ARG_SHIFT_ID, shiftId);
            startActivityForResult(i, REQUEST_SCAN_QR);
        } else {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.CAMERA}, REQUEST_CAMERA_PERMISSION);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_CAMERA_PERMISSION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null)) {
                    startActivityScanner(mShiftAdapter.getShiftId());
                }
            } else {
                Toaster.toastToUser(R.string.grant_camera_permission_alert, Toast.LENGTH_LONG);
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        if (requestCode == REQUEST_SCAN_QR) {
            if (resultCode == RESULT_OK && (data != null)) {
                LongId<VisitEntity> visitId = data.getParcelableExtra(QrScannerActivity.RESULT_VISIT_ID);
                VisitState visitState = VisitState.valueOf(data.getStringExtra(QrScannerActivity.RESULT_VISIT_STATE));
                Class<? extends AppCompatActivity> activClazz;
                String arg;
                switch (visitState) {
                    case IN_PROGRESS:
                        activClazz = VisitEditorActivity.class;
                        arg = VisitEditorActivity.ARG_VISIT_ID;
                        break;
                    case COMPLETED:
                    case FAILED:
                        activClazz = CompletedVisitDetailsActivity.class;
                        arg = CompletedVisitDetailsActivity.ARG_VISIT_ID;
                        break;
                    default:
                        activClazz = null;
                        arg = null;
                }
                if (activClazz != null) {
                    Intent i = new Intent(this, activClazz);
                    i.putExtra(arg, visitId);
                    startActivity(i);
                }
            }
        } else if (requestCode == REQUEST_CREATE_AD_HOC_VISIT) {
            if (resultCode == RESULT_OK && (data != null)) {
                VisitCreationModel vcm = data.getParcelableExtra(AdHocVisitCreateActivity.RESULT_VISIT_CREATION);
                mPresenter.createAdHocVisit(vcm);
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data);
        }
    }

    private void onListItemClicked(BaseAdapter adapter, int position) {
        ScheduledModel item = ((ShiftAdapter) adapter).getItem(position);
        if (item instanceof BaseNoteDbModel) {
            Intent i = new Intent(this, NoteDetailsActivity.class);
            i.putExtra(NoteDetailsActivity.ARG_NOTE, (BaseNoteDbModel) item);
            startActivity(i);
        } else if (item instanceof VisitDbModel) {
            VisitDbModel vis = (VisitDbModel) item;
            String argName;
            Class<? extends AppCompatActivity> clazz;
            switch (vis.state()) {
                case IN_PROGRESS:
                    clazz = VisitEditorActivity.class;
                    argName = VisitEditorActivity.ARG_VISIT_ID;
                    break;
                case COMPLETED:
                case FAILED:
                    clazz = CompletedVisitDetailsActivity.class;
                    argName = CompletedVisitDetailsActivity.ARG_VISIT_ID;
                    break;
                default:
                    return;
            }
            startActivity(new Intent(this, clazz).putExtra(argName, vis.id()));
        }
    }

    private void onListItemLongClicked(BaseAdapter adapter, int position) {
        ScheduledModel item = ((ShiftAdapter) adapter).getItem(position);
        if (item instanceof VisitDbModel) {
            final VisitDbModel vis = (VisitDbModel) item;
            if (vis.state() == VisitState.NEW) {
                //Location is not used in this variant!!!!!!!!!!!!!!!!!!
                //final boolean useLT = SettingsHolder.getInstance().getSettings().useLocationTracking();
                if (mShiftAdapter.isHasInProgressVisit()) {
                    // Has in-progress job
                    Toaster.showSnackbar(findViewById(R.id.activity_shift_content), getString(R.string.msg_already_edit_another_job));
                }

                //Location is not used in this variant!!!!!!!!!!!!!!!!!!
                /*else if (useLT && !DataUtils.checkDriverInPlace(vis.site().location(), getCurrentLocation())) {
                    // Not in the place
                    Toaster.showSnackbar(findViewById(R.id.activity_shift_content), getString(R.string.err_not_on_site));
                }*/ else {
                    // Start
                    new AlertDialog.Builder(this)
                            .setMessage(R.string.msg_confirm_start_job_manually)
                            .setPositiveButton(android.R.string.ok, (dialog, which) -> mPresenter.startVisit(vis))
                            .setNegativeButton(android.R.string.cancel, null)
                            .show();
                }
            }
        }
    }

    private void askForLogout() {
        new AlertDialog.Builder(this)
                .setTitle(R.string.action_logout)
                .setMessage(R.string.msg_confirm_logout)
                .setPositiveButton(android.R.string.ok, (dialog, which) -> {
                    SessionHolder.logout();
                    finish();
                })
                .setNegativeButton(android.R.string.cancel, null)
                .show();
    }

    /*******************************  View interface implementation  ******************************/

    @Override
    public void onShiftFound(@NonNull final ShiftDbModel savedShift) {
        dismissModalProgress();
        hideAll();
        vContentRefresher.setVisibility(View.VISIBLE);
        if (mShiftAdapter == null) {
            mShiftAdapter = new ShiftAdapter(this, savedShift);
            mShiftAdapter.setOnItemClickListener(this::onListItemClicked);
            mShiftAdapter.setOnItemLongClickListener(this::onListItemLongClicked);
            mShiftAdapter.setOnSearchNewTasksListener(v -> {
                if (mPresenter.searchNewShiftForToday()) {
                    showModalProgress();
                }
            });
            mShiftAdapter.setOnCreateAdHocVisitListener(shiftId -> {
                if (mShiftAdapter.getShiftState() == ShiftState.ACCEPTED) {
                    Intent i = new Intent(this, AdHocVisitCreateActivity.class);
                    i.putExtra(AdHocVisitCreateActivity.ARG_SHIFT_ID, shiftId);
                    startActivityForResult(i, REQUEST_CREATE_AD_HOC_VISIT);
                } else {
                    Toaster.toastToUser(R.string.msg_err_note_on_closed_shift, Toast.LENGTH_LONG);
                }
            });
            ListView lv = vContentRefresher.findViewById(R.id.list);
            lv.setAdapter(mShiftAdapter);
        } else {
            mShiftAdapter.setShift(savedShift);
        }
        vContentRefresher.setRefreshing(false);
        vContentRefresher.setEnabled(true);
        vContentRefresher.setOnRefreshListener(() -> mPresenter.reSyncSavedShift(savedShift.id()));

        if (mShiftAdapter.isHasUnsyncedData()) {
            mPresenter.syncVisitsBack(savedShift.id());
        }

        invalidateOptionsMenu();
    }

    @Override
    public void onNewShiftFound(@NonNull ShiftRestEntity shift, @NonNull Date serverTime) {
        hideAll();
        dismissModalProgress();
        final Date now = new Date();
        int dt = (int) Math.round((now.getTime() - serverTime.getTime()) / 1000.0);
        if (Math.abs(dt) > 300) {
            showTimeError(dt);
        } else {
            showNewShiftFound(shift);
        }
        invalidateOptionsMenu();
    }

    @Override
    public void onNewShiftNotFound(@Nullable AppError reason) {
        dismissModalProgress();
        if (mShiftAdapter != null) {
            //Already have a shift
            new AlertDialog.Builder(this).setMessage(R.string.msg_no_more_shifts).setPositiveButton(android.R.string.ok, null).show();
        } else if (reason != null) {
            // Request error
            hideAll();
            vSearchErrorGroup.setVisibility(View.VISIBLE);
            String msg = String.format("%s - %s", reason.getUserMessage(), reason.getInternalMessage());
            ((TextView) vSearchErrorGroup.findViewById(R.id.txt_search_error)).setText(msg);
        } else {// (reason == null) - not found
            hideAll();
            vNotFoundGroup.setVisibility(View.VISIBLE);
        }
        invalidateOptionsMenu();
    }

    @Override
    public void onNewShiftAcceptFailed(@NonNull AppError reason) {
        dismissModalProgress();
        Toaster.toastError(this, reason, R.string.msg_err_accept_shift);
    }

    @Override
    public void onNewShiftRejected(@NonNull LongId<ShiftRestEntity> shiftId) {
        dismissModalProgress();
        startSearchShiftDataFromScratch(null);
    }

    @Override
    public void onNewShiftRejectFailed(@NonNull AppError reason) {
        dismissModalProgress();
        Toaster.toastError(this, reason, R.string.msg_err_reject_shift);
    }

    @Override
    public void onShiftUpdateFailed(@NonNull LongId<ShiftRestEntity> shiftId, @NonNull AppError reason) {
        vContentRefresher.setRefreshing(false);
        Toaster.toastError(this, reason, R.string.err_refresh_shift);
    }


    private ProgressDialog dialogProgress;
    private int nProgressCount;

    @Override
    public void onStartVisitStartApi() {
        if (dialogProgress == null) {
            dialogProgress = new ProgressDialog(this);
            dialogProgress.setIndeterminate(true);
            dialogProgress.setMessage(getString(R.string.msg_starting_job));
            dialogProgress.setCancelable(false);
            dialogProgress.setCanceledOnTouchOutside(false);
        }

        if (!dialogProgress.isShowing()) {
            dialogProgress.show();
            nProgressCount = 1;
        } else {
            nProgressCount++;
        }
    }

    @Override
    public void onEndVisitStartApi() {
        if ((nProgressCount > 0) && (dialogProgress != null) && dialogProgress.isShowing()) {
            nProgressCount--;
            if (nProgressCount == 0) {
                dialogProgress.dismiss();
            }
        }
    }

    @Override
    public void onVisitStarted(VisitDbModel visit) {
        Intent i = new Intent(this, VisitEditorActivity.class);
        i.putExtra(VisitEditorActivity.ARG_VISIT_ID, visit.id());
        startActivity(i);
    }

    @Override
    public void onStartVisitFailed(AppError reason) {
        new AlertDialog.Builder(this)
                .setTitle(R.string.err_start_job)
                .setMessage(reason.getUserMessage())
                .setCancelable(false)
                .setPositiveButton(android.R.string.ok, (dialog, which) -> close())
                .setOnCancelListener(dialog -> close())
                .show();
    }

    @Override
    public void onAdHocVisitCreated(VisitDbModel adHocVisit) {
        if ((mShiftAdapter != null) && (mShiftAdapter.getShiftId() != null)) {
            mPresenter.reloadAcceptedShiftLocally(mShiftAdapter.getShiftId());
        }
    }

    @Override
    public void onAdHocVisitNotCreated(AppError resason) {
        Toaster.toastError(this, resason);
    }


    @Override
    public void onAllSitesLoaded(int nSites) {
        if (D) Log.i(LOG_TAG, nSites+" Sites have been loaded to use in Ad-Hoc visits creation");
    }

    @Override
    public void onLoadAllSitesError(AppError reason) {
        if (D) {
            Log.e(LOG_TAG, "Error load all Sites to use in Ad-Hoc visits creation - "+reason.getClass().getSimpleName());
            Log.e(LOG_TAG, "Error load all Sites to use in Ad-Hoc visits creation - "+reason.getUserMessage());
            Log.e(LOG_TAG, "Error load all Sites to use in Ad-Hoc visits creation - "+reason.getInternalMessage());
            if (reason.getException() != null) {
                Log.e(LOG_TAG, "Error load all Sites to use in Ad-Hoc visits creation - "+reason.getException().getClass().getName());
                reason.getException().printStackTrace();
            }
        } else {
            String title = "Sites not loaded";
            StringBuilder msgBld = new StringBuilder("Available sites have not been loaded, so no Ad-hoc visit can be created.");
            msgBld.append('\n');
            msgBld.append('\n');
            msgBld.append("Error details:");
            msgBld.append('\n');
            msgBld.append("msg1: "+reason.getUserMessage());
            msgBld.append('\n');
            msgBld.append("msg2: "+reason.getInternalMessage());
            msgBld.append('\n');
            msgBld.append('\n');
            msgBld.append("Please make screenshot of this message and send it to the Service provider");

            AlertDialog dialog = new AlertDialog.Builder(this)
                    .setCancelable(false)
                    .setTitle(title)
                    .setMessage(msgBld.toString())
                    .setPositiveButton("OK, I got it", (d, n) -> {})
                    .create();
            dialog.setCanceledOnTouchOutside(false);
            dialog.show();
        }
    }

    @NotNull
    @Override
    public Context getContext() {
        return this;
    }

    @Override
    public void close() {
        // Will not be implemented
    }


    /***************************  Location callbacks  *********************************************/

    /*@Override
    protected LiveData<? extends IActivateLocationSetting> getActivateSetting() {
        return SettingsHolder.getInstance().getSettingsLiveData();
    }*/

    /*@Override
    protected String getLocationPermissionRationaleText() {
        return getString(R.string.location_permission_rationale);
    }*/

    /*@Override
    protected LocationRequest getLocationRequest() {
        LocationRequest lr = new LocationRequest();
        lr.setInterval(10_000);
        lr.setFastestInterval(5_000);
        lr.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
        return lr;
    }*/

    /*@Override
    protected void onStartTrackingLocation() {
        // Not implemented
    }*/

    /*@Override
    protected void onStopTrackingLocation() {
        // Not implemented
    }*/

    /*@Override
    protected void onLocationUpdated(Location location) {
        if (mShiftAdapter != null) {
            mShiftAdapter.setLocation(location);
        }
    }*/

    /*@Override
    protected View getSnackbarContainerView() {
        return findViewById(R.id.activity_shift_content);
    }*/

    @Override
    public void onSyncVisitsStarted() {
        vSyncOverlayProgress.setVisibility(View.VISIBLE);
    }

    @Override
    public void onSyncVisitsFinished(int nChanged) {
        vSyncOverlayProgress.setVisibility(View.GONE);
        try {
            mPresenter.reSyncSavedShift(mShiftAdapter.getShiftId());
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (nChanged > 0) {
            mPresenter.reloadAcceptedShiftLocally(mShiftAdapter.getShiftId());
        }
    }
}
