package com.webssa.pda.sunmi.ui.activity;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.alperez.sunmi.pos.receiptengine.template.PublicUtils;
import com.webssa.library.core.error.AppError;
import com.webssa.library.widget.SignatureEditorView;
import com.webssa.library.widget.SignatureView;
import com.webssa.library.widget.SlidingViewFlipper;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.core.SessionHolder;
import com.webssa.pda.sunmi.core.SingleVisitEditorStorage;
import com.webssa.pda.sunmi.model.LocalCollectedGoodItem;
import com.webssa.pda.sunmi.model.LocalVisitResult;
import com.webssa.pda.sunmi.model.UntiedLocalNote;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.UserEntity;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.rest.model.impl.PricedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dbmodel.impl.PricedGoodDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.ui.adapter.GoodsSelectionAdapter;
import com.webssa.pda.sunmi.ui.annotation.Layout;
import com.webssa.pda.sunmi.ui.dialog.CheckoutDialogFragment;
import com.webssa.pda.sunmi.ui.dialog.PrintReceiptDialogFragment;
import com.webssa.pda.sunmi.ui.presenter.VisitEditorActivityPresenter;
import com.webssa.pda.sunmi.ui.widget.CollectedGoodItemView;
import com.webssa.pda.sunmi.util.Toaster;

import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by stanislav.perchenko on 11/10/2019, 7:51 PM.
 */
@Layout(R.layout.activity_visit_editor)
public class VisitEditorActivity extends BaseToolbarActivity implements VisitEditorActivityView, CheckoutDialogFragment.OnConfirmListener, PrintReceiptDialogFragment.OnCompleteListener {

    private static final int REQUEST_EDIT_NOTE = 1001;

    public static final String ARG_VISIT_ID = "visit_id";


    public static final int PAGE_INDEX_MAIN = 0;
    public static final int PAGE_INDEX_SEL_GOOD = 1;
    public static final int PAGE_INDEX_SEL_WEIGHT = 2;
    public static final int PAGE_INDEX_CUSTOMER = 3;

    private SlidingViewFlipper vFlipper;

    //--- Page 1 ---
    private LinearLayout vCollectedGoodsContainer;
    private View vProgressOverlay;
    private TextView vTxtTotal;

    //--- Signature section ---
    private TextView vTxtCustomerName;
    private View vActionFillInCustomer;
    private ViewGroup vSignContainer;
    private SignatureView vSignView;
    private View vHintTapToSign;
    private View vActionClearSignature;
    private View vActionChangeSignature;


    //--- Page 2 - select good category ---
    //private SelectGoodItemView vGoodsSelector;
    private ListView vListOfGoods;
    private View vClickerLockOnGoodsList;


    //---  Page 3 - select item weight  ---
    private TextView vTxtSelectedGoodCategoryName;
    private TextView vTxtSelectedGoodAmountTitle;
    private EditText vEdtSelectedGoodAmountValue;
    private TextView vTxtSelectedGoodPrice;
    private TextView vTxtSelectedGoodTotal;
    private View vActionCommitGoodSelection;


    //--- Page 4 ---
    private EditText vEdtCustomerName;
    private SignatureEditorView vEdtSignature;
    private View vActionClearSignEditor;
    private View vActionApplySignature;
    private View vHintSignHere;
    private TextView dbgTxtNSignTracks;

    private VisitEditorActivityPresenter mPresenter;

    private LongId<VisitEntity> argVisitId;
    private final List<PricedGoodDbModel> mGoodCategs = new ArrayList<>();

    private GoodsSelectionAdapter mGoodsAdapter;
    private PricedGoodDbModel tmpSelectedGoodCat;
    private int tmpSelectedGoodAmount = 0;

    private CheckBox whiteSacks, clearBags;
    private EditText clearBagsAmountValue;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        argVisitId = getIntent().getParcelableExtra(ARG_VISIT_ID);
        if (argVisitId == null) {
            throw new IllegalStateException(String.format("The '%s' argument is not provided", ARG_VISIT_ID));
        }

        super.onCreate(savedInstanceState);
        vFlipper = findViewById(R.id.flipper);
        vCollectedGoodsContainer = findViewById(R.id.collected_goods_container);
        vTxtTotal = findViewById(R.id.txt_total_payed);
        vListOfGoods = findViewById(R.id.list_of_goods);
        vClickerLockOnGoodsList = findViewById(R.id.clicker_lock_on_goods_list);
        vTxtSelectedGoodCategoryName = findViewById(R.id.txt_selected_good_category);
        vTxtSelectedGoodAmountTitle = findViewById(R.id.txt_selected_good_amount_title);
        vEdtSelectedGoodAmountValue = findViewById(R.id.edt_selected_good_amount_value);
        vTxtSelectedGoodPrice = findViewById(R.id.txt_selected_good_price);
        vTxtSelectedGoodTotal = findViewById(R.id.txt_selected_good_total);
        vActionCommitGoodSelection = findViewById(R.id.action_commit_good_selection);

        vProgressOverlay = findViewById(R.id.overlay_progress);
        vProgressOverlay.setVisibility(View.GONE);

        vFlipper.setOnChildChangedListener(childIndex -> invalidateOptionsMenu());




        mGoodsAdapter = new GoodsSelectionAdapter(this);
        vListOfGoods.setAdapter(mGoodsAdapter);
        mGoodsAdapter.setOnItemClickListener((parent, view, position, id) -> {
            if (id > 0) {
                vClickerLockOnGoodsList.setVisibility(View.VISIBLE);
                vListOfGoods.postDelayed(() -> {
                    vClickerLockOnGoodsList.setVisibility(View.GONE);
                    onGoodSelected(mGoodsAdapter.getSelectedGood());
                }, 280);
            }
        });

        findViewById(R.id.action_collect_more).setOnClickListener(v -> {
            mGoodsAdapter.clearSelection();
            vFlipper.showNext(PAGE_INDEX_SEL_GOOD);
        });


        whiteSacks = findViewById(R.id.white_sacks);
        whiteSacks.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (buttonView.isShown()) {
                    SingleVisitEditorStorage.getInstance().setWhiteSacks(isChecked);
                }
            }
        });
        clearBagsAmountValue = findViewById(R.id.clear_bags_amount_value);
        clearBagsAmountValue.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                try {
                    int count = Integer.parseInt(s.toString());
                    SingleVisitEditorStorage.getInstance().setClearBagsCount(count);
                } catch (Exception e) {
                    //
                }
            }
        });
        clearBags = findViewById(R.id.clear_bags);
        clearBags.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (buttonView.isShown()) {
                    clearBagsAmountValue.setVisibility(isChecked ? View.VISIBLE : View.GONE);
                    if(!isChecked) {
                        clearBagsAmountValue.setText("");
                        SingleVisitEditorStorage.getInstance().setClearBagsCount(0);
                    }
                }
            }
        });
        int clearBagsCount = SingleVisitEditorStorage.getInstance().getClearBagsCount();
        boolean isWhiteSacks = SingleVisitEditorStorage.getInstance().isWhiteSacks();
        whiteSacks.setChecked(isWhiteSacks);
        clearBags.setChecked(clearBagsCount != 0);
        if (clearBagsCount != 0) {
            clearBagsAmountValue.setVisibility(View.VISIBLE);
            clearBagsAmountValue.setText(String.valueOf(clearBagsCount));
        }

        vEdtSelectedGoodAmountValue.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                final int max = GlobalProperties.COLLECT_ITEM_WEIGHT_MAX_DIGITS;
                if (s.length() > max) {
                    vEdtSelectedGoodAmountValue.setText(s.subSequence(0, max));
                    vEdtSelectedGoodAmountValue.setSelection(max);
                    return;
                }
                try {
                    tmpSelectedGoodAmount = Integer.parseInt(s.toString());
                } catch (Exception e) {
                    tmpSelectedGoodAmount = 0;
                } finally {
                    String textTotal = PublicUtils.formatPriceAsCurrency(tmpSelectedGoodAmount * tmpSelectedGoodCat.priceValue(), tmpSelectedGoodCat.priceScale(),GlobalProperties.APPLICATION_LOCALE);
                    vTxtSelectedGoodTotal.setText(textTotal);
                    vActionCommitGoodSelection.setEnabled(tmpSelectedGoodAmount > 0);
                }
            }
        });

        vActionCommitGoodSelection.setOnClickListener(v -> {
            if ((tmpSelectedGoodCat != null) && (tmpSelectedGoodAmount > 0)) {
                SingleVisitEditorStorage.getInstance().addCollectedItem(argVisitId, new LocalCollectedGoodItem(tmpSelectedGoodCat.siteId(), tmpSelectedGoodCat.id(), tmpSelectedGoodAmount));
                addNewCollectedGoodToUi(tmpSelectedGoodCat, tmpSelectedGoodAmount);
                vEdtSelectedGoodAmountValue.setText("");
                mGoodsAdapter.clearSelection();
                vFlipper.showPrevious(PAGE_INDEX_MAIN);
                updateTotalPayed();
            }
        });


        findViewById(R.id.action_complete_visit).setOnClickListener(this::onTryCompleteVisit);

        UserEntity driver = SessionHolder.currentSession().getUser();
        PDA_Application.getPicasso().load(driver.getPhotoUrl()).into((ImageView) findViewById(R.id.ic_driver_photo));
        ((TextView) findViewById(R.id.txt_driver_name)).setText(driver.getFullName());

        setupSignatureEditorPage();
        setupCustomerUiSection();
        setupDriverNoteUi();

        mPresenter = new VisitEditorActivityPresenter(argVisitId, this);
        mPresenter.initializeView();
    }

    private void onGoodSelected(PricedGoodDbModel good) {
        tmpSelectedGoodCat = good;
        vTxtSelectedGoodCategoryName.setText(tmpSelectedGoodCat.categoryName());
        vTxtSelectedGoodAmountTitle.setText((tmpSelectedGoodCat.unitOfMeasure() == null) ? getString(R.string.weight) : getString(R.string.amount_with_unit, tmpSelectedGoodCat.unitOfMeasure()));
        vEdtSelectedGoodAmountValue.setText("");
        String txtPrice = PublicUtils.formatPrice(tmpSelectedGoodCat.priceValue(), tmpSelectedGoodCat.priceScale(), "\u00A3", null);
        vTxtSelectedGoodPrice.setText((tmpSelectedGoodCat.unitOfMeasure() == null) ? txtPrice : String.format("%s/%s", txtPrice, tmpSelectedGoodCat.unitOfMeasure()));
        vTxtSelectedGoodTotal.setText(mCurrencyFormatter.format(0));
        vActionCommitGoodSelection.setEnabled(false);
        vFlipper.showNext(PAGE_INDEX_SEL_WEIGHT);
        vEdtSelectedGoodAmountValue.requestFocus();
        showSoftwareKeyboard(vEdtSelectedGoodAmountValue);
    }

    @Nullable
    @Override
    protected Integer getToolbarResId() {
        return R.id.toolbar;
    }

    @Nullable
    @Override
    protected String getScreenTitle() {
        return getString(R.string.screen_title_edit_job);
    }

    @Nullable
    @Override
    protected String getScreenSubTitle() {
        return null;
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_edit_visit, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        boolean selectGoodDoneVisible = (vFlipper.getDisplayedChild() == PAGE_INDEX_SEL_GOOD) && mGoodsAdapter.isHasSelection();
        menu.findItem(R.id.menu_item_fail_visit).setVisible(vFlipper.getDisplayedChild() == PAGE_INDEX_MAIN);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;
            case R.id.menu_item_fail_visit:
                onTryFailVisit();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private final NumberFormat mCurrencyFormatter = NumberFormat.getCurrencyInstance(GlobalProperties.APPLICATION_LOCALE);

    @Override
    public void onBackPressed() {
        if (vFlipper.getDisplayedChild() == PAGE_INDEX_SEL_WEIGHT) {
            hideSoftwareKeyboard();
            vEdtSelectedGoodAmountValue.setText("");
            vFlipper.showPrevious(PAGE_INDEX_SEL_GOOD);
        } else if (vFlipper.getDisplayedChild() > PAGE_INDEX_MAIN) {
            mGoodsAdapter.clearSelection();
            vFlipper.showPrevious(PAGE_INDEX_MAIN);
        } else {
            super.onBackPressed();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mPresenter.release();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        if (requestCode == REQUEST_EDIT_NOTE) {
            if (resultCode == RESULT_OK) {
                onNoteCreated(data.getParcelableExtra(NoteCreateActivity.RESULT_NOTE));
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data);
        }
    }

    private PricedGoodDbModel getGoodCategoryById(LongId<PricedGoodEntity> goodId) {
        for (PricedGoodDbModel g_item : mGoodCategs) {
            if (g_item.id().equals(goodId)) return g_item;
        }
        return null;
    }

    private void populateCurrentlySelectedGoodsForVisit() {
        vCollectedGoodsContainer.removeAllViews();
        Collection<LocalCollectedGoodItem> goods = SingleVisitEditorStorage.getInstance().getAllCollectedGoods(argVisitId);
        for (LocalCollectedGoodItem collectedItem : goods) {
            PricedGoodDbModel good = getGoodCategoryById(collectedItem.getGoodcategoryId());
            if (good != null) {
                CollectedGoodItemView childView = new CollectedGoodItemView(this, good, collectedItem.getWeightKg());
                childView.setOnDeleteListener(this::onDeleteCollectedGood);
                vCollectedGoodsContainer.addView(childView, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            }
        }
    }

    private void addNewCollectedGoodToUi(PricedGoodDbModel good, int weightAmount) {
        CollectedGoodItemView childView = null;
        for (int i = 0; i < vCollectedGoodsContainer.getChildCount(); i++) {
            CollectedGoodItemView child = (CollectedGoodItemView) vCollectedGoodsContainer.getChildAt(i);
            if (child.getGoodId().equals(good.id())) {
                childView = child;
                vCollectedGoodsContainer.removeViewAt(i);
                break;
            }
        }
        if (childView == null) {
            childView = new CollectedGoodItemView(this, good, weightAmount);
            childView.setOnDeleteListener(this::onDeleteCollectedGood);
        } else {
            childView.setData(good, weightAmount);
        }
        vCollectedGoodsContainer.addView(childView, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private void updateTotalPayed() {
        int total_amount = 0, total_scale = 0;
        for (LocalCollectedGoodItem lgi : SingleVisitEditorStorage.getInstance().getAllCollectedGoods(argVisitId)) {
            PricedGoodDbModel good = getGoodCategoryById(lgi.getGoodcategoryId());
            if (good != null) {

                if (total_scale == 0) {
                    total_scale = good.priceScale();
                    total_amount = good.priceValue() * lgi.getWeightKg();
                } else if (good.priceScale() < total_scale) {
                    int dScale = total_scale - good.priceScale();
                    int itemPrice = good.priceValue() * (int) Math.pow(10, dScale);
                    total_amount += itemPrice * lgi.getWeightKg();
                } else if (good.priceScale() > total_scale) {
                    int dScale = good.priceScale() - total_scale;
                    total_amount *= (int) Math.pow(10, dScale);
                    total_scale += dScale;
                    total_amount += good.priceValue() * lgi.getWeightKg();
                } else {// good.priceScale() == total_scale
                    total_amount += good.priceValue() * lgi.getWeightKg();
                }
            }
        }
        String txtTotal = PublicUtils.formatPriceAsCurrency(total_amount, total_scale, GlobalProperties.APPLICATION_LOCALE);
        vTxtTotal.setText(txtTotal);
    }

    private void onDeleteCollectedGood(final CollectedGoodItemView goodView) {
        new AlertDialog.Builder(this).setMessage("Remove item?")
                .setPositiveButton(android.R.string.yes, (dialog, which) -> {
                    SingleVisitEditorStorage.getInstance().removeCollectedItem(argVisitId, goodView.getGoodId());
                    vCollectedGoodsContainer.removeView(goodView);
                    updateTotalPayed();
                }).setNegativeButton(android.R.string.no, null).show();
    }


    /***************************  Page 3 - signature editor  **************************************/
    private void setupSignatureEditorPage() {
        vEdtCustomerName = findViewById(R.id.edt_customer_name);
        vEdtSignature = findViewById(R.id.signature_editor);
        vActionClearSignEditor = findViewById(R.id.action_clear_signature_editor);
        vActionApplySignature = findViewById(R.id.action_apply_signature);
        vHintSignHere = findViewById(R.id.txt_hint_sign_here);
        //dbgTxtNSignTracks = findViewById(R.id.txt_num_sign_tracks);
        //dbgTxtNSignTracks.setVisibility(View.GONE);

        vEdtSignature.clear();
        vActionApplySignature.setVisibility(View.GONE);
        vActionClearSignEditor.setVisibility(View.GONE);
        vActionClearSignEditor.setOnClickListener(v -> {
            vEdtSignature.clear();
            updateSignEditorActionsVisibility(0);
        });

        vEdtSignature.setOnSignatureChangeListener(this::updateSignEditorActionsVisibility);
        vEdtSignature.setOnTouchStateChangeListener(vEdt -> {
            vHintSignHere.setVisibility((vEdt.isInTouch() || vEdt.isHasSignature()) ? View.GONE : View.VISIBLE);
            vActionApplySignature.setVisibility(((vEdtCustomerName.getText().length() > 0) && vEdt.isHasSignature() && !vEdt.isInTouch()) ? View.VISIBLE : View.GONE);
        });
        vEdtCustomerName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                // Not implemented
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                // Not implemented
            }

            @Override
            public void afterTextChanged(Editable s) {
                updateSignEditorActionsVisibility(vEdtSignature.getSignTracksCount());
            }
        });

        vActionApplySignature.setOnClickListener(v -> {
            String customerName = vEdtCustomerName.getText().toString();
            if (!TextUtils.isEmpty(customerName)) onSetCustomerSignature(customerName, vEdtSignature.getSignature());
        });
    }

    private void updateSignEditorActionsVisibility(int nSignTracks) {
        boolean hasSign = (nSignTracks > 0);

        //dbgTxtNSignTracks.setVisibility(hasSign ? View.VISIBLE : View.GONE);
        //dbgTxtNSignTracks.setText(""+nSignTracks);
        vActionClearSignEditor.setVisibility(hasSign ? View.VISIBLE : View.GONE);
        vHintSignHere.setVisibility(hasSign ? View.GONE : View.VISIBLE);
        vActionApplySignature.setVisibility(((vEdtCustomerName.getText().length() > 0) && hasSign && !vEdtSignature.isInTouch()) ? View.VISIBLE : View.GONE);
    }


    /***************************  page 1 - Customer/signature section  ****************************/
    private void setupCustomerUiSection() {
        vTxtCustomerName = findViewById(R.id.txt_customer_name);
        vActionFillInCustomer = findViewById(R.id.action_fill_in_customer);
        vSignContainer = findViewById(R.id.signature_container);
        vSignView = findViewById(R.id.signature_view);
        vHintTapToSign = findViewById(R.id.hint_tap_to_sign);
        vActionClearSignature = findViewById(R.id.action_clear_signature);
        vActionChangeSignature = findViewById(R.id.action_change_signature);

        vActionFillInCustomer.setOnClickListener(this::onStartEditCustomer);
        vSignContainer.setOnClickListener(this::onStartEditCustomer);
        vActionChangeSignature.setOnClickListener(this::onStartEditCustomer);

        vActionClearSignature.setOnClickListener(v -> {
            SingleVisitEditorStorage.getInstance().clearCustomerSignature(argVisitId);
            populateCustomerFromStorage();
        });

    }

    private void onStartEditCustomer(View v) {
        final String name = SingleVisitEditorStorage.getInstance().getCustomerName();
        if (name != null) vEdtCustomerName.setText(name);
        vEdtSignature.clear();
        vFlipper.showNext(PAGE_INDEX_CUSTOMER);
    }

    private void onSetCustomerSignature(@NonNull String customerName, @Nullable String signature) {
        SingleVisitEditorStorage.getInstance().setCustomerNameAndSignature(argVisitId, customerName, signature);
        populateCustomerFromStorage();
        onBackPressed();
    }

    private void populateCustomerFromStorage() {
        final String name = SingleVisitEditorStorage.getInstance().getCustomerName();
        final String sign = SingleVisitEditorStorage.getInstance().getCustomerSignature();

        if (name == null) {
            vTxtCustomerName.setVisibility(View.GONE);
            vActionFillInCustomer.setVisibility(View.VISIBLE);
            vSignView.setSignature(null);
            vHintTapToSign.setVisibility(View.VISIBLE);
            vActionClearSignature.setVisibility(View.GONE);
            vActionChangeSignature.setVisibility(View.GONE);
        } else {
            vTxtCustomerName.setText(name);
            vTxtCustomerName.setVisibility(View.VISIBLE);
            vActionFillInCustomer.setVisibility(View.GONE);
            vSignView.setSignature(sign);
            vActionChangeSignature.setVisibility(View.VISIBLE);
            if (sign == null) {
                vHintTapToSign.setVisibility(View.VISIBLE);
                vActionClearSignature.setVisibility(View.GONE);
            } else {
                vHintTapToSign.setVisibility(View.GONE);
                vActionClearSignature.setVisibility(View.VISIBLE);
            }
        }
    }


    /***************************  page 1 - Optional driver note  **********************************/
    private View vActionWriteNote;
    private View vLabelNoDriverNote;
    private TextView vTxtDriverNote;
    private TextView vTxtEmergCateg;
    private ViewGroup vEmergencyGroup;
    private ViewGroup vNoteIconsGroup;
    private ImageView[] vNoteIcons = new ImageView[5];

    private void setupDriverNoteUi() {
        vActionWriteNote = findViewById(R.id.action_fill_in_note);
        vLabelNoDriverNote = findViewById(R.id.label_no_driver_note);
        vTxtDriverNote = findViewById(R.id.txt_driver_note);
        vTxtEmergCateg = findViewById(R.id.txt_emergency);
        vEmergencyGroup = findViewById(R.id.emergency_group);
        vNoteIconsGroup = findViewById(R.id.note_images_group);
        vNoteIcons[0] = findViewById(R.id.note_icon_1);
        vNoteIcons[1] = findViewById(R.id.note_icon_2);
        vNoteIcons[2] = findViewById(R.id.note_icon_3);
        vNoteIcons[3] = findViewById(R.id.note_icon_4);
        vNoteIcons[4] = findViewById(R.id.note_icon_5);


        vActionWriteNote.setOnClickListener(v -> {
            Intent i = new Intent(this, NoteCreateActivity.class);
            i.putExtra(NoteCreateActivity.ARG_MAX_IMAGES, 5);
            i.putExtra(NoteCreateActivity.ARG_DO_NOT_CREATE_NOTE, true);
            UntiedLocalNote currNote = SingleVisitEditorStorage.getInstance().getOptDriverNote();
            if (currNote != null) i.putExtra(NoteCreateActivity.ARG_NOTE_TO_EDIT, currNote);
            startActivityForResult(i, REQUEST_EDIT_NOTE);
        });
    }

    private void onNoteCreated(@Nullable UntiedLocalNote note) {
        if (note == null) {
            SingleVisitEditorStorage.getInstance().clearOptDriverNote(argVisitId);
        } else {
            SingleVisitEditorStorage.getInstance().setOptDriverNote(argVisitId, note);
        }
        populateDriverNoteFromStorage();
    }

    private void populateDriverNoteFromStorage() {
        UntiedLocalNote note = SingleVisitEditorStorage.getInstance().getOptDriverNote();
        if (note == null) {
            vLabelNoDriverNote.setVisibility(View.VISIBLE);
            vTxtDriverNote.setVisibility(View.GONE);
            vEmergencyGroup.setVisibility(View.GONE);
            vNoteIconsGroup.setVisibility(View.GONE);
        } else {
            vLabelNoDriverNote.setVisibility(View.GONE);
            vTxtDriverNote.setVisibility(View.VISIBLE);
            vEmergencyGroup.setVisibility((note.getOptEmergency() == null) ? View.GONE : View.VISIBLE);
            vNoteIconsGroup.setVisibility(View.VISIBLE);

            vTxtDriverNote.setText(note.getNoteText());
            if (note.getOptEmergency() != null) {
                vTxtEmergCateg.setText(note.getOptEmergency());
            }

            for (int i = 0; i < vNoteIcons.length; i++) {
                if (i < note.getImportedImages().size()) {
                    String f_name = note.getImportedImages().get(i).localFile();
                    PDA_Application.getPicasso().load(new File(f_name)).into(vNoteIcons[i]);
                } else {
                    vNoteIcons[i].setImageDrawable(null);
                }
            }
        }

    }


    /***************************  Fail Visit Action  **********************************************/

    private void onTryFailVisit() {
        final UntiedLocalNote note = SingleVisitEditorStorage.getInstance().getOptDriverNote();
        if (note == null) {
            new AlertDialog.Builder(this)
                    .setMessage(R.string.msg_err_no_comment_for_fail_job)
                    .setPositiveButton(android.R.string.ok, null)
                    .show();
        } else if (note.getOptEmergency() == null) {
            new AlertDialog.Builder(this)
                    .setMessage(R.string.msg_err_no_emergency_for_fail_job)
                    .setPositiveButton(android.R.string.ok, null)
                    .show();
        } else if (note.getImportedImages().isEmpty()) {
            new AlertDialog.Builder(this)
                    .setMessage(R.string.msg_err_no_images_for_fail_job)
                    .setPositiveButton(android.R.string.ok, null)
                    .show();
        } else {
            new AlertDialog.Builder(this)
                    .setMessage(R.string.msg_confirm_fail_job)
                    .setPositiveButton(android.R.string.ok, (dialog, which) -> {
                        if (mPresenter.failVisit(argVisitId, note)) {
                            vProgressOverlay.setVisibility(View.VISIBLE);
                        }
                    })
                    .show();
        }
    }


    /***************************  Complete visit Action  ******************************************/
    private void onTryCompleteVisit(View v) {
        final LocalVisitResult visitResult = SingleVisitEditorStorage.getInstance().buildVisitResult();
        if (visitResult == null) {
            new AlertDialog.Builder(this)
                    .setMessage(R.string.msg_err_no_data_for_complete_job)
                    .setPositiveButton(android.R.string.ok, null)
                    .show();
        } else {
            CheckoutDialogFragment.newInstance(visitResult, (ArrayList<PricedGoodDbModel>) mGoodCategs).show(getSupportFragmentManager(), "confirm-dialog");
        }
    }

    @Override
    public void onCollectionConfirmed(LocalVisitResult visitResult) {
        if (mPresenter.completeVisit(visitResult)) {
            vProgressOverlay.setVisibility(View.VISIBLE);
        }
    }


    /***************************  View interface implementation  **********************************/
    @Override
    public void onVisitLoaded(@Nullable VisitDbModel visit) {
        if ((visit == null) || (visit.state() != VisitState.IN_PROGRESS)) {
            finish();
        }

        LongId<VisitEntity> edtId = SingleVisitEditorStorage.getInstance().inEditVisitId();
        if ((edtId != null) && (!edtId.equals(visit.id()))) {
            SingleVisitEditorStorage.getInstance().clear();
            edtId = null;
        }

        if (edtId == null) {
            SingleVisitEditorStorage.getInstance().setInEditVisit(visit.id());
        }
        populateDriverNoteFromStorage();
        populateCustomerFromStorage();
        updateTotalPayed();
    }

    boolean goodsLoadedFirstTime = true;

    @Override
    public void onGoodsLoaded(List<PricedGoodDbModel> goodCategs) {
        mGoodCategs.clear();
        mGoodCategs.addAll(goodCategs);
        mGoodsAdapter.setAllGoods(goodCategs);
        if (goodsLoadedFirstTime) {
            goodsLoadedFirstTime = false;
            populateCurrentlySelectedGoodsForVisit();
            updateTotalPayed();
        }
    }

    @Override
    public void onCompleteVisitCompleted(LocalVisitResult visitResult) {
        vProgressOverlay.setVisibility(View.GONE);
        PrintReceiptDialogFragment.newInstance(visitResult).show(getSupportFragmentManager(), "print-dialog");
    }

    @Override
    public void onCompleteVisitFailed(AppError reason) {
        vProgressOverlay.setVisibility(View.GONE);
        Toaster.toastError(this, reason, R.string.err_complete_job);
    }

    @Override
    public void onFailVisitCompleted() {
        vProgressOverlay.setVisibility(View.GONE);
        finish();
    }

    @Override
    public void onFailVisitFailed(AppError reason) {
        vProgressOverlay.setVisibility(View.GONE);
        Toaster.toastError(this, reason, R.string.err_fail_job);
    }

    @NotNull
    @Override
    public Context getContext() {
        return this;
    }

    @Override
    public void close() {
        throw new IllegalStateException("close option is not supported");
    }

    @Override
    public void onPrintDone() {
        finish();
    }

    @Override
    public void onPrintError() {
        Toaster.toastToUser(R.string.err_print_failed, Toast.LENGTH_LONG);
        vFlipper.postDelayed(() -> finish(), 300);
    }
}
