package com.webssa.pda.sunmi.ui.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.alperez.sunmi.pos.receiptengine.template.PublicUtils;
import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.PricedGoodEntity;
import com.webssa.pda.sunmi.storage.dbmodel.impl.PricedGoodDbModel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by stanislav.perchenko on 12/2/2019, 10:16 AM.
 */
public class GoodsSelectionAdapter extends BaseAdapter {

    public interface OnSelectionChangeListener {
        void onSelectionChanged(LongId<PricedGoodEntity> selectedId);
    }

    private final Context context;
    private final LayoutInflater inflater;

    private AdapterView.OnItemClickListener onItemClickListener;
    private OnSelectionChangeListener onSelectionChangeListener;

    private final List<PricedGoodDbModel> mGoods = new ArrayList<>(16);

    private LongId<PricedGoodEntity> selectedId;

    public GoodsSelectionAdapter(Context context) {
        this.context = context;
        inflater = LayoutInflater.from(context);
    }

    public void setOnSelectionChangeListener(@Nullable OnSelectionChangeListener l) {
        onSelectionChangeListener = l;
    }

    public void setOnItemClickListener(@Nullable AdapterView.OnItemClickListener onItemClickListener) {
        this.onItemClickListener = onItemClickListener;
    }

    public void setAllGoods(Collection<PricedGoodDbModel> goods) {
        mGoods.clear();
        mGoods.addAll(goods);
        if (selectedId != null && (getSelectedGood() == null)) {
            clearSelection();
        }
        notifyDataSetChanged();
    }

    public void clearSelection() {
        if (selectedId != null) {
            selectedId = null;
            notifyDataSetChanged();
            if (onSelectionChangeListener != null) {
                onSelectionChangeListener.onSelectionChanged(null);
            }
        }
    }

    @Nullable
    public PricedGoodDbModel getSelectedGood() {
        if (selectedId != null) {
            for (PricedGoodDbModel gm : mGoods) {
                if (gm.id().equals(selectedId)) return gm;
            }
        }
        return null;
    }

    public boolean isHasSelection() {
        return (selectedId != null);
    }

    @Override
    public int getCount() {
        return mGoods.size();
    }

    @Override
    public long getItemId(int position) {
        return mGoods.get(position).id().getValue();
    }

    @Override
    public PricedGoodDbModel getItem(int position) {
        return mGoods.get(position);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        View row = convertView;
        VH holder;
        if (row == null) {
            row = new FrameLayout(context);
            row.setClickable(false);
            inflater.inflate(R.layout.layout_good_item_checker, (ViewGroup) row, true);
            row.findViewById(R.id.click_container).setOnClickListener(itemClicker);
            row.setTag(holder = new VH(row));
        } else {
            holder = (VH) row.getTag();
        }
        PricedGoodDbModel item = getItem(position);
        PricedGoodDbModel selection = getSelectedGood();
        final boolean isSelected = (selection != null) && selection.id().equals(item.id());
        holder.bindItem(position, item, isSelected);
        return row;
    }

    private final View.OnClickListener itemClicker = v -> {
        int position = ((Integer) v.getTag()).intValue();
        PricedGoodDbModel item = getItem(position);
        boolean changed = !item.id().equals(selectedId);
        if (changed) {
            selectedId = item.id();
            notifyDataSetChanged();
        }

        if (onItemClickListener != null) {
            onItemClickListener.onItemClick(null, v, position, item.id().getValue());
        }

        if (changed && onSelectionChangeListener != null) {
            onSelectionChangeListener.onSelectionChanged(selectedId);
        }
    };


    /**********************************************************************************************/
    private class VH {

        final ViewGroup clickContainer;
        final View frameCheckable;
        final View checkmark;
        final TextView vTxtName;
        final TextView vTxtEWC;
        final TextView vTxtPrice;

        VH(View base) {
            clickContainer = base.findViewById(R.id.click_container);
            frameCheckable = base.findViewById(R.id.frame_bg_checkable);
            checkmark = base.findViewById(R.id.check_mark);
            vTxtName = base.findViewById(R.id.txt_good_name);
            vTxtEWC = base.findViewById(R.id.txt_ewc_code);
            vTxtPrice = base.findViewById(R.id.txt_price);
        }

        void bindItem(Integer position, PricedGoodDbModel gCat, boolean isSelected) {
            clickContainer.setTag(position);
            vTxtName.setText(gCat.categoryName());
            vTxtEWC.setText(gCat.ewcCode().getCode());
            String txtPrice = PublicUtils.formatPriceAsCurrency(gCat.priceValue(), gCat.priceScale(), GlobalProperties.APPLICATION_LOCALE);
            vTxtPrice.setText((gCat.unitOfMeasure() == null) ? txtPrice : String.format("%s/%s", txtPrice, gCat.unitOfMeasure()));
            int level = isSelected ? 1 : 0;
            frameCheckable.getBackground().setLevel(level);
            checkmark.getBackground().setLevel(level);
        }
    }
}
