package com.webssa.pda.sunmi.ui.adapter;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Checkable;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;

import java.util.List;

/**
 * Created by stanislav.perchenko on 07.07.2020 at 20:21.
 */
public class SitesSingleChoiceAdapter extends BaseAdapter {
    public interface OnSelectionChangedListener {
        void onSelectionChanged(SiteEntity site);
    }

    private final LayoutInflater inflater;
    private final List<SiteEntity> sites;

    private SiteEntity mSelectedSite;

    private OnSelectionChangedListener callback;

    public SitesSingleChoiceAdapter(Context context, @NonNull List<SiteEntity> sites) {
        inflater = LayoutInflater.from(context);
        this.sites = sites;
    }

    public void setOnSelectionChangedListener(OnSelectionChangedListener l) {
        callback = l;
    }

    public SiteEntity getSelectedSite() {
        return mSelectedSite;
    }

    @Override
    public int getCount() {
        return sites.size();
    }

    @Override
    public SiteEntity getItem(int position) {
        return sites.get(position);
    }

    @Override
    public long getItemId(int position) {
        return sites.get(position).id().getValue();
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        View row = convertView;
        VH holder;
        if (row == null) {
            row = inflater.inflate(R.layout.checkable_site_list_item, parent, false);
            holder = new VH(row, this);
            row.setTag(holder);
        } else {
            holder = (VH) row.getTag();
        }
        holder.bindData(position, getItem(position));
        return row;
    }


    private class VH {
        final View vBase;
        int boundPosition;
        SiteEntity boundSite;

        final Checkable vCheckMark;
        final TextView vTitle, vCity, vAddr1, vAddr2;

        VH(View base, final BaseAdapter adapter) {
            vBase = base;
            vCheckMark = (Checkable) base.findViewById(R.id.check_mark);
            vTitle = base.findViewById(R.id.txt_site_title);
            vCity = base.findViewById(R.id.txt_site_city);
            vAddr1 = base.findViewById(R.id.txt_site_addr1);
            vAddr2 = base.findViewById(R.id.txt_site_addr2);
            base.setOnClickListener(v -> {
                if ((mSelectedSite == null) || !mSelectedSite.id().equals(boundSite.id())) {
                    mSelectedSite = boundSite;
                    adapter.notifyDataSetChanged();
                    if (callback != null) callback.onSelectionChanged(boundSite);
                }
            });
        }

        void bindData(int position, SiteEntity site) {
            boundPosition = position;
            boundSite = site;
            final String resolvedShopCode = TextUtils.isEmpty(site.getShopCodeText()) ? (""+site.getCode()) : site.getShopCodeText();
            vTitle.setText(String.format("%s - %s", resolvedShopCode, site.getSiteName()));
            StringBuilder sb = new StringBuilder();
            if (!TextUtils.isEmpty(site.getPostCode())) {
                sb.append(site.getPostCode());
                sb.append(", ");
            }
            sb.append(site.getCity());
            if (!TextUtils.isEmpty(site.getTown())) {
                sb.append(", ");
                sb.append(site.getTown());
            }
            vCity.setText(sb.toString());
            vAddr1.setText(site.getAddressLine1());
            if (TextUtils.isEmpty(site.getAddressLine2())) {
                vAddr2.setVisibility(View.GONE);
            } else {
                vAddr2.setVisibility(View.VISIBLE);
                vAddr2.setText(site.getAddressLine2());
            }

            vCheckMark.setChecked(mSelectedSite != null && site.id().equals(mSelectedSite.id()));
        }

    }

}
