package com.webssa.pda.sunmi.ui.dialog;

import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.alperez.sunmi.pos.receiptengine.template.PublicUtils;
import com.webssa.library.widget.SignatureView;
import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.model.LocalCollectedGoodItem;
import com.webssa.pda.sunmi.model.LocalVisitResult;
import com.webssa.pda.sunmi.storage.dbmodel.impl.PricedGoodDbModel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by stanislav.perchenko on 12/4/2019, 11:14 AM.
 */
public class CheckoutDialogFragment extends DialogFragment {

    public interface OnConfirmListener {
        void onCollectionConfirmed(LocalVisitResult visitResult);
    }

    private OnConfirmListener mListener;
    private LocalVisitResult visitResult;
    private ArrayList<PricedGoodDbModel> allGoods;


    public static final String ARG_VISIT_RESULT = "visit_result";
    public static final String ARG_ALL_GOODS = "all_goods";

    public static CheckoutDialogFragment newInstance(@NonNull LocalVisitResult visitResult, @NonNull ArrayList<PricedGoodDbModel> allGoods) {
        assert (visitResult != null);
        assert (allGoods != null);
        Bundle args = new Bundle();
        args.putParcelable(ARG_VISIT_RESULT, visitResult);
        args.putParcelableArrayList(ARG_ALL_GOODS, allGoods);
        CheckoutDialogFragment f = new CheckoutDialogFragment();
        f.setArguments(args);
        return f;
    }



    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (!(context instanceof OnConfirmListener)) {
            throw new IllegalStateException("Parent activity must implement OnConfirmListener");
        }
        mListener = (OnConfirmListener) context;
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Bundle args = getArguments();
        if (args != null) {
            visitResult = args.getParcelable(ARG_VISIT_RESULT);
            allGoods = args.getParcelableArrayList(ARG_ALL_GOODS);
        }
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        Drawable bgTransparent = new ColorDrawable(Color.TRANSPARENT);

        Dialog dialog = new Dialog(getActivity(), getTheme()) {
            @Override
            public boolean onKeyUp(int keyCode, KeyEvent event) {
                if (keyCode == KeyEvent.KEYCODE_BACK && event.isTracking() && !event.isCanceled()) {
                    onBackIntercepted();
                    // No auto-dismiss because of not calling super
                    return true;
                }
                return super.onKeyUp(keyCode, event);
            }
        };

        dialog.getWindow().requestFeature(Window.FEATURE_NO_TITLE);
        dialog.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        dialog.getWindow().setBackgroundDrawable(bgTransparent);

        //----  The hack to hide title-content divider on some phones  ----
        int dividerId = getActivity().getResources().getIdentifier("android:id/titleDivider", null, null);
        if(dividerId > 0) {
            View vDiv = dialog.findViewById(dividerId);
            if(vDiv != null) {
                if(Build.VERSION.SDK_INT >= 16) {
                    vDiv.setBackground(bgTransparent);
                } else {
                    vDiv.setBackgroundDrawable(bgTransparent);
                }
            }
        }

        return dialog;
    }

    private void onBackIntercepted() {
        dismiss();
    }


    private View vContent;
    private ViewGroup vItemContainer;
    private TextView vTxtTotal;
    private TextView vTxtCustomer;
    private SignatureView vCustomerSign;


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (vContent == null) {
            vContent = inflater.inflate(R.layout.checkout_dialog, container, false);

            vItemContainer = vContent.findViewById(R.id.container_collected_items);
            vTxtTotal = vContent.findViewById(R.id.txt_total_payed);
            vTxtCustomer = vContent.findViewById(R.id.txt_customer_name);
            vCustomerSign = vContent.findViewById(R.id.customer_signature);

            vContent.findViewById(R.id.action_back).setOnClickListener(v -> dismiss());
            vContent.findViewById(R.id.action_confirm).setOnClickListener(v -> {
                if (mListener != null) mListener.onCollectionConfirmed(visitResult);
                dismiss();
            });
        } else {
            container.removeView(vContent);
        }
        return vContent;
    }


    private static final String KEY_AMOUNT = "amount";
    private static final String KEY_SCALE = "scale";

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        vItemContainer.removeAllViews();
        if ((visitResult != null) && (allGoods != null) && !allGoods.isEmpty()) {
            Map<String, Integer> accTotal = new HashMap<>(2);
            for (LocalCollectedGoodItem item : visitResult.collectedGoods()) {
                addItemToUi(item, allGoods, accTotal);
            }

            Integer totalAmount = accTotal.get(KEY_AMOUNT);
            Integer totalScale  = accTotal.get(KEY_SCALE);
            String txtTotal = PublicUtils.formatPriceAsCurrency((totalAmount == null) ? 0 : totalAmount, (totalScale == null) ? 2 : totalScale, GlobalProperties.APPLICATION_LOCALE);
            vTxtTotal.setText(txtTotal);

            vTxtCustomer.setText(getString(R.string.customer_with_name, visitResult.customerName()));
            vCustomerSign.setSignature(visitResult.customerSignature());
        }
    }

    private void addItemToUi(LocalCollectedGoodItem item, ArrayList<PricedGoodDbModel> allGoods, Map<String, Integer> accTotal) {
        PricedGoodDbModel theGood = null;
        for (PricedGoodDbModel good : allGoods) {
            if (good.id().equals(item.getGoodcategoryId())) {
                theGood = good;
                break;
            }
        }

        if (theGood == null) {
            return;
        } else if (vItemContainer.getChildCount() > 0) {
            getLayoutInflater().inflate(R.layout.checkout_dialog_items_divider, vItemContainer, true);
        }

        String txtItemAmount = PublicUtils.formatPrice(item.getWeightKg() * theGood.priceValue(), theGood.priceScale(), null, null);

        getLayoutInflater().inflate(R.layout.checkout_dialog_collected_item, vItemContainer, true);

        View vItem = vItemContainer.getChildAt(vItemContainer.getChildCount() - 1);
        ((TextView) vItem.findViewById(R.id.txt_good_name)).setText(theGood.categoryName());
        ((TextView) vItem.findViewById(R.id.txt_collected_weight)).setText("" + item.getWeightKg());
        ((TextView) vItem.findViewById(R.id.txt_collected_amount)).setText(txtItemAmount);

        //--- Update total amount ---

        int totAmount;
        int totScale;
        if (accTotal.isEmpty()) {
            totAmount = 0;
            totScale = 1;
        } else {
            totAmount = accTotal.get(KEY_AMOUNT);
            totScale = accTotal.get(KEY_SCALE);
        }

        if (theGood.priceScale() > totScale) {
            int dScale = theGood.priceScale() - totScale;
            totScale += dScale;
            totAmount *= (int)Math.pow(10, dScale);
            totAmount += item.getWeightKg() * theGood.priceValue();

        } else if (totScale > theGood.priceScale()) {
            int dScale = totScale - theGood.priceScale();
            totAmount += item.getWeightKg() * theGood.priceValue() * (int)Math.pow(10, dScale);

        } else {
            totAmount += item.getWeightKg() * theGood.priceValue();
        }
        accTotal.put(KEY_AMOUNT, totAmount);
        accTotal.put(KEY_SCALE, totScale);
    }
}
