package com.webssa.pda.sunmi.ui.dialog;

import android.app.AlertDialog;
import android.app.Dialog;
import android.os.Bundle;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.core.EmergencyHolder;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.util.Toaster;

import java.util.List;

/**
 * Created by stanislav.perchenko on 11/5/2019, 3:12 PM.
 */
public class SelectEmergencyCategoryDialog extends DialogFragment {

    public interface OnEmergencySelectionListener {
        void onEmergSelected(@Nullable EmergencyCategoryEntity emerg);
    }

    public static SelectEmergencyCategoryDialog newInstance(@Nullable LongId<EmergencyCategoryEntity> selectedEmergId, @Nullable OnEmergencySelectionListener callback) {
        SelectEmergencyCategoryDialog frag = new SelectEmergencyCategoryDialog();
        frag.selectedEmergId = selectedEmergId;
        frag.callback = callback;
        return frag;
    }

    private @Nullable LongId<EmergencyCategoryEntity> selectedEmergId;
    private @Nullable OnEmergencySelectionListener callback;
    private @Nullable EmergencyCategoryEntity selEmerg;

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        List<EmergencyCategoryEntity> eee = EmergencyHolder.getInstance().getEmergencies();
        if (eee.size() == 0) {
            Toaster.toastToUser(R.string.err_no_emergencies, Toast.LENGTH_SHORT);
            dismiss();
            return null;
        }


        final CharSequence[] dialItems = new CharSequence[eee.size()];
        int indexSelected = -1;
        for (int i=0; i<dialItems.length; i++) {
            EmergencyCategoryEntity e_item = eee.get(i);
            dialItems[i] = e_item;
            if (e_item.id().equals(selectedEmergId)) {
                indexSelected = i;
                selEmerg = e_item;
            }
        }


        return new AlertDialog.Builder(getActivity())
                .setTitle(R.string.dialog_title_select_emergency)
                .setSingleChoiceItems(dialItems, indexSelected, (dialog, which) -> {
                    CharSequence item = dialItems[which];
                    if (item instanceof EmergencyCategoryEntity) {
                        selEmerg = (EmergencyCategoryEntity) item;
                        selectedEmergId = selEmerg.id();
                    } else {
                        selEmerg = null;
                        selectedEmergId = null;
                    }
                })
                .setPositiveButton(android.R.string.ok, (dialog, id) -> {
                    if (callback != null) callback.onEmergSelected(selEmerg);
                })
                .setNegativeButton(android.R.string.cancel, null)
                .create();
    }
}
