package com.webssa.pda.sunmi.ui.presenter;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.library.core.error.AppError;
import com.webssa.library.core.processor.model.Callback;
import com.webssa.library.core.processor.model.Disposable;
import com.webssa.pda.sunmi.core.SessionHolder;
import com.webssa.pda.sunmi.dataapi.ApiLogin;
import com.webssa.pda.sunmi.rest.model.DriverLoginRequest;
import com.webssa.pda.sunmi.rest.model.UserEntity;
import com.webssa.pda.sunmi.ui.activity.LoginActivityView;
import com.webssa.pda.sunmi.ui.presenter.base.BasePresenter;

public class LoginPresenter extends BasePresenter<LoginActivityView> {

    private boolean loginInProgress;



    public LoginPresenter(@NonNull LoginActivityView v) {
        super(v);

    }

    @Override
    public void initializeView() {

    }

    public void startLogin(@NonNull DriverLoginRequest creeds) {
        assert (creeds != null);
        if (loginInProgress) {
            cancelAllInProgressRequests();
        } else {
            loginInProgress = true;
        }
        new ApiLogin(getView().getContext(), creeds).on(PDA_Application.getDataProvidingApiExecutor()).executeAsync(new Callback<UserEntity>() {
            @Override
            public void onSubscribe(@NonNull Disposable d) {
                registerApiDisposable(d);
            }

            @Override
            public void onSuccess(@Nullable UserEntity profile) {
                //if (!isReleased()) continueAfterLogin(profile);
                loginInProgress = false;
                if (!isReleased() && (getView() != null)) {
                    getView().onLoginCompleted(profile);
                }

            }

            @Override
            public void onError(@NonNull AppError reason) {
                SessionHolder.logout();
                onLoginError(reason);
            }

            @Override
            public void onComplete(@NonNull Disposable d) {
                unregisterApiDisposable(d);
            }
        });
    }

    /*private void continueAfterLogin(final UserEntity profile) {
        new ApiUpdateConfig(getView().getContext()).on(AvousApplication.getDataProvidingApiExecutor()).executeAsync(new Callback<Boolean>() {
            @Override
            public void onSubscribe(@NonNull Disposable d) {
                registerApiDisposable(d);
            }

            @Override
            public void onSuccess(@Nullable Boolean isUpdated) {
                loginInProgress = false;
                if (!isReleased() && (getView() != null)) {
                    getView().onLoginCompleted(profile);
                }
            }

            @Override
            public void onError(@NonNull AppError reason) {
                SessionHolder.logout();
                onLoginError(reason);
            }

            @Override
            public void onComplete(@NonNull Disposable d) {
                unregisterApiDisposable(d);
            }
        });
    }*/


    private void onLoginError(@NonNull AppError reason) {
        loginInProgress = false;
        if (!isReleased() && (getView() != null)) {
            getView().onLoginFailed(reason);
        }
    }



    public boolean isLoginInProgress() {
        return loginInProgress;
    }





}
