package com.webssa.pda.sunmi.ui.presenter;

import android.annotation.SuppressLint;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.google.zxing.Result;
import com.webssa.library.core.error.AppError;
import com.webssa.library.core.processor.BackgroundExecutor;
import com.webssa.library.core.processor.model.Callback;
import com.webssa.library.core.processor.model.Disposable;
import com.webssa.library.utils.AsyncTaskCompat;
import com.webssa.pda.sunmi.broadcasting.BroadcastController;
import com.webssa.pda.sunmi.dataapi.ApiStartVisit;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.storage.dao.impl.ShiftDAO;
import com.webssa.pda.sunmi.storage.dbhelper.DbSelectionCache;
import com.webssa.pda.sunmi.storage.dbmodel.impl.ShiftDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.ui.activity.ScannerActivityView;
import com.webssa.pda.sunmi.ui.presenter.base.BasePresenter;

import java.util.HashSet;
import java.util.Set;

import me.dm7.barcodescanner.zxing.ZXingScannerView;

/**
 * Created by stanislav.perchenko on 11/7/2019, 9:39 AM.
 */
public class ScannerPresenter extends BasePresenter<ScannerActivityView> implements ZXingScannerView.ResultHandler {

    private final LongId<ShiftRestEntity> shiftId;

    private BackgroundExecutor exec;

    public ScannerPresenter(@NonNull ScannerActivityView view, LongId<ShiftRestEntity> shiftId) {
        super(view);
        assert (shiftId != null);
        this.shiftId = shiftId;
    }

    @Override
    public void initializeView() {
        cancelAllInProgressRequests();
        if (exec == null) exec = new BackgroundExecutor(Looper.getMainLooper());
    }

    @Override
    public synchronized void release() {
        super.release();
        if (task != null) {
            task.cancel(true);
            task = null;
        }
        if (exec != null) {
            exec.release();
            exec = null;
        }
    }

    @Override
    public void handleResult(Result result) {
        playSound();
        ScannerActivityView v = getView();
        if (!isReleased() && (v != null)) {
            v.pausePreview();
            v.onScanned();
            processScannedCode(result.getText());
        }
    }

    private void playSound() {
        try {
            Uri notification = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
            Ringtone r = RingtoneManager.getRingtone(getView().getContext(), notification);
            r.play();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private AsyncTaskCompat<Void, Void, VisitDbModel> task;

    @SuppressLint("StaticFieldLeak")
    private void processScannedCode(final String code) {
        task = new AsyncTaskCompat<Void, Void, VisitDbModel>() {
            @Override
            protected VisitDbModel doInBackground(Void... args) {
                final String uuid = code;

                try { Thread.sleep(800); } catch (InterruptedException e) { }

                try (ShiftDAO dao = new ShiftDAO(new DbSelectionCache())) {
                    ShiftDbModel shift = dao.selectEntryById(shiftId);
                    if (shift != null) {
                        for (VisitDbModel visit : shift.visits()) {
                            SiteEntity site = visit.site();
                            if (uuid.equalsIgnoreCase(site.getUuid())) {
                                return visit;
                            }
                        }
                    }
                }
                return null;
            }

            @Override
            protected void onPostExecute(VisitDbModel visit) {
                ScannerActivityView v = getView();
                if (!isReleased() && (v != null)) {
                    if (visit == null) {
                        v.pausePreview();
                        v.showVisitNotFound();
                    } else {
                        v.onVisitFound(visit);
                    }
                }
                onCancelled(visit);
            }

            @Override
            protected void onCancelled(VisitDbModel visitDbModel) {
                task = null;
            }
        };
        task.safeExecute();
    }





    private final Set<Long> startingVisits = new HashSet<>();

    public void startVisit(VisitDbModel visit) {
        final LongId<VisitEntity> visitId = visit.id();

        if (!startingVisits.add(visitId.getValue())) return;

        new ApiStartVisit(getView().getContext(), visitId).on(exec).executeAsync(new Callback<VisitDbModel>() {
            @Override
            public void onSubscribe(@NonNull Disposable d) {
                registerApiDisposable(d);
                getView().onStartVisitStartApi();
            }

            @Override
            public void onSuccess(@Nullable VisitDbModel result) {
                BroadcastController.broadcastVisitStateChanged(result);
                ScannerActivityView v = getView();
                if (!isReleased() && (v != null)) {
                    v.onVisitStarted(result);
                }
            }

            @Override
            public void onError(@NonNull AppError reason) {
                ScannerActivityView v = getView();
                if (!isReleased() && (v != null)) {
                    v.onStartVisitFailed(reason);
                }
            }

            @Override
            public void onComplete(@NonNull Disposable d) {
                unregisterApiDisposable(d);
                startingVisits.remove(visitId.getValue());
                ScannerActivityView v = getView();
                if (!isReleased() && (v != null)) {
                    v.onEndVisitStartApi();
                }
            }
        });
    }





}
