package com.webssa.pda.sunmi.ui.widget;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.webssa.library.attachment.model.CoverImportModel;
import com.webssa.library.mypicasso.Callback;
import com.webssa.library.mypicasso.NetworkPolicy;
import com.webssa.pda.sunmi.PDA_Application;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.core.EmergencyHolder;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.storage.dbmodel.BaseNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.LocalNoteDbModel;
import com.webssa.pda.sunmi.storage.dbmodel.impl.NoteDbModel;

import java.io.File;
import java.util.List;

/**
 * Created by stanislav.perchenko on 10/23/2019.
 */
public class NoteListItemView extends FrameLayout {

    public interface OnClicker {
        void onClick(int position);
    }

    private TextView vTxtTimeCreated;
    private TextView vTxtEmergency;
    private TextView vTxtNoteText;
    private ImageView[] vImgIcons = new ImageView[5];
    private View vNotSyncedMark;

    private Integer mPosition;

    private final OnClicker clicker;

    public NoteListItemView(@NonNull LayoutInflater inflater, @Nullable OnClicker clicker) {
        super(inflater.getContext());
        this.clicker = clicker;
        setBackground(new ColorDrawable(Color.TRANSPARENT));
        inflater.inflate(R.layout.layout_view_note_list_item, this, true);
        vTxtTimeCreated = findViewById(R.id.txt_time_created);
        vTxtEmergency = findViewById(R.id.txt_emerg);
        vTxtNoteText = findViewById(R.id.txt_note_text);
        vImgIcons[0] = findViewById(R.id.icon_1);
        vImgIcons[1] = findViewById(R.id.icon_2);
        vImgIcons[2] = findViewById(R.id.icon_3);
        vImgIcons[3] = findViewById(R.id.icon_4);
        vImgIcons[4] = findViewById(R.id.icon_5);
        vNotSyncedMark = findViewById(R.id.ic_not_synced);

        View vClickContainer = findViewById(R.id.click_container);
        vClickContainer.setOnClickListener(v -> {
            if ((clicker != null) && (mPosition != null)) {
                clicker.onClick(mPosition);
            }
        });
        vClickContainer.setOnLongClickListener(v -> {
            if ((longClicker != null) && (mPosition != null)) {
                longClicker.onClick(mPosition);
                return true;
            } else {
                return false;
            }
        });
    }

    private OnClicker longClicker;

    public void setOnLongClickListener(OnClicker clicker) {
        longClicker = clicker;
    }

    public void setData(BaseNoteDbModel note, int position) {
        mPosition = position;
        vNotSyncedMark.setVisibility(note.isRemoteNote() ? View.GONE : View.VISIBLE);
        if (note instanceof LocalNoteDbModel) {
            setNoteInternal((LocalNoteDbModel) note);
        } else if (note instanceof NoteDbModel) {
            setNoteInternal((NoteDbModel) note);
        }
    }

    private void setNoteInternal(LocalNoteDbModel note) {
        vTxtTimeCreated.setText(String.format("%tR", note.createdAt()));

        EmergencyCategoryEntity emerg = EmergencyHolder.getInstance().getEmergencyById(note.optEmergencyId());
        if (emerg == null) {
            vTxtEmergency.setVisibility(View.INVISIBLE);
        } else {
            vTxtEmergency.setVisibility(View.VISIBLE);
            vTxtEmergency.setText(emerg.getTitle());
        }
        vTxtNoteText.setText(note.noteText());
        List<CoverImportModel> images = note.attachedImages();
        for (int i = 0; i < vImgIcons.length; i++) {
            if (images.size() > i) {
                vImgIcons[i].setVisibility(View.VISIBLE);
                PDA_Application.getPicasso().load(new File(images.get(i).localFile())).into(vImgIcons[i]);
            } else {
                vImgIcons[i].setVisibility(View.INVISIBLE);
            }
        }
    }

    private void setNoteInternal(NoteDbModel note) {
        vTxtTimeCreated.setText(String.format("%tR", note.createdAt()));

        EmergencyCategoryEntity emerg = EmergencyHolder.getInstance().getEmergencyById(note.optEmergencyId());
        if (emerg == null) {
            vTxtEmergency.setVisibility(View.INVISIBLE);
        } else {
            vTxtEmergency.setVisibility(View.VISIBLE);
            vTxtEmergency.setText(emerg.getTitle());
        }
        vTxtNoteText.setText(note.noteText());

        List<String> images = note.attachedImages();
        for (int i = 0; i < vImgIcons.length; i++) {
            if (images.size() > i) {
                vImgIcons[i].setVisibility(View.VISIBLE);
                int finalI = i;
                PDA_Application.getPicasso().load(images.get(i))
                        .into(vImgIcons[i], new Callback() {
                            @Override
                            public void onSuccess() {
                            }

                            @Override
                            public void onError() {
                                PDA_Application.getPicasso().load(images.get(finalI))
                                        .networkPolicy(NetworkPolicy.OFFLINE)
                                        .into(vImgIcons[finalI]);
                            }
                        });
            } else {
                vImgIcons[i].setVisibility(View.INVISIBLE);
            }
        }
    }

}
