package com.webssa.pda.sunmi.ui.widget;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.util.DataUtils;

/**
 * Created by stanislav.perchenko on 11/7/2019, 9:25 PM.
 */
public class ScanVisitResultView extends FrameLayout {

    public interface OnOpenVisitActionListener {
        void onOpenVisit(VisitDbModel visit);
    }

    public interface OnStartVisitActionListener {
        void onStartVisit(VisitDbModel visit);
    }

    private TextView vTxtVisitState;
    private TextView vTxtSiteName;
    private TextView vTxtSiteAddress;
    private ViewGroup vTimerGroup;
    private TextView vTxtTimerHH, vTxtTimerMM, vTxtTimerSS;
    private TextView vTxtActionStart;

    private OnOpenVisitActionListener onOpenVisitActionListener;
    private OnStartVisitActionListener onStartVisitActionListener;

    private VisitDbModel mVisit;

    public ScanVisitResultView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init(context);
    }

    public ScanVisitResultView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context);
    }

    public ScanVisitResultView(Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(context);
    }

    private void init(Context context) {
        LayoutInflater.from(context).inflate(R.layout.layout_scan_visit_result_view, this, true);
        setBackground(new ColorDrawable(Color.parseColor("#F6F6F6")));
        vTxtVisitState = findViewById(R.id.txt_visit_state);
        vTxtSiteName = findViewById(R.id.txt_site_name);
        vTxtSiteAddress = findViewById(R.id.txt_site_address);
        vTimerGroup = findViewById(R.id.group_timer);
        vTxtTimerHH = vTimerGroup.findViewById(R.id.txt_timer_hh);
        vTxtTimerMM = vTimerGroup.findViewById(R.id.txt_timer_mm);
        vTxtTimerSS = vTimerGroup.findViewById(R.id.txt_timer_ss);
        vTxtActionStart = findViewById(R.id.action_start);
        vTxtActionStart.setVisibility(View.GONE);
        vTxtActionStart.setOnClickListener(v -> {
            if (mVisit != null) {
                if (((mVisit.state() == VisitState.IN_PROGRESS) || (mVisit.state() == VisitState.COMPLETED) || (mVisit.state() == VisitState.FAILED)) && (onOpenVisitActionListener !=null)) {
                    onOpenVisitActionListener.onOpenVisit(mVisit);
                } else if ((mVisit.state() == VisitState.NEW) && (onStartVisitActionListener != null)) {
                    onStartVisitActionListener.onStartVisit(mVisit);
                }
            }
        });
    }

    public void setOnOpenVisitActionListener(OnOpenVisitActionListener l) {
        this.onOpenVisitActionListener = l;
    }

    public void setOnStartVisitActionListener(OnStartVisitActionListener l) {
        this.onStartVisitActionListener = l;
    }

    public void setVisit(@NonNull VisitDbModel visit) {
        mVisit = visit;

        vTxtSiteName.setText(visit.site().getSiteName());
        vTxtSiteAddress.setText(DataUtils.getDiaplayAddress(visit.site()));
        DataUtils.populateVisitState(vTxtVisitState, visit.state());
        populateActionName(visit.state());
        populateVisitTimer(visit);
    }

    private void populateActionName(VisitState state) {
        int textId;
        switch (state) {
            case NEW:
                textId = R.string.action_start_job;
                break;
            case IN_PROGRESS:
                textId = R.string.action_go_to_job;
                break;
            case COMPLETED:
                textId = R.string.action_view_job;
                break;
            case FAILED:
            default:
                vTxtActionStart.setVisibility(View.GONE);
                return;
        }
        vTxtActionStart.setVisibility(View.VISIBLE);
        vTxtActionStart.setText(textId);
    }

    private void populateVisitTimer(VisitDbModel visit) {
        switch (visit.state()) {
            case IN_PROGRESS:
                setupTimerFrom(visit.actualStartTime().getTime());
                break;
            case COMPLETED:
                cancelTimer();
                int ss = (int)((visit.actualCompleteTime().getTime() - visit.actualStartTime().getTime()) / 1000L);
                populateTimerFromSeconds(ss);
                break;
            case  NEW:
            case FAILED:
            default:
                cancelTimer();
                vTimerGroup.setVisibility(View.GONE);
                return;
        }
        vTimerGroup.setVisibility(View.VISIBLE);
    }

    /**********************************************************************************************/
    private Long timerReference;


    private void setupTimerFrom(long timestamp) {
        timerReference = timestamp;
        resumeTimerIfAttached();
    }

    private void cancelTimer() {
        timerReference = null;
        pauseTimer();
    }

    private void resumeTimerIfAttached() {
        if (isAttachedToWindow()) {
            post(mTimerRunnable);
        }
    }

    private void pauseTimer() {
        removeCallbacks(mTimerRunnable);
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        if (timerReference != null) {
            resumeTimerIfAttached();
        }
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        pauseTimer();
    }


    private final Runnable mTimerRunnable = new Runnable() {
        @Override
        public void run() {
            if (timerReference != null) {
                int ss = (int)((System.currentTimeMillis() - timerReference.longValue()) / 1000L);
                populateTimerFromSeconds(ss);
                postDelayed(this, 250);
            }
        }
    };

    @SuppressLint("SetTextI18n")
    private void populateTimerFromSeconds(int ss) {
        int mm = ss / 60;
        ss -= mm * 60;
        int hh = mm / 60;
        mm -= hh * 60;
        vTxtTimerHH.setText(String.format("%02d", hh));
        vTxtTimerMM.setText(String.format("%02d", mm));
        vTxtTimerSS.setText(String.format("%02d", ss));
    }
}
