package com.webssa.pda.sunmi.ui.widget;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.google.common.truth.Truth;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.rest.model.ShiftState;
import com.webssa.pda.sunmi.storage.dbmodel.impl.ShiftDbModel;
import com.webssa.pda.sunmi.util.ViewUtils;

/**
 * Created by stanislav.perchenko on 11/21/2019, 7:36 PM.
 */
public class ShiftListItemDesignedView extends FrameLayout {

    public interface OnSearchNewTasksListener {
        void onStartSearching(ShiftListItemDesignedView v);
    }

    private TextView vTxtTitle;
    private TextView vTxtLabelScheduledDuration;
    private TextView vTxtScheduledDuration;
    private TextView vTxtRedLabel;
    private ViewGroup vGroupCompletedShift;
    private ViewGroup vGroupAcceptedShift;
    private ViewGroup vGroupCompletedShiftMissedVisits;
    private View vActionSearch;

    private TextView vTxtTimerHH, vTxtTimerMM, vTxtTimerSS;

    private ShiftDbModel shift;
    private OnSearchNewTasksListener onSearchNewTasksListener;


    public ShiftListItemDesignedView(@NonNull LayoutInflater inflater) {
        super(inflater.getContext());
        inflater.inflate(R.layout.view_designed_shift_list_item, this, true);
        vTxtTitle = findViewById(R.id.txt_title);
        vTxtLabelScheduledDuration = findViewById(R.id.txt_label_sched_duration);
        vTxtScheduledDuration = findViewById(R.id.txt_scheduled_duration);
        vTxtRedLabel = findViewById(R.id.txt_red_label);
        vGroupCompletedShift = findViewById(R.id.group_completed_shift);
        vGroupAcceptedShift = findViewById(R.id.group_accepted_shift);
        vGroupCompletedShiftMissedVisits = findViewById(R.id.group_completed_shift_missed_visits);
        vActionSearch = findViewById(R.id.action_search_new_tasks);
        vActionSearch.setOnClickListener(v -> {
            if ((shift != null) && (onSearchNewTasksListener != null)) onSearchNewTasksListener.onStartSearching(this);
        });

        vTxtTimerHH = vGroupAcceptedShift.findViewById(R.id.txt_time_elapsed_hh);
        vTxtTimerMM = vGroupAcceptedShift.findViewById(R.id.txt_time_elapsed_mm);
        vTxtTimerSS = vGroupAcceptedShift.findViewById(R.id.txt_time_elapsed_ss);
    }

    public void setOnSearchNewTasksListener(OnSearchNewTasksListener l) {
        this.onSearchNewTasksListener = l;
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        if (shift != null) {
            setShiftDurationTimerEnabled(true);
        }
    }

    @Override
    protected void onDetachedFromWindow() {
        setShiftDurationTimerEnabled(false);
        super.onDetachedFromWindow();
    }

    public void setData(@NonNull ShiftDbModel shift) {
                Truth.assertThat(this.shift = shift).isNotNull();
                hideAll();
                switch (shift.state()) {
                    case NEW:
                        detachShiftDurationTimer();
                        break;
                    case ACCEPTED:
                        populateAccepted(shift);
                        attachShiftDurationTimer();
                break;
            case CANCELLED:
                detachShiftDurationTimer();
                populateCancelled(shift);
                break;
            case CLOSED:
                detachShiftDurationTimer();
                populateClosed(shift);
        }
        setShiftDurationTimerEnabled(shift.state() == ShiftState.ACCEPTED);
    }

    private void hideAll() {
        vTxtLabelScheduledDuration.setVisibility(View.GONE);
        vTxtScheduledDuration.setVisibility(View.GONE);
        vTxtRedLabel.setVisibility(View.GONE);
        vGroupCompletedShift.setVisibility(View.GONE);
        vGroupAcceptedShift.setVisibility(View.GONE);
        vGroupCompletedShiftMissedVisits.setVisibility(View.GONE);
        vActionSearch.setVisibility(View.GONE);
    }

    private void populateAccepted(ShiftDbModel shift) {
        vTxtLabelScheduledDuration.setVisibility(View.VISIBLE);
        vTxtScheduledDuration.setVisibility(View.VISIBLE);
        vGroupAcceptedShift.setVisibility(View.VISIBLE);

        vTxtTitle.setText(R.string.shift_title_accepted);
        String template = (shift.visits().size() > 1) ? "%s%s  (%d jobs)" : "%s%s  (%d job)";
        vTxtScheduledDuration.setText(String.format(template, ((shift.scheduledTimeStart() == null) ? "" : shift.scheduledTimeStart().toStringHM()+"  "), shift.executionDate().toStringDMY(), shift.visits().size()));

        ((TextView) vGroupAcceptedShift.findViewById(R.id.txt_actually_started_hh)).setText((shift.actualTimeStart() == null) ? "--" : String.format("%tH", shift.actualTimeStart()));
        ((TextView) vGroupAcceptedShift.findViewById(R.id.txt_actually_started_mm)).setText((shift.actualTimeStart() == null) ? "--" : String.format("%tM", shift.actualTimeStart()));


        //Fix for the #9693 - Gone this
        vGroupAcceptedShift.findViewById(R.id.group_accepted_shift_scheduled_duration).setVisibility(View.GONE);
        /*if (shift.scheduledDuration() == null) {
            vGroupAcceptedShift.findViewById(R.id.group_accepted_shift_scheduled_duration).setVisibility(View.GONE);
        } else {
            vGroupAcceptedShift.findViewById(R.id.group_accepted_shift_scheduled_duration).setVisibility(View.VISIBLE);
            ((TextView) vGroupAcceptedShift.findViewById(R.id.txt_estimated_duration_hh)).setText(String.format("%02dh", shift.scheduledDuration().hours()));
            ((TextView) vGroupAcceptedShift.findViewById(R.id.txt_estimated_duration_mm)).setText(String.format("%02dmin", shift.scheduledDuration().minutes()));
        }*/
    }

    private void populateCancelled(ShiftDbModel shift) {
        vTxtRedLabel.setVisibility(View.VISIBLE);
        vActionSearch.setVisibility(View.VISIBLE);

        vTxtTitle.setText(R.string.shift_title_cancelled);
        vTxtRedLabel.setText(String.format("%s  (%d visits)", shift.executionDate().toStringDMY(), shift.visits().size()));
    }


    private void populateClosed(ShiftDbModel shift) {
        final int nVisits = shift.visits().size();
        final int nProcessedVisits = shift.nCompletedVisits() + shift.nFailedVisits();
        if (nProcessedVisits < nVisits) {
            // Not all Visits are passed
            vTxtTitle.setText(R.string.shift_title_closed_not_all_passed);
            vTxtRedLabel.setVisibility(View.VISIBLE);
            vGroupCompletedShiftMissedVisits.setVisibility(View.VISIBLE);

            vTxtRedLabel.setText(getResources().getString(R.string.shift_subtitle_one_missed, (nVisits - nProcessedVisits), nVisits));

            ((TextView) vGroupCompletedShiftMissedVisits.findViewById(R.id.txt_actually_started_hh)).setText((shift.actualTimeStart() == null) ? "null" : String.format("%tH", shift.actualTimeStart()));
            ((TextView) vGroupCompletedShiftMissedVisits.findViewById(R.id.txt_actually_started_mm)).setText((shift.actualTimeStart() == null) ? "null" : String.format("%tM", shift.actualTimeStart()));

            if ((shift.actualTimeClosed() != null) && (shift.actualTimeStart() != null)) {
                int durMinutes = (int) ((shift.actualTimeClosed().getTime() - shift.actualTimeStart().getTime()) / 1000 / 60);
                int durHours = durMinutes / 60;
                durMinutes -= durHours * 60;

                ((TextView) vGroupCompletedShiftMissedVisits.findViewById(R.id.txt_time_elapsed_hh)).setText(String.format("%02d", durHours));
                ((TextView) vGroupCompletedShiftMissedVisits.findViewById(R.id.txt_time_elapsed_mm)).setText(String.format("%02d", durMinutes));
            } else {
                ((TextView) vGroupCompletedShiftMissedVisits.findViewById(R.id.txt_time_elapsed_hh)).setText("null");
                ((TextView) vGroupCompletedShiftMissedVisits.findViewById(R.id.txt_time_elapsed_mm)).setText("null");
            }
        } else {
            vTxtTitle.setText(R.string.shift_title_closed_all_passed);
            vGroupCompletedShift.setVisibility(View.VISIBLE);
            TextView vTxtNum = vGroupCompletedShift.findViewById(R.id.txt_num_completed_visits);
            TextView vTxtDescr = vGroupCompletedShift.findViewById(R.id.txt_completed_visits_description);

            int color;
            String text2;
            if (shift.nCompletedVisits() == nVisits) {
                // All visits are passed OK
                color = ViewUtils.getColorFromResourcesCompat(getResources(), R.color.app_color_green, null);
                text2 = (nVisits > 1)
                        ? getResources().getString(R.string.shift_subtitle_all_done, nVisits)
                        : getResources().getString(R.string.shift_subtitle_single_single_done);
            } else {
                // All visits are passed but with failures
                color = ViewUtils.getColorFromResourcesCompat(getResources(), R.color.app_color_red, null);
                text2 = getResources().getString(R.string.shift_jobs_are_done_well);
            }
            vTxtNum.setText(String.format("%d / %d", shift.nCompletedVisits(), nVisits));
            vTxtNum.setTextColor(color);
            vTxtDescr.setText(text2);
            vTxtDescr.setTextColor(color);

            ((TextView) vGroupCompletedShift.findViewById(R.id.txt_complete_visit_date)).setText(String.format("%1$td-%1$tm-%1$tY", shift.actualTimeStart()));
            ((TextView) vGroupCompletedShift.findViewById(R.id.txt_complete_visit_time_start_end)).setText(String.format("%1$tH:%1$tM - %2$tH:%2$tM", shift.actualTimeStart(), shift.actualTimeClosed()));
        }
        vActionSearch.setVisibility(View.VISIBLE);
    }



    /******************************  Timer-related section  ***************************************/

    private boolean timerEnabled;

    private void setShiftDurationTimerEnabled(boolean enabled){
        if (enabled == timerEnabled) {
            return;
        } else if (enabled) {
            post(timerRunnable);
        } else {
            removeCallbacks(timerRunnable);
        }
        timerEnabled = enabled;
    }

    private final Runnable timerRunnable = new Runnable() {
        @Override
        public void run() {
            if (timerEnabled){
                if (isTimerAttachedToUI && (shift != null)) {
                    int ss = (int)((System.currentTimeMillis() - shift.actualTimeStart().getTime()) / 1000);
                    int hh = ss / 3600;
                    ss -= hh*3600;
                    int mm = ss / 60;
                    ss -= mm*60;
                    vTxtTimerHH.setText(String.format("%02d", hh));
                    vTxtTimerMM.setText(String.format("%02d", mm));
                    vTxtTimerSS.setText(String.format("%02d", ss));
                }
                postDelayed(timerRunnable, 200);
            }
        }
    };


    /*****************************   Attach/Detach text field to timer   **************************/
    private boolean isTimerAttachedToUI;

    private void attachShiftDurationTimer() {
        isTimerAttachedToUI = true;
    }

    private void detachShiftDurationTimer() {
        isTimerAttachedToUI = false;
    }
}
