package com.webssa.pda.sunmi.ui.widget;

import android.annotation.SuppressLint;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.location.Location;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.cardview.widget.CardView;

import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.storage.dbmodel.impl.VisitDbModel;
import com.webssa.pda.sunmi.util.DataUtils;
import com.webssa.pda.sunmi.util.ViewUtils;

/**
 * Created by stanislav.perchenko on 10/23/2019.
 */
public class VisitListItemView extends FrameLayout {

    private CardView vCard;
    private ViewGroup vLocationFrame;
    private View vLocationMark;
    private TextView vTxtVisitState;

    private View vNotSyncedMark;
    private TextView vTxtShopName;
    private TextView vTxtShopAddress;
    private ViewGroup vGroupTimeStart;
    private TextView vTxtTimeStart;

    private ViewGroup vTimerGroup;
    private TextView vTxtTimerHH, vTxtTimerMM, vTxtTimerSS;

    private Integer mPosition;
    private VisitDbModel mVisit;
    //private Location mSiteLocation;
    private Location mLocation;

    public interface OnClicker {
        void onClick(int position, VisitDbModel visit);
    }


    private final int colorCompleteVisit;
    private final int colorNewVisit;


    public VisitListItemView(@NonNull LayoutInflater inflater, @Nullable OnClicker clicker) {
        super(inflater.getContext());

        colorCompleteVisit = ViewUtils.getColorFromResourcesCompat(getResources(), R.color.bg_visit_completed, null);
        colorNewVisit = ViewUtils.getColorFromResourcesCompat(getResources(), R.color.default_background, null);

        setBackground(new ColorDrawable(Color.TRANSPARENT));
        inflater.inflate(R.layout.layout_view_visit_list_item, this, true);
        vCard = findViewById(R.id.card);
        vLocationFrame = findViewById(R.id.visit_location_frame);
        vLocationMark = findViewById(R.id.visit_location_mark);
        vLocationMark.setVisibility(View.GONE);
        vTxtVisitState = findViewById(R.id.txt_visit_state);

        vNotSyncedMark = findViewById(R.id.ic_not_synced);
        vTxtShopName = findViewById(R.id.txt_shop_name);
        vTxtShopAddress = findViewById(R.id.txt_shop_address);
        vGroupTimeStart = findViewById(R.id.group_time_start);
        vTxtTimeStart = vGroupTimeStart.findViewById(R.id.txt_time_start);

        vTimerGroup = findViewById(R.id.group_timer);
        vTxtTimerHH = vTimerGroup.findViewById(R.id.txt_timer_hh);
        vTxtTimerMM = vTimerGroup.findViewById(R.id.txt_timer_mm);
        vTxtTimerSS = vTimerGroup.findViewById(R.id.txt_timer_ss);

        View vClickContainer =findViewById(R.id.click_container);
        vClickContainer.setOnClickListener(v -> {
            if ((clicker != null) && (mVisit != null) && (mPosition != null)) {
                clicker.onClick(mPosition, mVisit);
            }
        });
        vClickContainer.setOnLongClickListener(v -> {
            if ((longClicker != null) && (mVisit != null) && (mPosition != null)) {
                longClicker.onClick(mPosition, mVisit);
                return true;
            } else {
                return false;
            }
        });
    }

    private OnClicker longClicker;

    public void setOnLongClickListener(OnClicker clicker) {
        longClicker = clicker;
    }

    public void setData(VisitDbModel visit, int position) {
        mVisit = visit;
        mPosition = position;

        vCard.setCardBackgroundColor((visit.state() == VisitState.COMPLETED || visit.state() == VisitState.FAILED) ? colorCompleteVisit : colorNewVisit);


        if ((visit.state() == VisitState.NEW) && (visit.suggestedTime() != null)) {
            vGroupTimeStart.setVisibility(View.VISIBLE);
            vTxtTimeStart.setText(visit.suggestedTime().toStringHM());
        } else {
            vGroupTimeStart.setVisibility(View.GONE);
        }

        DataUtils.populateVisitState(vTxtVisitState, visit.state());

        vNotSyncedMark.setVisibility(visit.isSynced() ? View.GONE : View.VISIBLE);
        vTxtShopName.setText(visit.site().getSiteName());
        vTxtShopAddress.setText(DataUtils.getDiaplayAddress(visit.site()));

        populateVisitTimer();

        updateUiWithLocation();
    }



    /**
     * This must be called after setData().
     * @param location
     */
    public void setLocation(@Nullable Location location) {
        boolean upd;
        if ((mLocation == null) && (location != null)) {
            upd = true;
        } else if ((mLocation != null) && (location == null)) {
            upd = true;
        } else if ((location != null) && (location.getTime() != mLocation.getTime())) {
            upd = true;
        } else {
            upd = false;
        }
        mLocation = location;
        if (upd) updateUiWithLocation();
    }



    private void updateUiWithLocation() {
        if ((mVisit != null) && (mVisit.state() == VisitState.NEW) && DataUtils.checkDriverInPlace(mVisit.site().location(), mLocation)) {
            vLocationMark.setVisibility(View.VISIBLE);
            vLocationFrame.getBackground().setLevel(1);
        } else {
            vLocationMark.setVisibility(View.GONE);
            vLocationFrame.getBackground().setLevel(0);
        }
    }


    private void populateVisitTimer() {

        if ((mVisit != null) && (mVisit.state() == VisitState.IN_PROGRESS)) {
            setupTimerFrom(mVisit.actualStartTime().getTime());
            vTimerGroup.setVisibility(View.VISIBLE);
        } else {
            cancelTimer();
            vTimerGroup.setVisibility(View.GONE);
        }
    }



    /**********************************************************************************************/
    private Long timerReference;


    private void setupTimerFrom(long timestamp) {
        timerReference = timestamp;
        resumeTimerIfAttached();
    }

    private void cancelTimer() {
        timerReference = null;
        pauseTimer();
    }

    private void resumeTimerIfAttached() {
        if (isAttachedToWindow()) {
            post(mTimerRunnable);
        }
    }

    private void pauseTimer() {
        removeCallbacks(mTimerRunnable);
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        if (timerReference != null) {
            resumeTimerIfAttached();
        }
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        pauseTimer();
    }

    private final Runnable mTimerRunnable = new Runnable() {
        @Override
        public void run() {
            if (timerReference != null) {
                int ss = (int)((System.currentTimeMillis() - timerReference.longValue()) / 1000L);
                populateTimerFromSeconds(ss);
                postDelayed(this, 250);
            }
        }
    };

    @SuppressLint("SetTextI18n")
    private void populateTimerFromSeconds(int ss) {
        int mm = ss / 60;
        ss -= mm * 60;
        int hh = mm / 60;
        mm -= hh * 60;
        vTxtTimerHH.setText(String.format("%02d", hh));
        vTxtTimerMM.setText(String.format("%02d", mm));
        vTxtTimerSS.setText(String.format("%02d", ss));
    }
}
