package com.webssa.pda.sunmi.util;

import android.location.Location;
import android.text.TextUtils;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.webssa.pda.sunmi.GlobalProperties;
import com.webssa.pda.sunmi.R;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;

/**
 * Created by stanislav.perchenko on 11/6/2019, 1:13 PM.
 */
public final class DataUtils {

    private DataUtils() { }

    public static String getDiaplayAddress(SiteEntity site) {
        StringBuilder sb = new StringBuilder();
        sb.append(site.getAddressLine1());
        if (!TextUtils.isEmpty(site.getAddressLine2())) {
            sb.append(site.getAddressLine2());
        }
        if (!TextUtils.isEmpty(site.getTown())) {
            sb.append(", ");
            sb.append(site.getTown());
        }
        if (!TextUtils.isEmpty(site.getCity())) {
            sb.append(", ");
            sb.append(site.getCity());
        }
        sb.append(", ");
        sb.append(site.getPostCode());
        return sb.toString();
    }

    public static void populateVisitState(TextView tv, VisitState state) {
        int textId;
        int level;
        switch (state) {
            case NEW:
                level = 0;
                textId = R.string.job_sate_new;
                break;
            case IN_PROGRESS:
                level = 1;
                textId = R.string.job_state_in_process;
                break;
            case COMPLETED:
                level = 2;
                textId = R.string.job_state_completed;
                break;
            case FAILED:
                level = 3;
                textId = R.string.job_state_failed;
                break;
            default:
                tv.setVisibility(View.GONE);
                return;
        }
        tv.setVisibility(View.VISIBLE);
        tv.setText(textId);
        tv.getBackground().setLevel(level);
    }

    public static boolean checkDriverInPlace(@NonNull Location locTarget, @Nullable Location optLocActual) {
        if (optLocActual == null) {
            return false;
        } else {
            int dist = Math.round(locTarget.distanceTo(optLocActual));
            return (dist <= GlobalProperties.SITE_PROXIMITY_DISTANCE_METERS);
        }
    }

}
