package com.webssa.pda.sunmi.util;

import android.content.res.Resources;
import android.text.TextUtils;

import com.webssa.pda.sunmi.rest.model.LoginMode;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

public final class StringUtils {
    private StringUtils() {
        throw new IllegalStateException("No entities");
    }

    public static boolean validateLogin(String login, LoginMode loginMode) {
        switch (loginMode) {
            case BY_EMAIL:
                return validateEmail(login);
            case BY_PHONE_NUMBER:
                return !TextUtils.isEmpty(login);
            default:
                return false;
        }
    }

    public static boolean validatePassword(CharSequence pass) {
        return (pass != null) && (pass.length() >= 4);
    }

    public static String removeFirstSubstring(@NonNull String src, @NonNull String substring) {
        int index = src.indexOf(substring);
        if (index < 0) {
            throw new Resources.NotFoundException(String.format("The %s does not contain %s", src, substring));
        } else if (index == 0) {
            return src.substring(substring.length());
        } else if ((index + substring.length()) == src.length()) {
            return src.substring(0, index);
        } else {
            return src.substring(0, index) + src.substring(index+substring.length());
        }
    }

    public static boolean validateEmail(@Nullable String email) {
        return (email == null)
                ? false
                : email.matches("[A-Za-z0-9._%+-][A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,}");
    }

    public static boolean isSpacesOnly(CharSequence text) {
        for (int i=0; i<text.length(); i++) {
            if (text.charAt(i) != ' ') return false;
        }
        return true;
    }

    public static String trimTextEmptyIfNull(CharSequence text) {
        if (text == null) return "";
        int len = text.length();
        int st = 0;

        while ((st < len) && (text.charAt(st) <= ' ')) {
            st++;
        }
        while ((st < len) && (text.charAt(len - 1) <= ' ')) {
            len--;
        }
        String s = (text instanceof String) ? (String) text : text.toString();
        return ((st > 0) || (len < text.length())) ? s.substring(st, len) : s;
    }
}
