package com.webssa.pda.sunmi.util;

import android.os.Parcelable;

import androidx.annotation.NonNull;

import com.google.auto.value.AutoValue;

import java.text.ParseException;

@AutoValue
public abstract class TimeOfDayHMS implements Parcelable {

    public abstract int hours();
    public abstract int minutes();
    public abstract int seconds();

    public static TimeOfDayHMS parseTime(String time) throws ParseException {
        try {
            String[] parts = time.split(":");
            return builder()
                    .setHours(Integer.parseInt(parts[0]))
                    .setMinutes(Integer.parseInt(parts[1]))
                    .setSeconds(Integer.parseInt(parts[2]))
                    .build();
        } catch (Exception e) {
            throw new ParseException("Error parsing time-of-day. Must be hh:mm:ss. Cause - "+e.getMessage(), 0);
        }
    }

    public static Builder builder() {
        return new AutoValue_TimeOfDayHMS.Builder();
    }

    @AutoValue.Builder
    public abstract static class Builder {
        public abstract Builder setHours(int hours);
        public abstract Builder setMinutes(int minutes);
        public abstract Builder setSeconds(int seconds);

        abstract TimeOfDayHMS actualBuild();

        public TimeOfDayHMS build() {
            final TimeOfDayHMS ent = actualBuild();
            if (ent.hours() < 0 || ent.hours() > 23) {
                throw new IllegalStateException("Invalid hours value - " + ent.hours());
            } else if (ent.minutes() < 0 || ent.minutes() > 59) {
                throw new IllegalStateException("Invalid minutes value - " + ent.minutes());
            } else if (ent.seconds() < 0 || ent.seconds() > 59) {
                throw new IllegalStateException("Invalid seconds value - " + ent.seconds());
            }
            return ent;
        }
    }

    private String cachedTextForm;

    @NonNull
    @Override
    public String toString() {
        if (cachedTextForm == null) {
            synchronized (this) {
                if (cachedTextForm == null) {
                    cachedTextForm = String.format("%02d:%02d:%02d", hours(), minutes(), seconds());
                }
            }
        }
        return cachedTextForm;
    }

    public String toStringHM() {
        return String.format("%02d:%02d", hours(), minutes());
    }

    public int getTimeMillis() {
        return (hours()*3600 + minutes()*60 + seconds())*1000;
    }


}
