package com.webssa.pda.sunmi;

import com.squareup.moshi.Moshi;
import com.squareup.moshi.Types;
import com.squareup.moshi.kotlin.reflect.KotlinJsonAdapterFactory;
import com.webssa.pda.sunmi.rest.helper.ParserProvider;
import com.webssa.pda.sunmi.rest.jsonadapter.DateJsonAdapter;
import com.webssa.pda.sunmi.rest.jsonadapter.DateNoTimeJsonAdapter;
import com.webssa.pda.sunmi.rest.jsonadapter.DurationHMSJsonAdapter;
import com.webssa.pda.sunmi.rest.jsonadapter.LongIdJsonAdapter;
import com.webssa.pda.sunmi.rest.jsonadapter.TimeOfDayHMSJsonAdapter;
import com.webssa.pda.sunmi.rest.model.LongId;
import com.webssa.pda.sunmi.rest.model.ShiftState;
import com.webssa.pda.sunmi.rest.model.VisitState;
import com.webssa.pda.sunmi.rest.model.impl.CollectedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.EmergencyCategoryEntity;
import com.webssa.pda.sunmi.rest.model.impl.NoteEntity;
import com.webssa.pda.sunmi.rest.model.impl.PriceEntity;
import com.webssa.pda.sunmi.rest.model.impl.PricedGoodEntity;
import com.webssa.pda.sunmi.rest.model.impl.ShiftRestEntity;
import com.webssa.pda.sunmi.rest.model.impl.SiteEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitEntity;
import com.webssa.pda.sunmi.rest.model.impl.VisitStateEntity;

import org.junit.Test;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;

import static com.google.common.truth.Truth.assertThat;

public class ParsersCompositionTest {

    private static final String SHIFT_ENTITY_JSON = "{" +
            "\"shift_id\": 482," +
            "\"driver_id\": \"6b01ef7e-9bbe-4cab-8efb-820aad98d0b7\"," +
            "\"execution_date\": \"2021-05-12\"," +
            "\"scheduled_time_start\": \"08:00:00\"," +
            "\"scheduled_duration\": \"06:30:00\"," +
            "\"num_visits\": 12," +
            "\"num_visits_completed\": 8," +
            "\"num_visits_failed\": 1," +
            "\"actual_time_start\": \"2001-05-12T07:20:47.245+0100\"," +
            "\"actual_time_closed\": null," +
            "\"state\": \"accepted\"" +
            "}";

    private static final String PRICED_GOOD_ENTITY_JSON = "{" +
            "\"goodcategory_id\": 3," +
            "\"name\": \"Shoes\"," +
            "\"price_id\": 2457," +
            "\"price_value\": 31255," +
            "\"price_scale\": 2," +
            "\"price_time_set\": \"2019-05-12T07:20:47.245+0100\"" +
            "}";

    private static final String SITE_ENTITY_JSON = "{" +
            "\"site_id\": 41," +
            "\"name\": \"Maruthy\"," +
            "\"code\": 327," +
            "\"address\": \"324 Wickham Ln, London SE2 0NZ, Великобритания\"," +
            "\"latitude\": 51.477897," +
            "\"longitude\": 0.105669," +
            "\"uuid\": \"81a04df8-1281-4bc5-8a98-e9a2970312cc\"" +
            "}";

    private static final String NOTE_ENTITY_JSON_1 = "{" +
            "\"note_id\": 8726," +
            "\"shift_id\": 482," +
            "\"visit_id\": 5248," +
            "\"number\": 56," +
            "\"author_id\": \"6b01ef7e-9bbe-4cab-8efb-820aad98d0b7\"," +
            "\"time_created\": \"2019-08-15T13:47:12.123+0100\"," +
            "\"text\": \"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nunc pulvinar a neque eu pellentesque. Vivamus quis ipsum at velit bibendum mollis id id ex. Morbi pulvinar ex et lorem convallis mollis.\"," +
            "\"images\": [\"https://www.ifjheg.com/hdjfe/gidfg.png\", \"http://djfhsdfgsd.fhsdofhsdf.com/dfusdofsd/iufgi.jpg\"]," +
            "\"opt_emergency_id\": 4" +
            "}";

    private static final String NOTE_ENTITY_JSON_2 = "{" +
            "\"note_id\": 8726," +
            "\"shift_id\": 482," +
            "\"visit_id\": null," +
            "\"number\": 56," +
            "\"author_id\": \"6b01ef7e-9bbe-4cab-8efb-820aad98d0b7\"," +
            "\"time_created\": \"2019-08-15T13:47:12.123+0100\"," +
            "\"text\": \"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nunc pulvinar a neque eu pellentesque. Vivamus quis ipsum at velit bibendum mollis id id ex. Morbi pulvinar ex et lorem convallis mollis.\"," +
            "\"images\": [\"https://www.ifjheg.com/hdjfe/gidfg.png\"]," +
            "\"opt_emergency_id\": null" +
            "}";

    private static final String NOTE_ENTITY_JSON_3 = "{" +
            "\"note_id\": 8726," +
            "\"shift_id\": 482," +
            "\"number\": 56," +
            "\"author_id\": \"6b01ef7e-9bbe-4cab-8efb-820aad98d0b7\"," +
            "\"time_created\": \"2019-08-15T13:47:12.123+0100\"," +
            "\"text\": \"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nunc pulvinar a neque eu pellentesque. Vivamus quis ipsum at velit bibendum mollis id id ex. Morbi pulvinar ex et lorem convallis mollis.\"," +
            "\"images\": []" +
            "}";

    private static final String EMERGENCY_ENTITY_JSON = "{" +
            "\"emergency_id\": 7," +
            "\"title\": \"Light car accident\"" +
            "}";


    private static final String VISIT_STATE_ENTITY_JSON_1 = "{" +
            "\"visit_id\": 5248," +
            "\"state\": \"new\"," +
            "\"complete_time\": null" +
            "}";

    private static final String VISIT_STATE_ENTITY_JSON_2 = "{" +
            "\"visit_id\": 5248," +
            "\"state\": \"in_progress\"," +
            "\"start_time\": \"2019-08-15T13:47:12.123+0100\"," +
            "\"complete_time\": null" +
            "}";

    private static final String VISIT_STATE_ENTITY_JSON_3 = "{" +
            "\"visit_id\": 5248," +
            "\"state\": \"completed\"," +
            "\"start_time\": \"2019-08-15T13:47:12.123+0100\"," +
            "\"complete_time\": \"2019-08-15T14:03:22.457+0100\"" +
            "}";

    private static final String COLLECTED_GOOD_ENTITY_JSON = "{" +
            "\"visit_id\": 2546," +
            "\"goodcategory_id\": 8," +
            "\"name\": \"Books\"," +
            "\"weight\": 47," +
            "\"total_amount\": 875," +
            "\"total_scale\": 2," +
            "\"order\": 4" +
            "}";


    private static final String VISIT_ENTITY_JSON_1 = "{" +
            "\"visit_id\": 2546," +
            "\"shift_id\": 854," +
            "\"site_id\": 173," +
            "\"order\": 6," +
            "\"state\": \"new\"," +
            "\"suggest_time\": null," +
            "\"start_time\": null," +
            "\"complete_time\": null," +
            "\"customer_name\": null," +
            "\"customer_signature\": null," +
            "\"total_payed_amount\": 0," +
            "\"total_payed_scale\": 0" +
            "}";

    private static final String VISIT_ENTITY_JSON_2 = "{" +
            "\"visit_id\": 2546," +
            "\"shift_id\": 854," +
            "\"site_id\": 173," +
            "\"order\": 6," +
            "\"state\": \"new\"," +
            "\"total_payed_amount\": 0," +
            "\"total_payed_scale\": 0" +
            "}";

    private static final String VISIT_ENTITY_JSON_3 = "{" +
            "\"visit_id\": 2546," +
            "\"shift_id\": 854," +
            "\"site_id\": 173," +
            "\"order\": 6," +
            "\"state\": \"in_progress\"," +
            "\"suggest_time\": \"14:35:00\"," +
            "\"start_time\": \"2019-09-15T14:32:12.123+001\"," +
            "\"total_payed_amount\": 0," +
            "\"total_payed_scale\": 0" +
            "}";

    private static final String VISIT_ENTITY_JSON_4 = "{" +
            "\"visit_id\": 2546," +
            "\"shift_id\": 854," +
            "\"site_id\": 173," +
            "\"order\": 6," +
            "\"state\": \"completed\"," +
            "\"suggest_time\": \"14:35:00\"," +
            "\"start_time\": \"2019-09-15T14:32:12.123+001\"," +
            "\"complete_time\": \"2019-09-15T14:74:22.549+001\"," +
            "\"customer_name\": \"Sally May\"," +
            "\"customer_signature\": \"ZGpoZzc4cnQgM3JnNzlydCA5cWdydXcgZ2R2cWcgeTU0dC0wOC1yIDg5cigmJTY3ZQ==\"," +
            "\"total_payed_amount\": 24579," +
            "\"total_payed_scale\": 2" +
            "}";

    @Test
    public void parseVisitEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(DateJsonAdapter.FACTORY)
                .add(TimeOfDayHMSJsonAdapter.FACTORY)
                .add(Types.newParameterizedType(LongId.class, SiteEntity.class), new LongIdJsonAdapter<SiteEntity>())
                .add(Types.newParameterizedType(LongId.class, VisitEntity.class), new LongIdJsonAdapter<VisitEntity>())
                .add(Types.newParameterizedType(LongId.class, ShiftRestEntity.class), new LongIdJsonAdapter<ShiftRestEntity>())
                .add(new KotlinJsonAdapterFactory())
                .build();
        checkVisitEntity_minimal(m.adapter(VisitEntity.class).fromJson(VISIT_ENTITY_JSON_1));
        checkVisitEntity_minimal(m.adapter(VisitEntity.class).fromJson(VISIT_ENTITY_JSON_2));
        checkVisitEntity_3(m);
        checkVisitEntity_4(m);
    }

    @Test
    public void parseVisitEntity_onApplicationMoshi() throws IOException {
        Moshi m = ParserProvider.getApplicationMoshi();
        checkVisitEntity_minimal(m.adapter(VisitEntity.class).fromJson(VISIT_ENTITY_JSON_1));
        checkVisitEntity_minimal(m.adapter(VisitEntity.class).fromJson(VISIT_ENTITY_JSON_2));
        checkVisitEntity_3(m);
        checkVisitEntity_4(m);
    }

    private void checkVisitEntity_minimal(VisitEntity ent) {
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(2546);
        assertThat(ent.getShiftId()).isNotNull();
        assertThat(ent.getShiftId().getValue()).isEqualTo(854);
        assertThat(ent.getSiteId()).isNotNull();
        assertThat(ent.getSiteId().getValue()).isEqualTo(173);
        assertThat(ent.getOrder()).isEqualTo(6);
        assertThat(ent.getState()).isNotNull();
        assertThat(ent.getState()).isEqualTo(VisitState.NEW);
        assertThat(ent.getSuggestedTime()).isNull();
        assertThat(ent.getActualStartTime()).isNull();
        assertThat(ent.getActualCompleteTime()).isNull();
        assertThat(ent.getCustomerName()).isNull();
        assertThat(ent.getCustomerSignature()).isNull();
        assertThat(ent.getTotalPayedAmount()).isEqualTo(0);
        assertThat(ent.getTotalPayedScale()).isEqualTo(0);
    }

    private void checkVisitEntity_3(Moshi m) throws IOException {
        VisitEntity ent = m.adapter(VisitEntity.class).fromJson(VISIT_ENTITY_JSON_3);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(2546);
        assertThat(ent.getShiftId()).isNotNull();
        assertThat(ent.getShiftId().getValue()).isEqualTo(854);
        assertThat(ent.getSiteId()).isNotNull();
        assertThat(ent.getSiteId().getValue()).isEqualTo(173);
        assertThat(ent.getOrder()).isEqualTo(6);
        assertThat(ent.getState()).isNotNull();
        assertThat(ent.getState()).isEqualTo(VisitState.IN_PROGRESS);
        assertThat(ent.getSuggestedTime()).isNotNull();
        assertThat(ent.getSuggestedTime().toString()).isEqualTo("14:35:00");
        assertThat(ent.getActualStartTime()).isNotNull();
        assertThat(ent.getActualStartTime().getTime()).isEqualTo(1568547132123L);
        assertThat(ent.getActualCompleteTime()).isNull();
        assertThat(ent.getCustomerName()).isNull();
        assertThat(ent.getCustomerSignature()).isNull();
        assertThat(ent.getTotalPayedAmount()).isEqualTo(0);
        assertThat(ent.getTotalPayedScale()).isEqualTo(0);
    }

    private void checkVisitEntity_4(Moshi m) throws IOException {
        VisitEntity ent = m.adapter(VisitEntity.class).fromJson(VISIT_ENTITY_JSON_4);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(2546);
        assertThat(ent.getShiftId()).isNotNull();
        assertThat(ent.getShiftId().getValue()).isEqualTo(854);
        assertThat(ent.getSiteId()).isNotNull();
        assertThat(ent.getSiteId().getValue()).isEqualTo(173);
        assertThat(ent.getOrder()).isEqualTo(6);
        assertThat(ent.getState()).isNotNull();
        assertThat(ent.getState()).isEqualTo(VisitState.COMPLETED);
        assertThat(ent.getSuggestedTime()).isNotNull();
        assertThat(ent.getSuggestedTime().toString()).isEqualTo("14:35:00");
        assertThat(ent.getActualStartTime()).isNotNull();
        assertThat(ent.getActualStartTime().getTime()).isEqualTo(1568547132123L);
        assertThat(ent.getActualCompleteTime()).isNotNull();
        assertThat(ent.getActualCompleteTime().getTime()).isEqualTo(1568549662549L);
        assertThat(ent.getCustomerName()).isEqualTo("Sally May");
        assertThat(ent.getCustomerSignature()).isEqualTo("ZGpoZzc4cnQgM3JnNzlydCA5cWdydXcgZ2R2cWcgeTU0dC0wOC1yIDg5cigmJTY3ZQ==");
        assertThat(ent.getTotalPayedAmount()).isEqualTo(24579);
        assertThat(ent.getTotalPayedScale()).isEqualTo(2);
    }


    @Test
    public void parseCollectedGoodEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(DateJsonAdapter.FACTORY)
                .add(Types.newParameterizedType(LongId.class, VisitEntity.class), new LongIdJsonAdapter<VisitEntity>())
                .add(Types.newParameterizedType(LongId.class, PricedGoodEntity.class), new LongIdJsonAdapter<PricedGoodEntity>())
                .add(new KotlinJsonAdapterFactory())
                .build();
        checkCollectedGoodEntity(m);
    }

    @Test
    public void parseCollectedGoodEntity_onApplicationMoshi() throws IOException {
        checkCollectedGoodEntity(ParserProvider.getApplicationMoshi());
    }

    private void checkCollectedGoodEntity(Moshi m) throws IOException {
        CollectedGoodEntity ent = m.adapter(CollectedGoodEntity.class).fromJson(COLLECTED_GOOD_ENTITY_JSON);

        assertThat(ent).isNotNull();
        assertThat(ent.getVisitId()).isNotNull();
        assertThat(ent.getVisitId().getValue()).isEqualTo(2546);
        assertThat(ent.getGoodcategoryId()).isNotNull();
        assertThat(ent.getGoodcategoryId().getValue()).isEqualTo(8);
        assertThat(ent.getGoodName()).isEqualTo("Books");
        assertThat(ent.getCollectedWeight()).isEqualTo(47);
        assertThat(ent.getCollectedTotalAmount()).isEqualTo(875);
        assertThat(ent.getCollectedTotalScale()).isEqualTo(2);
        assertThat(ent.getOrder()).isEqualTo(4);
    }

    @Test
    public void parseVisitStateEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(Types.newParameterizedType(LongId.class, VisitEntity.class), new LongIdJsonAdapter<VisitEntity>())
                .add(DateJsonAdapter.FACTORY)
                .add(new KotlinJsonAdapterFactory())
                .build();
        checkVisitStateEntity_1(m);
        checkVisitStateEntity_2(m);
        checkVisitStateEntity_3(m);
    }

    @Test
    public void parseVisitStateEntity_onApplicationMoshi() throws IOException {
        Moshi m = ParserProvider.getApplicationMoshi();
        checkVisitStateEntity_1(m);
        checkVisitStateEntity_2(m);
        checkVisitStateEntity_3(m);
    }

    private void checkVisitStateEntity_1(Moshi m) throws IOException {
        VisitStateEntity ent = m.adapter(VisitStateEntity.class).fromJson(VISIT_STATE_ENTITY_JSON_1);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(5248);
        assertThat(ent.getState()).isNotNull();
        assertThat(ent.getState()).isEqualTo(VisitState.NEW);
        assertThat(ent.getActualStartTime()).isNull();
        assertThat(ent.getActualCompleteTime()).isNull();
    }

    private void checkVisitStateEntity_2(Moshi m) throws IOException {
        VisitStateEntity ent = m.adapter(VisitStateEntity.class).fromJson(VISIT_STATE_ENTITY_JSON_2);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(5248);
        assertThat(ent.getState()).isNotNull();
        assertThat(ent.getState()).isEqualTo(VisitState.IN_PROGRESS);
        assertThat(ent.getActualStartTime()).isNotNull();
        assertThat(ent.getActualStartTime().getTime()).isEqualTo(1565873232123L);
        assertThat(ent.getActualCompleteTime()).isNull();
    }

    private void checkVisitStateEntity_3(Moshi m) throws IOException {
        VisitStateEntity ent = m.adapter(VisitStateEntity.class).fromJson(VISIT_STATE_ENTITY_JSON_3);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(5248);
        assertThat(ent.getState()).isNotNull();
        assertThat(ent.getState()).isEqualTo(VisitState.COMPLETED);
        assertThat(ent.getActualStartTime()).isNotNull();
        assertThat(ent.getActualStartTime().getTime()).isEqualTo(1565873232123L);
        assertThat(ent.getActualCompleteTime()).isNotNull();
        assertThat(ent.getActualCompleteTime().getTime()).isEqualTo(1565874202457L);
    }

    @Test
    public void parseEmergencyEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(Types.newParameterizedType(LongId.class, EmergencyCategoryEntity.class), new LongIdJsonAdapter<EmergencyCategoryEntity>())
                .add(new KotlinJsonAdapterFactory())
                .build();
        checkEmergencyEntity(m);
    }

    @Test
    public void parseEmergencyEntity_onApplicationMoshi() throws IOException {
        checkEmergencyEntity(ParserProvider.getApplicationMoshi());
    }

    private void checkEmergencyEntity(Moshi m) throws IOException {
        EmergencyCategoryEntity ent = m.adapter(EmergencyCategoryEntity.class).fromJson(EMERGENCY_ENTITY_JSON);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(7);
        assertThat(ent.getTitle()).isEqualTo("Light car accident");
    }

    @Test
    public void parseNoteEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(DateJsonAdapter.FACTORY)
                .add(Types.newParameterizedType(LongId.class, NoteEntity.class), new LongIdJsonAdapter<NoteEntity>())
                .add(Types.newParameterizedType(LongId.class, VisitEntity.class), new LongIdJsonAdapter<VisitEntity>())
                .add(Types.newParameterizedType(LongId.class, ShiftRestEntity.class), new LongIdJsonAdapter<ShiftRestEntity>())
                .add(Types.newParameterizedType(LongId.class, EmergencyCategoryEntity.class), new LongIdJsonAdapter<EmergencyCategoryEntity>())
                .add(new KotlinJsonAdapterFactory())
                .build();

        checkNoteEntity_1(m);
        checkNoteEntity_2(m);
        checkNoteEntity_3(m);
    }

    @Test
    public void parseNoteEntity_onApplicationMoshi() throws IOException {
        Moshi m = ParserProvider.getApplicationMoshi();
        checkNoteEntity_1(m);
        checkNoteEntity_2(m);
        checkNoteEntity_3(m);
    }

    private void checkNoteEntity_1(Moshi m) throws IOException {
        NoteEntity ent = m.adapter(NoteEntity.class).fromJson(NOTE_ENTITY_JSON_1);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(8726);
        assertThat(ent.getShiftId()).isNotNull();
        assertThat(ent.getShiftId().getValue()).isEqualTo(482);
        assertThat(ent.getVisitId()).isNotNull();
        assertThat(ent.getVisitId().getValue()).isEqualTo(5248);
        assertThat(ent.getSequenceNumber()).isEqualTo(56);
        assertThat(ent.getAuthorId()).isEqualTo("6b01ef7e-9bbe-4cab-8efb-820aad98d0b7");
        assertThat(ent.getTimeCreated()).isNotNull();
        assertThat(ent.getTimeCreated().getTime()).isEqualTo(1565873232123L);
        assertThat(ent.getText()).isEqualTo("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nunc pulvinar a neque eu pellentesque. Vivamus quis ipsum at velit bibendum mollis id id ex. Morbi pulvinar ex et lorem convallis mollis.");


        assertThat(ent.getImagesUrlSet()).isNotNull();
        assertThat(ent.getImagesUrlSet()).isEqualTo(new HashSet<>(Arrays.asList("https://www.ifjheg.com/hdjfe/gidfg.png", "http://djfhsdfgsd.fhsdofhsdf.com/dfusdofsd/iufgi.jpg")));

        assertThat(ent.getOptEmergencyId()).isNotNull();
        assertThat(ent.getOptEmergencyId().getValue()).isEqualTo(4);
    }

    private void checkNoteEntity_2(Moshi m) throws IOException {
        NoteEntity ent = m.adapter(NoteEntity.class).fromJson(NOTE_ENTITY_JSON_2);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(8726);
        assertThat(ent.getShiftId()).isNotNull();
        assertThat(ent.getShiftId().getValue()).isEqualTo(482);
        assertThat(ent.getVisitId()).isNull();
        assertThat(ent.getSequenceNumber()).isEqualTo(56);
        assertThat(ent.getAuthorId()).isEqualTo("6b01ef7e-9bbe-4cab-8efb-820aad98d0b7");
        assertThat(ent.getTimeCreated()).isNotNull();
        assertThat(ent.getTimeCreated().getTime()).isEqualTo(1565873232123L);
        assertThat(ent.getText()).isEqualTo("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nunc pulvinar a neque eu pellentesque. Vivamus quis ipsum at velit bibendum mollis id id ex. Morbi pulvinar ex et lorem convallis mollis.");


        assertThat(ent.getImagesUrlSet()).isNotNull();
        assertThat(ent.getImagesUrlSet()).isEqualTo(new HashSet<>(Arrays.asList("https://www.ifjheg.com/hdjfe/gidfg.png")));

        assertThat(ent.getOptEmergencyId()).isNull();
    }

    private void checkNoteEntity_3(Moshi m) throws IOException {
        NoteEntity ent = m.adapter(NoteEntity.class).fromJson(NOTE_ENTITY_JSON_3);
        assertThat(ent).isNotNull();
        assertThat(ent.id()).isNotNull();
        assertThat(ent.id().getValue()).isEqualTo(8726);
        assertThat(ent.getShiftId()).isNotNull();
        assertThat(ent.getShiftId().getValue()).isEqualTo(482);
        assertThat(ent.getVisitId()).isNull();
        assertThat(ent.getSequenceNumber()).isEqualTo(56);
        assertThat(ent.getAuthorId()).isEqualTo("6b01ef7e-9bbe-4cab-8efb-820aad98d0b7");
        assertThat(ent.getTimeCreated()).isNotNull();
        assertThat(ent.getTimeCreated().getTime()).isEqualTo(1565873232123L);
        assertThat(ent.getText()).isEqualTo("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nunc pulvinar a neque eu pellentesque. Vivamus quis ipsum at velit bibendum mollis id id ex. Morbi pulvinar ex et lorem convallis mollis.");
        assertThat(ent.getImagesUrlSet()).isNotNull();
        assertThat(ent.getImagesUrlSet()).isEmpty();
        assertThat(ent.getOptEmergencyId()).isNull();
    }





    @Test
    public void parseSiteEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(Types.newParameterizedType(LongId.class, SiteEntity.class), new LongIdJsonAdapter<SiteEntity>())
                .add(new KotlinJsonAdapterFactory())
                .build();

        checkSiteEntity(m);
    }

    @Test
    public void parseSiteEntity_onApplicationMoshi() throws IOException {
        checkSiteEntity(ParserProvider.getApplicationMoshi());
    }

    private void checkSiteEntity(Moshi m) throws IOException {
        SiteEntity entity = m.adapter(SiteEntity.class).fromJson(SITE_ENTITY_JSON);
        assertThat(entity).isNotNull();
        assertThat(entity.id()).isNotNull();
        assertThat(entity.id().getValue()).isEqualTo(41);
        assertThat(entity.getSiteName()).isEqualTo("Maruthy");
        assertThat(entity.getCode()).isEqualTo(327);
        assertThat(entity.getAddressLine1()).isEqualTo("324 Wickham Ln, London SE2 0NZ, Великобритания");
        assertThat(Math.abs(entity.getLatitude() - 51.477897)).isLessThan(0.000001);
        assertThat(Math.abs(entity.getLongitude() - 0.105669)).isLessThan(0.000001);
        assertThat(entity.getUuid()).isEqualTo("81a04df8-1281-4bc5-8a98-e9a2970312cc");
    }

    @Test
    public void parsePricedGoodEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(Types.newParameterizedType(LongId.class, PriceEntity.class), new LongIdJsonAdapter<PriceEntity>())
                .add(Types.newParameterizedType(LongId.class, PricedGoodEntity.class), new LongIdJsonAdapter<PricedGoodEntity>())
                .add(DateJsonAdapter.FACTORY)
                .add(new KotlinJsonAdapterFactory())
                .build();

        checkPricedGoodEntity(m);
    }

    @Test
    public void parsePricedGoodEntity_onApplicationMoshi() throws IOException {
        checkPricedGoodEntity(ParserProvider.getApplicationMoshi());
    }

    private void checkPricedGoodEntity(Moshi m) throws IOException {
        PricedGoodEntity entity = m.adapter(PricedGoodEntity.class).fromJson(PRICED_GOOD_ENTITY_JSON);
        assertThat(entity).isNotNull();
        assertThat(entity.id()).isNotNull();
        assertThat(entity.id().getValue()).isEqualTo(3);
        assertThat(entity.getCategoryName()).isEqualTo("Shoes");
        assertThat(entity.getPriceId()).isNotNull();
        assertThat(entity.getPriceId().getValue()).isEqualTo(2457);
        assertThat(entity.getPriceValue()).isEqualTo(31255);
        assertThat(entity.getPriceScale()).isEqualTo(2);
        assertThat(entity.getPriceSetDate()).isNotNull();
        assertThat(entity.getPriceSetDate().getTime()).isEqualTo(1557642047245L);
    }

    @Test
    public void parseShiftEntity_onLocalMoshi() throws IOException {
        Moshi m = new Moshi.Builder()
                .add(Types.newParameterizedType(LongId.class, ShiftRestEntity.class), new LongIdJsonAdapter<ShiftRestEntity>())
                .add(DateJsonAdapter.FACTORY)
                .add(DateNoTimeJsonAdapter.FACTORY)
                .add(TimeOfDayHMSJsonAdapter.FACTORY)
                .add(DurationHMSJsonAdapter.FACTORY)
                .add(new KotlinJsonAdapterFactory())
                .build();

        checkShiftEntityParse(m);
    }

    @Test
    public void parseShiftEntity_onApplicationMoshi() throws IOException {
        checkShiftEntityParse(ParserProvider.getApplicationMoshi());
    }

    private void checkShiftEntityParse(Moshi m) throws IOException {
        ShiftRestEntity entity = m.adapter(ShiftRestEntity.class).fromJson(SHIFT_ENTITY_JSON);
        assertThat(entity).isNotNull();
        assertThat(entity.id()).isNotNull();
        assertThat(entity.id().getValue()).isEqualTo(482);
        assertThat(entity.getDriverId()).isEqualTo("6b01ef7e-9bbe-4cab-8efb-820aad98d0b7");
        assertThat(entity.getExecutionDate()).isNotNull();
        assertThat(entity.getExecutionDate().toString()).isEqualTo("2021-05-12");
        assertThat(entity.getScheduledTimeStart()).isNotNull();
        assertThat(entity.getScheduledTimeStart().toString()).isEqualTo("08:00:00");
        assertThat(entity.getScheduledDuration()).isNotNull();
        assertThat(entity.getScheduledDuration().toString()).isEqualTo("6:30:00");
        assertThat(entity.getNVisits()).isEqualTo(12);
        assertThat(entity.getNCompletedVisits()).isEqualTo(8);
        assertThat(entity.getNFailedVisits()).isEqualTo(1);
        assertThat(entity.getActualTimeStart()).isNotNull();
        assertThat(entity.getActualTimeStart().getTime()).isEqualTo(989648447245L);
        assertThat(entity.getActualTimeClosed()).isNull();
        assertThat(entity.getState()).isNotNull();
        assertThat(entity.getState()).isEqualTo(ShiftState.ACCEPTED);
    }
}
