package com.webssa.pda.sunmi;

import com.webssa.pda.sunmi.rest.jsonadapter.DateJsonAdapter;
import com.webssa.pda.sunmi.rest.jsonadapter.DateNoTimeJsonAdapter;
import com.webssa.pda.sunmi.rest.jsonadapter.DurationHMSJsonAdapter;
import com.webssa.pda.sunmi.rest.jsonadapter.TimeOfDayHMSJsonAdapter;
import com.webssa.pda.sunmi.util.DateNoTime;
import com.webssa.pda.sunmi.util.DurationHMS;
import com.webssa.pda.sunmi.util.TimeOfDayHMS;

import org.junit.Assert;
import org.junit.Test;

import java.io.IOException;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;

import static com.google.common.truth.Truth.assertThat;

public final class ParsersLowLevelTest {



    @Test
    public void parseDurationHMS_onAdapter() {
        DurationHMSJsonAdapter adapter = (DurationHMSJsonAdapter) DurationHMSJsonAdapter.FACTORY.create(DurationHMS.class, null, null);
        assertThat(adapter).isNotNull();

        checkDurationHMS(adapter, "11:05:47", 11, 5, 47);
        checkDurationHMS(adapter, "3:11:07", 3, 11, 7);
        checkDurationHMS(adapter, "3:1:5", 3, 1, 5);
        checkDurationHMS(adapter, "327:21:55", 327, 21, 55);
        checkDurationHMS_failure(adapter, "12:11:64", "Error parsing Duration", "Must be hh...h:mm:ss", "Invalid seconds value - 64");
        checkDurationHMS_failure(adapter, "25:68:99", "Error parsing Duration", "Must be hh...h:mm:ss", "Invalid minutes value - 68");
        checkDurationHMS_failure(adapter, "08:-11:99", "Error parsing Duration", "Must be hh...h:mm:ss", "Invalid minutes value - -11");
        checkDurationHMS_failure(adapter, "-12:47:99", "Error parsing Duration", "Must be hh...h:mm:ss", "Invalid hours value - -12");

        assertThat(adapter.fromJsonValue("11:05:47").toString()).isEqualTo("11:05:47");
        assertThat(adapter.fromJsonValue("327:21:55").toString()).isEqualTo("327:21:55");
    }

    private void checkDurationHMS(DurationHMSJsonAdapter adapter, String text, int hh, int mm, int ss) {
        DurationHMS dur = adapter.fromJsonValue(text);
        assertThat(dur).isNotNull();
        assertThat(dur.hours()).isEqualTo(hh);
        assertThat(dur.minutes()).isEqualTo(mm);
        assertThat(dur.seconds()).isEqualTo(ss);
    }

    private void checkDurationHMS_failure(DurationHMSJsonAdapter adapter, String text, String... exMessages) {
        try {
            adapter.fromJsonValue(text);
            Assert.fail("This test should throw an IOException");
        } catch (Throwable e) {
            assertThat(e.getCause().getCause()).isInstanceOf(ParseException.class);
            String to_check = e.getCause().getCause().getMessage();
            for (String msg : exMessages) {
                assertThat(to_check).contains(msg);
            }
        }
    }

    @Test
    public void parseTimeOfDayHMS_onAdapter() {
        TimeOfDayHMSJsonAdapter adapter = (TimeOfDayHMSJsonAdapter) TimeOfDayHMSJsonAdapter.FACTORY.create(TimeOfDayHMS.class, null, null);
        assertThat(adapter).isNotNull();

        checkTimeOfDayHMS(adapter, "11:05:47", 11, 5, 47);
        checkTimeOfDayHMS(adapter, "3:11:07", 3, 11, 7);
        checkTimeOfDayHMS(adapter, "3:1:5", 3, 1, 5);
        checkTimeOfDayHMS_failure(adapter, "3:11:64", "Error parsing time-of-day", "Must be hh:mm:ss", "Invalid seconds value - 64");
        checkTimeOfDayHMS_failure(adapter, "25:11:99", "Error parsing time-of-day", "Must be hh:mm:ss", "Invalid hours value - 25");
        checkTimeOfDayHMS_failure(adapter, "08:-11:99", "Error parsing time-of-day", "Must be hh:mm:ss", "Invalid minutes value - -11");
        checkTimeOfDayHMS_failure(adapter, "13:120:99", "Error parsing time-of-day", "Must be hh:mm:ss", "Invalid minutes value - 120");

        assertThat(adapter.fromJsonValue("11:05:47").toString()).isEqualTo("11:05:47");
    }

    private void checkTimeOfDayHMS_failure(TimeOfDayHMSJsonAdapter adapter, String text, String... exMessages) {
        try {
            adapter.fromJsonValue(text);
            Assert.fail("This test should throw an IOException");
        } catch (Throwable e) {
            assertThat(e.getCause().getCause()).isInstanceOf(ParseException.class);
            String to_check = e.getCause().getCause().getMessage();
            for (String msg : exMessages) {
                assertThat(to_check).contains(msg);
            }
        }
    }

    private void checkTimeOfDayHMS(TimeOfDayHMSJsonAdapter adapter, String text, int hh, int mm, int ss) {
        TimeOfDayHMS tod = adapter.fromJsonValue(text);
        assertThat(tod).isNotNull();
        //assertThat(tod.toString()).isEqualTo(text);
        assertThat(tod.hours()).isEqualTo(hh);
        assertThat(tod.minutes()).isEqualTo(mm);
        assertThat(tod.seconds()).isEqualTo(ss);
    }

    @Test
    public void parseDateNoTime_onAdapter() throws IOException {
        DateNoTimeJsonAdapter adapter = (DateNoTimeJsonAdapter) DateNoTimeJsonAdapter.FACTORY.create(DateNoTime.class, null, null);
        assertThat(adapter).isNotNull();

        checkDateNoTime(adapter, "2019-05-12", 2019, 5, 12);
        checkDateNoTime(adapter, "2087-11-28", 2087, 11, 28);
        checkDateNoTime(adapter, "2119-01-03", 2119, 1, 3);

        try {
            checkDateNoTime(adapter, "3754-05-12", 3754, 5, 12);
            Assert.fail("This test should throw an IOException");
        } catch (Throwable e) {
            assertThat(e.getCause().getCause()).isInstanceOf(ParseException.class);
            assertThat(e.getCause().getCause()).hasMessageThat().contains("Invalid years value - 3754. Allowed range - [2019, 2119]");
        }
    }

    private void checkDateNoTime(DateNoTimeJsonAdapter adapter, String text, int y, int m, int d) throws IOException {
        DateNoTime dnt = adapter.fromJsonValue(text);
        assertThat(dnt).isNotNull();
        assertThat(dnt.toString()).isEqualTo(text);
        assertThat(dnt.years()).isEqualTo(y);
        assertThat(dnt.months()).isEqualTo(m);
        assertThat(dnt.days()).isEqualTo(d);
    }

    @Test
    public void parseDate_onAdapter() {
        DateJsonAdapter adapter = new DateJsonAdapter();
        checkDate(adapter, "2001-05-12T07:20:47.245+0100", 989648447245L);
        checkDate(adapter, "1983-11-03T22:05:12.127+0300", 436734312127L);
        checkDate(adapter, "2113-08-27T07:20:47.245-0700", 4533286847245L);
        checkDate(adapter, "2113-08-27T22:20:47.245-0700", 4533340847245L);

        checkDate(adapter, "2001-05-12T07:20:47.245+01:00", 989648447245L);
        checkDate(adapter, "1983-11-03T22:05:12.127+03:00", 436734312127L);
        checkDate(adapter, "2113-08-27T07:20:47.245-07:00", 4533286847245L);
        checkDate(adapter, "2113-08-27T22:20:47.245-07:00", 4533340847245L);

        checkDate(adapter, "2001-05-12T07:20:47.245", 2001, 5, 12, 7, 20, 47, 245);
        checkDate(adapter, "2001-05-12T07:20:47.0", 2001, 5, 12, 7, 20, 47, 0);
        checkDate(adapter, "2001-05-12T07:20:47", 2001, 5, 12, 7, 20, 47, 0);
    }


    private static void checkDate(DateJsonAdapter adapter, String text, long ms) {
        Date d = adapter.fromJsonValue(text);
        assertThat(d).isNotNull();
        assertThat(d.getTime()).isEqualTo(ms);
    }

    private static void checkDate(DateJsonAdapter adapter, String text, int year, int month, int day, int h, int m, int s, int ms) {
        Date d = adapter.fromJsonValue(text);
        assertThat(d).isNotNull();
        Calendar c = Calendar.getInstance();
        c.setTime(d);

        assertThat(c.get(Calendar.YEAR)).isEqualTo(year);
        assertThat(c.get(Calendar.MONTH)+1).isEqualTo(month);
        assertThat(c.get(Calendar.DAY_OF_MONTH)).isEqualTo(day);
        assertThat(c.get(Calendar.HOUR)).isEqualTo(h);
        assertThat(c.get(Calendar.MINUTE)).isEqualTo(m);
        assertThat(c.get(Calendar.SECOND)).isEqualTo(s);
        assertThat(c.get(Calendar.MILLISECOND)).isEqualTo(ms);
    }
}
