package com.alperez.sunmi.pos.receiptengine.template;

import org.junit.Test;

import java.util.Locale;

import static com.google.common.truth.Truth.assertThat;

public class PriceFormattingTests {

    @Test
    public void validateFormatPriceFromLongAndScale() {
        Locale loc = new Locale("en", "GB", "");
        assertThat(PublicUtils.formatPriceAsCurrency(16435, 2, loc)).isEqualTo("\u00A3164.35");
        assertThat(PublicUtils.formatPriceAsCurrency(16435, 1, loc)).isEqualTo("\u00A31,643.50");
        assertThat(PublicUtils.formatPriceAsCurrency(4578569, 3, loc)).isEqualTo("\u00A34,578.57");
        assertThat(PublicUtils.formatPriceAsCurrency(35464578563L, 3, loc)).isEqualTo("\u00A335,464,578.56");

        assertThat(PublicUtils.formatPrice(16435, 2, null, null)).isEqualTo("164.35");
        assertThat(PublicUtils.formatPrice(16435, 1, null, null)).isEqualTo("1643.5");
        assertThat(PublicUtils.formatPrice(4578569, 3, null, null)).isEqualTo("4578.569");

        loc = new Locale("en", "US", "");
        assertThat(PublicUtils.formatPriceAsCurrency(16435, 2, loc)).isEqualTo("$164.35");
        assertThat(PublicUtils.formatPriceAsCurrency(16435, 1, loc)).isEqualTo("$1,643.50");
        assertThat(PublicUtils.formatPriceAsCurrency(4578569, 3, loc)).isEqualTo("$4,578.57");
        assertThat(PublicUtils.formatPriceAsCurrency(5464578566L, 3, loc)).isEqualTo("$5,464,578.57");



    }



    @Test
    public void validateBuildValueFromLongAndScale() {
        assertThat(Math.abs(PublicUtils.buildPriceValue(254356, 3) - 254.356)).isLessThan(0.000000001);
    }

}
