package com.webssa.library.attachment;

import android.content.Context;
import android.net.Uri;

import com.webssa.library.attachment.imports.AudioAttachmentImporter;
import com.webssa.library.attachment.imports.ImageAttachmentImporter;
import com.webssa.library.attachment.model.AttachmentType;

/**
 * The helper class to create concrete BAseAttachmentImporter implementation
 *
 * Created by stanislav.perchenko on 11/7/2016.
 */

public class ImportAttachment {

    public static ImportAttachment from(Uri sourceUri) {
        return new ImportAttachment(sourceUri);
    }

    private Uri sourceUri;
    private Context context;

    private ImportAttachment(Uri sourceUri) {
        this.sourceUri = sourceUri;
    }

    public ImportAttachment setContext(Context ctx) {
        context = ctx;
        return this;
    }

    public BaseAttachmentImporter typeOff(final AttachmentType aType) {
        if (sourceUri == null) {
            throw new IllegalStateException("A valid instance of the source Uri must be provided at the ImportAttachment instance creation");
        } else if (context == null) {
            throw new IllegalStateException("Context is not set. Call the setContext() first");
        }
        switch (aType) {
            case IMAGE:
                return new ImageAttachmentImporter(context, sourceUri);
            /*
            //TODO Restore this !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            case VIDEO:
                return new VideoAttachmentImporter(sourceUri);
                */
            case AUDIO:
                return new AudioAttachmentImporter(context, sourceUri);
            default:
                throw new IllegalStateException("Not supported type of attachment - "+aType);
        }
    }

}

