package com.webssa.library.attachment.crop;

import android.content.Context;
import android.graphics.Rect;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.webssa.library.core.R;
import com.webssa.library.core.databinding.FragmentCropAvatarImageBinding;

import java.io.File;

import androidx.annotation.Nullable;
import androidx.databinding.DataBindingUtil;
import androidx.fragment.app.Fragment;

/**
 * Created by Alex H. on 1/4/2017.
 */

public class CropAvatarImageFragment extends Fragment {

    public CropAvatarImageFragment() {
    }

    public interface OnCropAvatarImageListener {
        void onTryAgain();
        void onCancel();
        void onImageCropped(File imageFile, Rect selectedRegion);
    }

    private static final String ARG_FILE_PATH = "file";

    public static CropAvatarImageFragment newInstance(String filePath) {
        Bundle args = new Bundle();
        args.putString(ARG_FILE_PATH, filePath);
        CropAvatarImageFragment f = new CropAvatarImageFragment();
        f.setArguments(args);
        return f;
    }

    private File argImageFile;
    private FragmentCropAvatarImageBinding binding;
    private boolean firstTimeCreated = true;

    @Override
    public void onAttach(Context context) {
        if (!(context instanceof OnCropAvatarImageListener)) {
            throw new IllegalStateException("A host activity must implement the OnCropCoverImageListener interface");
        }
        super.onAttach(context);
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (binding == null) {
            binding = DataBindingUtil.inflate(inflater, R.layout.fragment_crop_avatar_image, container, false);
        } else {
            container.removeView(binding.getRoot());
        }
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (firstTimeCreated) {
            firstTimeCreated = false;

            argImageFile = new File(getArguments().getString(ARG_FILE_PATH));
            binding.croppedAvatar.setShowProgressBar(true);
            binding.croppedAvatar.setImageUriAsync(Uri.fromFile(argImageFile));
            binding.setClickerCancel(v -> ((OnCropAvatarImageListener) getActivity()).onCancel());
            binding.setClickerDone(v -> {
                binding.croppedAvatar.getCropRect();
                ((OnCropAvatarImageListener) getActivity()).onImageCropped(argImageFile, binding.croppedAvatar.getCropRect());
            });
            binding.setClickerRedo(v -> ((OnCropAvatarImageListener) getActivity()).onTryAgain());
        }
    }

    public void setNewImage(File file) {
        argImageFile = file;
        if (binding != null) {
            binding.croppedAvatar.setShowProgressBar(true);
            if (file != null) {
                binding.croppedAvatar.setImageUriAsync(Uri.fromFile(file));
            } else {
                binding.croppedAvatar.clearImage();
            }
        }
    }
}

