package com.webssa.library.attachment.imports;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.MediaMetadataRetriever;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Base64;
import android.webkit.MimeTypeMap;

import com.webssa.library.attachment.BaseAttachmentImporter;
import com.webssa.library.attachment.model.AttachmentType;
import com.webssa.library.attachment.util.FileUtils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.security.MessageDigest;

/**
 * Created by Alex H. on 1/19/2017.
 */

public class AudioAttachmentImporter extends BaseAttachmentImporter {

    public AudioAttachmentImporter(Context ctx, Uri sourceUri) {
        super(ctx, sourceUri);
    }

    @Override
    public BaseAttachmentImporter downscaleToMaxPixelCount(int maxPixelCount) {
        return this;
    }

    @Override
    protected void onImportInBackground(Uri sourceUri) throws Exception {
        File originalAudio = FileUtils.getNewRandomFile(getAttachmentCacheDirectory(getContext()));
        if (originalAudio == null) {
            throw new Exception("Error create cache file for importing content");
        }

        InputStream iStream = (sourceUri.toString().startsWith("content:"))
                ? new BufferedInputStream(getContext().getContentResolver().openInputStream(sourceUri))
                : new BufferedInputStream(new FileInputStream(sourceUri.getPath()));
        BufferedOutputStream tmpOs = new BufferedOutputStream(new FileOutputStream(originalAudio));

        try {

            //--- Copy content from a URI to the temporary file and calculate MD5 ---
            MessageDigest digester = MessageDigest.getInstance("MD5");
            byte[] transBuff = new byte[2048];
            int nBytes;
            while ((nBytes = iStream.read(transBuff)) > 0) {
                digester.update(transBuff, 0, nBytes);
                tmpOs.write(transBuff, 0, nBytes);
            }
            tmpOs.flush();


            String[] metas = getMimeTypeAndDurationOfRecord(originalAudio, sourceUri);
            String mimeType = metas[0];
            int duration = 0;
            if (!TextUtils.isEmpty(metas[1])) {
                duration = Integer.parseInt(metas[1]);
            }

            //--- Get MD5 Base64 and get final file name based on it ---
            String strMD5Base64 = Base64.encodeToString(digester.digest(), Base64.NO_WRAP | Base64.NO_PADDING | Base64.URL_SAFE);
            String relativeHashName = strMD5Base64 + "." + mimeType.substring(mimeType.lastIndexOf('/') + 1);

            //TODO Get options for cover images for the importing track
            /*ImageDbModel coverDbModel = ConfigHolder.getInstance().getSoundContentCovers().get(0);
            File coverFile = loadAndDownscaleCoverImage(coverDbModel.url(), coverDbModel.metaWidth(), coverDbModel.metaHeight());*/

            setResult(originalAudio, mimeType, duration, relativeHashName, AttachmentType.AUDIO);
            setCoverOptions(new File[]{/*coverFile*/}); //TODO provide cover options here
        } finally {
            iStream.close();
            tmpOs.close();
        }
    }



    private String[] getMimeTypeAndDurationOfRecord(File importedFile, Uri sourceUri) throws Exception {
        String mimeType = null;

        String originFilePath = FileUtils.getPath(getContext(), sourceUri);
        if (originFilePath != null) {
            mimeType = MimeTypeMap.getSingleton().getMimeTypeFromExtension(MimeTypeMap.getFileExtensionFromUrl(originFilePath.toLowerCase()));
        }

        MediaMetadataRetriever mediaRetriever = new MediaMetadataRetriever();
        mediaRetriever.setDataSource(importedFile.getAbsolutePath());
        if (mimeType == null) {
            mimeType = mediaRetriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_MIMETYPE);
        }
        if (mimeType == null) {
            throw new Exception("Importing resource is probably not an audio");
        }
        return new String[]{mimeType, mediaRetriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION)};
    }



    private File loadAndDownscaleCoverImage(String contentPath, int imageWidth, int imageHeight) {
        try {
            URL url = new URL(contentPath);
            InputStream inputStream = url.openConnection().getInputStream();
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int len;
            while ((len = inputStream.read(buffer)) > -1 ) {
                baos.write(buffer, 0, len);
            }
            baos.flush();
            InputStream is1 = new ByteArrayInputStream(baos.toByteArray());
            InputStream is2 = new ByteArrayInputStream(baos.toByteArray());
            final BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;
            BitmapFactory.decodeStream(is1, null, options);

            //calculate InSampleSize
            final int height = options.outHeight;
            final int width = options.outWidth;

            int inSampleSize = 1;
            if (height > imageHeight || width > imageWidth) {

                final int halfHeight = height / 2;
                final int halfWidth = width / 2;
                // Calculate the largest inSampleSize value that is a power of 2 and keeps both
                // height and width larger than the requested height and width.
                while ((halfHeight / inSampleSize) >= imageHeight
                        && (halfWidth / inSampleSize) >= imageWidth) {
                    inSampleSize *= 2;
                }
            }
            options.inSampleSize = inSampleSize;
            options.inJustDecodeBounds = false;
            Bitmap btmp = BitmapFactory.decodeStream(is2, null, options);

            File file = FileUtils.getNewRandomFile(getAttachmentCacheDirectory(getContext()));
            OutputStream tmpOs = new BufferedOutputStream(new FileOutputStream(file));
            btmp.compress(Bitmap.CompressFormat.PNG, 0, tmpOs);
            tmpOs.flush();

            return file;
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
}
