package com.webssa.library.attachment.model;

import android.os.Parcelable;

import com.google.auto.value.AutoValue;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * Created by stanislav.perchenko on 11/8/2016.
 */
@AutoValue
public abstract class CoverImportModel implements Parcelable, Cloneable {
    public abstract String localFile();
    public abstract String mimeType();
    public abstract String relativeHashName();
    public abstract int width();
    public abstract int height();


    public static CoverImportModel fromJson(String json) {
        try {
            JSONObject jObj = new JSONObject(json);
            return fromJsonObject(jObj);
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
    }

    public static CoverImportModel fromJsonObject(JSONObject jObj) throws JSONException {
        String localFile = jObj.getString("localFile");
        String mimeType = jObj.getString("mimeType");
        String hash = jObj.getString("hash");
        int w = jObj.getInt("w");
        int h = jObj.getInt("h");
        return new AutoValue_CoverImportModel(localFile, mimeType, hash, w, h);
    }

    public static CoverImportModel create(String localFile, String mimeType, String relativeHashName, int width, int height) {
        return new AutoValue_CoverImportModel(localFile, mimeType, relativeHashName, width, height);
    }

    @Override
    public CoverImportModel clone() {
        return new AutoValue_CoverImportModel(localFile(), mimeType(), relativeHashName(), width(), height());
    }

    public JSONObject toJsonObject() throws JSONException {
        JSONObject json = new JSONObject();
        json.put("localFile", localFile());
        json.put("mimeType", mimeType());
        json.put("hash", relativeHashName());
        json.put("w", width());
        json.put("h", height());
        return json;
    }

    public String toJson() {
        try {
            return toJsonObject().toString();
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
    }
}