package com.webssa.library.ui.widget;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Context;
import android.os.Build;
import android.util.AttributeSet;
import android.view.SoundEffectConstants;
import android.view.View;
import android.widget.Checkable;
import android.widget.ImageView;

import androidx.annotation.Nullable;
import androidx.databinding.BindingAdapter;
import androidx.databinding.InverseBindingAdapter;
import androidx.databinding.InverseBindingListener;
import androidx.databinding.InverseBindingMethod;
import androidx.databinding.InverseBindingMethods;

/**
 * Created by stanislav.perchenko on 5/21/2018
 */
@SuppressLint("AppCompatCustomView")
/*@InverseBindingMethods({
        @InverseBindingMethod(
                type = CheckedImageView.class,
                attribute = "checked",
                event = "android:checkedAttrChanged"),
})*/
public class CheckedImageView extends ImageView implements Checkable {

    private OnCheckedChangeListener mOnCheckedChangeListener;
    private boolean mChecked;
    private boolean mBroadcasting;

    public CheckedImageView(Context context) {
        super(context);
        refreshDrawableState();
    }

    public CheckedImageView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        refreshDrawableState();
    }

    public CheckedImageView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        refreshDrawableState();
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public CheckedImageView(Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        refreshDrawableState();
    }


    /**
     * Register a callback to be invoked when the checked state of this button
     * changes.
     *
     * @param listener the callback to call on checked state change
     */
    public void setOnCheckedChangeListener(@Nullable OnCheckedChangeListener listener) {
        mOnCheckedChangeListener = listener;
    }

    public OnCheckedChangeListener getOnCheckedChangeListener() {
        return mOnCheckedChangeListener;
    }

    @Override
    public void setChecked(boolean checked) {
        if (mChecked != checked) {
            mChecked = checked;
            refreshDrawableState();
            invalidate();

            // Avoid infinite recursions if setChecked() is called from a listener
            if (mBroadcasting) {
                return;
            }

            mBroadcasting = true;
            if (mOnCheckedChangeListener != null) {
                mOnCheckedChangeListener.onCheckedChanged(this, mChecked);
            }

            mBroadcasting = false;
        }
    }

    @Override
    public void toggle() {
        setChecked(!mChecked);
    }

    @Override
    public boolean performClick() {
        toggle();

        final boolean handled = super.performClick();
        if (!handled) {
            // View only makes a sound effect if the onClickListener was
            // called, so we'll need to make one here instead.
            playSoundEffect(SoundEffectConstants.CLICK);
        }

        return handled;
    }

    @Override
    public boolean isChecked() {
        return mChecked;
    }

    @Override
    public int[] onCreateDrawableState(int extraSpace) {
        final int[] drawableState = super.onCreateDrawableState(extraSpace + 1);
        if (isChecked()) {
            mergeDrawableStates(drawableState, CHECKED_STATE_SET);
        }
        return drawableState;
    }

    private static final int[] CHECKED_STATE_SET = {
            android.R.attr.state_checked
    };



    /**
     * Interface definition for a callback to be invoked when the checked state
     * of a compound button changed.
     */
    public interface OnCheckedChangeListener {
        /**
         * Called when the checked state of a compound button has changed.
         *
         * @param v The compound button view whose state has changed.
         * @param isChecked  The new checked state of buttonView.
         */
        void onCheckedChanged(View v, boolean isChecked);
    }


    /**********************************************************************************************/
    /******************  2-way binding support on the 'checked' property  *************************/
    /**********************************************************************************************/

    public static class BindingAdapters {

        @BindingAdapter("checked")
        public static void setChecked(CheckedImageView civ, boolean checked) {
            civ.setChecked(checked);
        }

        @InverseBindingAdapter(attribute = "checked")
        public static boolean getChecked(CheckedImageView civ) {
            return civ.isChecked();
        }

        @BindingAdapter("app:checkedAttrChanged")
        public static void setCheckChangeListener(CheckedImageView civ, final InverseBindingListener attrChange) {
            if (attrChange == null) {
                civ.setOnCheckedChangeListener(null);
            } else {
                civ.setOnCheckedChangeListener((v, isChecked) -> attrChange.onChange());
            }
        }
    }
}
