package com.webssa.library.widget;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.util.Log;
import android.widget.ScrollView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.webssa.library.core.R;

/**
 * Created by stanislav.perchenko on 12/4/2019, 2:12 PM.
 */
public class MaxDimensFrameLayout extends ScrollView {

    private int attrMaxWidth, attrMaxHeight;

    public MaxDimensFrameLayout(@NonNull Context context) {
        super(context);
    }

    public MaxDimensFrameLayout(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        extractCustomAttrs(context, attrs);
    }

    public MaxDimensFrameLayout(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        extractCustomAttrs(context, attrs);
    }

    public MaxDimensFrameLayout(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        extractCustomAttrs(context, attrs);
    }

    private void extractCustomAttrs(Context context, AttributeSet attrs) {
        if (attrs == null) return;
        TypedArray a = context.getResources().obtainAttributes(attrs, R.styleable.MaxDimensFrameLayout);
        try {
            attrMaxWidth = a.getDimensionPixelSize(R.styleable.MaxDimensFrameLayout_maxWidth, attrMaxWidth);
            attrMaxHeight = a.getDimensionPixelSize(R.styleable.MaxDimensFrameLayout_maxHeight, attrMaxHeight);
        } finally {
            a.recycle();
        }
    }


    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);

        int measuerdW = getMeasuredWidth();
        int measuerdH = getMeasuredHeight();
        boolean changed = false;

        if (attrMaxWidth > 0 && (measuerdW > attrMaxWidth)) {
            measuerdW = attrMaxWidth;
            changed = true;
        }

        if (attrMaxHeight > 0 && (measuerdH > attrMaxHeight)) {
            measuerdH = attrMaxHeight;
            changed = true;
        }

        if (changed) {
            Log.e("MAX_DIMEN", "Cut-down measures.");
            setMeasuredDimension(measuerdW, measuerdH);
            invalidate();
            requestLayout();
        }

    }
}
