﻿using Microsoft.Extensions.Options;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using PDA.BL.Interfaces;
using PDA.BL.Models.Interfaces;
using PDA.BL.Models.Models;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;

namespace PDA.BL.Services
{
    public class AzureBlobStorageService: IAzureBlobStorageService
    {
        private readonly AzureBlobStorageConfiguration _storageConfig;

        public AzureBlobStorageService(IOptionsMonitor<AzureBlobStorageConfiguration> storageConfig) => _storageConfig = storageConfig.CurrentValue;
        public async Task<string> UploadFile(byte[] fileBinary, string blobName, string containerName)
        {
            var blockBlob = await GetBlobReference(containerName, blobName);
            await blockBlob.UploadFromByteArrayAsync(fileBinary, 0, fileBinary.Length);
            return blockBlob.Uri.AbsoluteUri;
        }
        private async Task<CloudBlockBlob> GetBlobReference(string containerName, string blobName)
        {
            var blobContainer = await GetBlobContainer(containerName);
            return blobContainer.GetBlockBlobReference(blobName);
        }

        private async Task<CloudBlobContainer> GetBlobContainer(string containerName)
        {
            var blobClient = GetBlobClient();

            // Retrieve a reference to a container.(it gives a non-null reference even if containerName doesn't exist)
            var container = blobClient.GetContainerReference(containerName);

            // Create the container if it doesn't already exist.
            await container.CreateIfNotExistsAsync(BlobContainerPublicAccessType.Blob, null, null);

            return container;
        }
        private CloudBlobClient GetBlobClient()
        {
            return GetStorageAccount().CreateCloudBlobClient();
        }

        private CloudStorageAccount GetStorageAccount()
        {
            return CloudStorageAccount.Parse(_storageConfig.AzureBlobStorageConnectionString);
        }
    }
}
