﻿using Microsoft.AspNetCore.Http;
using PDA.BL.Interfaces;
using PDA.Database.EntityModels;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading.Tasks;

namespace PDA.BL.Services
{
    public class UploadFileService : IUploadFileService
    {
        private readonly IAzureBlobStorageService _storageService;
        private readonly INoteService _noteService;
        private const string imageContainerName = "images";
        public UploadFileService(IAzureBlobStorageService storageService, INoteService noteService)
        {
            _storageService = storageService;
            _noteService = noteService;
        }
        public async Task<List<string>> UploadNoteAttachments(IList<IFormFile> images, int noteId)
        {
            var imageUrls = new List<string>();
            for(var i = 0; i < images.Count; i++)
            {
                var noteAttachment = await GetNoteAttachment(images[i], noteId);
                imageUrls.Add(await _noteService.SaveAttahment(noteAttachment.PhotoURL, noteAttachment.NoteId, i));
            }
            return imageUrls;
        }
        public async Task<NoteAttachment> GetNoteAttachment(IFormFile image, int noteId)
        {
            var noteAttachment = new NoteAttachment();
            var imageBlobFileName = $"{noteAttachment.GetHashCode()}_{image.FileName}";
            using (var ms = new MemoryStream())
            {
                await image.CopyToAsync(ms);
                noteAttachment.NoteId = noteId;
                noteAttachment.PhotoURL = await _storageService.UploadFile(ms.ToArray(), imageBlobFileName, imageContainerName); 
            
            }
            return noteAttachment;
        }

        public async Task<List<string>> UploadImages(IList<IFormFile> images)
        {
            var imageUrls = new List<string>();
            for (var i = 0; i < images.Count; i++)
            {
                var imageBlobFileName = $"{images[i].GetHashCode()}_{images[i].FileName}";
                using (var ms = new MemoryStream())
                {
                    await images[i].CopyToAsync(ms);
                    var imageUrl = await _storageService.UploadFile(ms.ToArray(), imageBlobFileName, imageContainerName);
                    imageUrls.Add(imageUrl);
                 
                }
            }
            return imageUrls;
        }
    }
}
