﻿using Microsoft.AspNetCore.Identity.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PDA.Database.EntityModels
{
    public class PDADbContext: IdentityDbContext<User>
    {
        public PDADbContext(DbContextOptions<PDADbContext> options): base(options)
        {

        }
        protected override void OnModelCreating(ModelBuilder builder)
        {
            var cascadeFKs = builder.Model.GetEntityTypes()
        .SelectMany(t => t.GetForeignKeys())
        .Where(fk => !fk.IsOwnership && fk.DeleteBehavior == DeleteBehavior.Cascade);

            foreach (var fk in cascadeFKs)
                fk.DeleteBehavior = DeleteBehavior.Restrict;

            base.OnModelCreating(builder);
        }
        public DbSet<Visit> Visits { get; set; }
        public DbSet<CollectedGood> CollectedGoods { get; set; }
        public DbSet<Emergency> Emergencies { get; set; }
        public DbSet<GoodCategory> GoodCategories { get; set; }
        public DbSet<Note> Notes { get; set; }
        public DbSet<NoteAttachment> NoteAttachments { get; set; }
        public DbSet<Price> Prices { get; set; }
        public DbSet<Shift> Shifts { get; set; }
        public DbSet<Site> Sites { get; set; }
        public DbSet<VisitState> VisitStates { get; set; }
        public DbSet<ShiftState> ShiftStates { get; set; }
        public DbSet<UserToken> UsersTokens { get; set; }
        public DbSet<EWCCode> EWCCodes { get; set; }
    }

}
