﻿using Microsoft.EntityFrameworkCore;
using PDA.Database.EntityModels;
using PDA.Database.Interfaces;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace PDA.Database.Repositories
{
    public class GoodsRepository : IGoodsRepository
    {
        private readonly PDADbContext _context;
        public GoodsRepository(PDADbContext context)
        {
            _context = context;
        }
        public async Task<List<Price>> GetAllGoods()
        {
            var goods = _context.Prices.Include(x => x.GoodCategory).ThenInclude(x => x.EWCCode).Select(x => x);
            return await goods.GroupBy(p => new {p.SiteId, p.GoodCategoryId}, (key, c) => c.OrderByDescending(x => x.TimeUpdate).FirstOrDefault()).ToListAsync();
        }
        public async Task SaveVisitGoods(List<CollectedGood> goods, int visitId)
        {
            var visitGoods = _context.CollectedGoods.Where(x => x.VisitId == visitId);
            if (visitGoods.Count() > 0)
                _context.CollectedGoods.RemoveRange(visitGoods);
            for(var i = 0; i < goods.Count; i++)
            {
                goods[i].VisitId = visitId;
                goods[i].Order = i;
                await _context.CollectedGoods.AddAsync(goods[i]);
            }
        }
        public async Task<List<CollectedGood>> GetVisitGoodsAsync(int visitId) => await _context.CollectedGoods.Include(x => x.Price).Include(x => x.GoodCategory)
            .Where(x => x.VisitId == visitId).ToListAsync();
        public async Task<List<EWCCode>> GetEWCCodes() => await _context.EWCCodes.ToListAsync();
    }
}
