﻿using PDA.Database.CustomExceptions;
using PDA.Database.EntityModels;
using PDA.Database.Interfaces;
using PDA.Database.Models;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using System.Linq;
using Microsoft.EntityFrameworkCore;

namespace PDA.Database.Repositories
{
    public class NoteRepository : INoteRepository
    {
        private readonly PDADbContext _context;
        private readonly IShiftRepository _shiftRepository;

        public NoteRepository(PDADbContext context, IShiftRepository shiftRepository)
        {
            _context = context;
            _shiftRepository = shiftRepository;

        }
        public async Task<Note> CreateNote(string userId, Note note)
        {
            var parentShift = await _shiftRepository.GetByShiftId(note.ShiftId, userId);
            if (parentShift == null || parentShift.StateId != (int)ShiftStateEnum.Accepted)
                throw new NoteNotCreatableException();
            if (note.VisitId.HasValue)
            {
                var visit = await _context.Visits.FirstOrDefaultAsync(x => x.VisitId == note.VisitId.Value);
                if(visit.ShiftId != note.ShiftId || visit.StateId == (int)VisitStateEnum.New)
                    throw new NoteNotCreatableException();
            }
            note.UserId = userId;
            note.Number = await GetUserNotesCount(userId) + 1;
            await _context.Notes.AddAsync(note);
            await _context.SaveChangesAsync();
            return note;
        }
        public async Task<int> SaveNoteFromVisit(Note note, string userId)
        {
            note.Number = await GetUserNotesCount(userId) + 1;
            note.UserId = userId;
            _context.Notes.Add(note);
            await _context.SaveChangesAsync();
            return note.NoteId;
        }
        public async Task<int> GetUserNotesCount(string userId) => await _context.Notes.Where(x => x.UserId == userId).CountAsync();
        public async Task<string> SaveAttahment(string url, int noteId, int imageOrder) 
        {
            _context.NoteAttachments.Add(new NoteAttachment { NoteId = noteId, PhotoURL = url, ImageOrder = imageOrder });
            await _context.SaveChangesAsync();
            return url;
        }
    }
}
