﻿using Microsoft.EntityFrameworkCore;
using PDA.Database.EntityModels;
using PDA.Database.Interfaces;
using PDA.Database.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PDA.Database.Repositories
{
    public class TokenRepository : ITokenRepository
    {
        private readonly PDADbContext _context;
        public TokenRepository(PDADbContext context)
        {
            _context = context;
        }
        public async Task<UserToken> GenerateNewAccesssToken(string userId)
        {
            var oldToken = await GetAccessToken(userId);
            if (oldToken != null)
                oldToken.ExpiresAt = DateTime.UtcNow.AddDays(-1);
            var token = CreateNewToken(userId, "AccessToken", DateTime.UtcNow, DateTime.UtcNow.AddDays(7));
            await SaveToken(token);
            return token;
        }
        public async Task<string> GetAccessTokenId(string userId)
        { 
           var token =  await _context.UsersTokens.FirstOrDefaultAsync(x => x.ExpiresAt > DateTime.UtcNow && x.UserId == userId && x.TokenType == "AccessToken");
            return token.TokenId.ToString();
        }
        public async Task<UserToken> GetAccessToken(string userId) => await _context.UsersTokens.FirstOrDefaultAsync(x => x.ExpiresAt > DateTime.UtcNow && x.UserId == userId && x.TokenType == "AccessToken");
      
        public async Task<string> GetRefreshToken(string userId)
        {
            var refreshtoken = await GetValidRefreshToken(userId);
            if(refreshtoken == null)
            {
                refreshtoken = CreateNewToken(userId, "RefreshToken", DateTime.UtcNow, DateTime.UtcNow.AddYears(1));
                await SaveToken(refreshtoken);
                
            }
            return refreshtoken.TokenId.ToString();
        }

      
        public Task<bool> RefreshTokenIsValid(string userId)
        {
            throw new NotImplementedException();
        }
        private async Task<UserToken> GetValidRefreshToken(string userId) => await _context.UsersTokens.FirstOrDefaultAsync(x => x.ExpiresAt < DateTime.UtcNow && x.UserId == userId && x.TokenType == "RefreshToken");

        public async Task<bool> TokenIsValidAsync(string phoneNumber, string pin)
        {
            throw new NotImplementedException();
        }

        private UserToken CreateNewToken(string userId, string tokenType, DateTime createdAt, DateTime expiresAt) => new UserToken { TokenId = Guid.NewGuid(), UserId = userId, CreatedAt = createdAt,
            TokenType = tokenType, ExpiresAt = expiresAt };

        private async Task SaveToken(UserToken token)
        {

            await _context.UsersTokens.AddAsync(token);
            await _context.SaveChangesAsync();
        }

    }
}
