﻿using PDA.BL.Interfaces;
using PDA.Database.Interfaces;
using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace PDA.Database.Services
{
    public class RijndaelService: IRijndaelService
    {
        private readonly ISecurityConfiguration securityConfig;

        public RijndaelService(ISecurityConfiguration securityConfig)
        {
            this.securityConfig = securityConfig;
        }

        public string Encrypt(string plainText)
        {
            if (string.IsNullOrEmpty(plainText))
                return "";
            byte[] initVectorBytes = Encoding.ASCII.GetBytes(securityConfig.RijndaelInitVector);
            byte[] saltValueBytes = Encoding.ASCII.GetBytes(securityConfig.RijndaelSalt);
            byte[] plainTextBytes = Encoding.UTF8.GetBytes(plainText);

            PasswordDeriveBytes password = new PasswordDeriveBytes
            (
                securityConfig.RijndaelPassPhrase,
                saltValueBytes,
                securityConfig.RijndaelAlgorithm,
                securityConfig.RijndaelPasswordIteration
            );
            byte[] keyBytes = password.GetBytes(securityConfig.RijndaelKeySize / 8);
            RijndaelManaged symmetricKey = new RijndaelManaged
            {
                Mode = CipherMode.CBC
            };
            ICryptoTransform encryptor = symmetricKey.CreateEncryptor
            (
                keyBytes,
                initVectorBytes
            );

            MemoryStream memoryStream = new MemoryStream();

            CryptoStream cryptoStream = new CryptoStream
            (
                memoryStream,
                encryptor,
                CryptoStreamMode.Write
            );


            cryptoStream.Write(plainTextBytes, 0, plainTextBytes.Length);
            cryptoStream.FlushFinalBlock();
            byte[] cipherTextBytes = memoryStream.ToArray();
            memoryStream.Close();
            cryptoStream.Close();

            string cipherText = Convert.ToBase64String(cipherTextBytes);
            return cipherText;
        }

        public string Decrypt(string cipherText)
        {
            if (string.IsNullOrEmpty(cipherText))
                return null;
            try
            {
                byte[] initVectorBytes = Encoding.ASCII.GetBytes(securityConfig.RijndaelInitVector);
                byte[] saltValueBytes = Encoding.ASCII.GetBytes(securityConfig.RijndaelSalt);
                byte[] cipherTextBytes = Convert.FromBase64String(cipherText);

                PasswordDeriveBytes password = new PasswordDeriveBytes
                (
                securityConfig.RijndaelPassPhrase,
                saltValueBytes,
                securityConfig.RijndaelAlgorithm,
                securityConfig.RijndaelPasswordIteration
                );
                byte[] keyBytes = password.GetBytes(securityConfig.RijndaelKeySize / 8);
                RijndaelManaged symmetricKey = new RijndaelManaged
                {
                    Mode = CipherMode.CBC
                };
                ICryptoTransform decryptor = symmetricKey.CreateDecryptor
                (
                    keyBytes,
                    initVectorBytes
                );

                MemoryStream memoryStream = new MemoryStream(cipherTextBytes);

                CryptoStream cryptoStream = new CryptoStream
                (
                    memoryStream,
                    decryptor,
                    CryptoStreamMode.Read
                );

                byte[] plainTextBytes = new byte[cipherTextBytes.Length];

                int decryptedByteCount = cryptoStream.Read
                (
                    plainTextBytes,
                    0,
                    plainTextBytes.Length
                );

                memoryStream.Close();
                cryptoStream.Close();

                string plainText = Encoding.UTF8.GetString
                (
                    plainTextBytes,
                    0,
                    decryptedByteCount
                );
                return plainText;
            }
            catch
            {
                return cipherText;
            }
        }
    }
}
