﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using PDA.API.Filters;
using PDA.API.WebModels;
using PDA.BL.Interfaces;
using PDA.BL.Models;
using PDA.Database.CustomExceptions;
using System.Threading.Tasks;

namespace PDA.API.Controllers.v1
{
    [LoginExceptionFilter]
    public class AuthorizationController : ControllerBase
    {
        private readonly IDriverService _driverService;
        private readonly ITokenService _tokenService;
        private readonly IMapper _mapper;
        private readonly MobileAppSettings _mobileAppSettings;

        public AuthorizationController(IDriverService driverService, ITokenService tokenService, IMapper mapper, IOptions<MobileAppSettings> options)
        {
            _driverService = driverService;
            _tokenService = tokenService;
            _mapper = mapper;
            _mobileAppSettings = options.Value;
        }

        [HttpPut("logindriver")]
        public async Task<LoginAnswer> Login([FromBody]UserLoginModel loginModel)
        {
            if (loginModel.Imei == null || loginModel.Pwd == null || (loginModel.Phone == null && loginModel.Email == null))
                throw new InvalidParamsException();
            var loginAnswer = new LoginAnswer();
            var driverId = string.Empty;
            if (loginModel.Phone != null)
                driverId = await _driverService.CheckDriverExists(loginModel.Phone, loginModel.Pwd);
            else if (loginModel.Email != null)
                driverId = await _driverService.CheckDriverExistsEmail(loginModel.Email, loginModel.Pwd);
            if (driverId == null)
                loginAnswer = CreateFailedAnswer();
            else
                loginAnswer = await CreateSuccessAnswer(driverId);
            return loginAnswer;
        }
        private async Task<LoginAnswer> CreateSuccessAnswer(string driverId)
        {
            var login = new LoginAnswer();
            //var accessToken = await _tokenService.GetAccessToken(driverId);
            //if (accessToken == null)
            var accessToken = await _tokenService.GenerateNewAccesssToken(driverId);
            var driver = await _driverService.GetDriverAsync(driverId);
            login.User = _mapper.Map<UserEntity>(driver);
            login.User.Role = _driverService.GetUserRole(driverId);
            login.AccessToken = accessToken.TokenId.ToString();
            login.ExpiresIn = /*Convert.ToInt32((accessToken.ExpiresAt - accessToken.CreatedAt).TotalSeconds)*/0x7FFFFFFF;
            login.RefreshToken = await _tokenService.GetRefreshToken(driverId);
            login.Success = true;
            login.DieselPurchase = _mobileAppSettings.DieselPurchase;
            login.ShiftEnd = _mobileAppSettings.ShiftEnd;
            login.ShiftStart = _mobileAppSettings.ShiftStart;
            return login;
        }
        private LoginAnswer CreateFailedAnswer() => new LoginAnswer { Success = false, ErrorMessage = "Please enter valid credentials" };

    }
}
