﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using PDA.API.Filters;
using PDA.API.WebModels;
using PDA.BL.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace PDA.API.Controllers.v1
{
    [ServiceFilter(typeof(AuthorizationTokenAttribute))]
    public class GoodsController : ControllerBase
    {
        private readonly IGoodsService _goodsService;
        private readonly IMapper _mapper;
        public GoodsController(IGoodsService goodsService, IMapper mapper)
        {
            _goodsService = goodsService;
            _mapper = mapper;
        }
        [HttpGet("pricedgoods")]
        public async Task<List<PricedGood>> GetAllGoodsWithPrices()
        {
            var goods = await _goodsService.GetAllGoods();
            var pricedGoods = new List<PricedGood>();
            foreach (var good in goods)
                pricedGoods.Add(_mapper.Map<PricedGood>(good));
            return pricedGoods;
        }

        [HttpGet("pricedgoodsv2")]
        public async Task<List<PricedGoodV2>> GetAllGoodsWithPricesv2()
        {
            var goods = await _goodsService.GetAllGoods();
            var pricedGoods = new List<PricedGoodV2>();
            foreach (var good in goods)
                pricedGoods.Add(_mapper.Map<PricedGoodV2>(good));
            return pricedGoods;
        }

        [HttpGet("ewccodes")]
        public async Task<List<EWCCode>> GetEWCCodes()
        {
            var codes = await _goodsService.GetEWCCodes();
            var mappedCodes = new List<EWCCode>();
            foreach (var code in codes)
                mappedCodes.Add(_mapper.Map<EWCCode>(code));
            return mappedCodes;
        }
    }
}
