﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using BrunoZell.ModelBinding;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.WebUtilities;
using Newtonsoft.Json;
using PDA.API.Filters;
using PDA.API.WebModels;
using PDA.BL.Interfaces;
using PDA.Database.CustomExceptions;
using PDA.Database.EntityModels;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace PDA.API.Controllers.v1
{
    [NoteExceptionFilter]
    [ServiceFilter(typeof(AuthorizationTokenAttribute))]
    public class NoteController : Controller
    {
        private readonly INoteService _noteService;
        private readonly IMapper _mapper;
        private readonly IUploadFileService _fileService;
        private string CurrentUserId => Request.Headers["User"];
        public NoteController(INoteService noteService, IMapper mapper, IUploadFileService fileService)
        {
            _noteService = noteService;
            _mapper = mapper;
            _fileService = fileService;
        }
        [HttpPost("note")]
        public async Task<NoteEntity> CreateNote([ModelBinder(BinderType = typeof(JsonModelBinder))] NotePostBody note_data,
            IList<IFormFile> note_image)
        {
            if (note_image.Count() > 5)
                throw new MaxImagesException();
            if (note_data == null)
                throw new NoteNotProvidedException();
            var note = await _noteService.CreateNote(CurrentUserId, _mapper.Map<Note>(note_data));
            var noteEntity = _mapper.Map<NoteEntity>(note);
            noteEntity.Images = note_image.Count > 0 ? await _fileService.UploadNoteAttachments(note_image, noteEntity.NoteId) : new List<string>();
            return noteEntity;
        }

    }
}
