﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using PDA.API.CustomExtensions;
using PDA.API.Filters;
using PDA.API.WebModels;
using PDA.BL.Interfaces;
using PDA.Database.EntityModels;
using PDA.Database.Models;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace PDA.API.Controllers.v1
{
    [ShiftExceptionFilter]
    [ServiceFilter(typeof(AuthorizationTokenAttribute))]
    public class ShiftController : ControllerBase
    {
        private readonly IShiftService _shiftService;
        private readonly IMapper _mapper;
        private string CurrentUserId => Request.Headers["User"];
        public ShiftController(IShiftService shiftService, IMapper mapper)
        {
            _shiftService = shiftService;
            _mapper = mapper;
        }
        // GET: api/<controller>
        [HttpGet("todaynewshift")]
        public async Task<FindNewShiftResponse> GetTodayNewShift()
        {
            var shift = await _shiftService.FindNewShiftForDate(CurrentUserId);
            return new FindNewShiftResponse { ServerTime = DateTimeOffset.UtcNow.ToISOString(), Shift = _mapper.Map<ShiftEntity>(shift) };
        }

        // GET api/<controller>/5
       [HttpPut("shift/{id}/accept")]
        public async Task<AcceptShiftResponse> AcceptShift(int id)
        {
            var shift = await _shiftService.AcceptShift(id);
            return CreateAcceptShiftResponse(shift);
        }

        // POST api/<controller>
        [HttpPut("shift/{id}/reject")]
        public async Task<ShiftEntity> RejectShift(int id)
        {
            var shift = await _shiftService.RejectShift(id, CurrentUserId);
            return _mapper.Map<ShiftEntity>(shift);
        }

        // PUT api/<controller>/5
        [HttpGet("shift/{id}")]
        public async Task<ShiftEntity> GetShift(int id)
        {
            var shift = await _shiftService.GetByShiftId(id, CurrentUserId);
            return _mapper.Map<ShiftEntity>(shift);
        }

        // DELETE api/<controller>/5
        [HttpPut("shift/{id}/close")]
        public async Task<ShiftEntity> CloseShift(int id, [FromBody] DateTime close_time)
        {
            var shift = await _shiftService.CloseShift(id, close_time, CurrentUserId);
            return _mapper.Map<ShiftEntity>(shift);
        }
        private AcceptShiftResponse CreateAcceptShiftResponse(Shift shift)
        {
            var visits = new List<VisitEntity>();
            var sites = new List<SiteEntity>();
            var shiftEntity = _mapper.Map<ShiftEntity>(shift);
            foreach (var visit in shift.Visits)
            {
                var visitEntity = _mapper.Map<VisitEntity>(visit);
                visits.Add(visitEntity);
                var siteEntity = _mapper.Map<SiteEntity>(visit.Site);
                sites.Add(siteEntity);
            }
            return new AcceptShiftResponse { Shift = shiftEntity, Visits = visits, Sites = sites };
        }
    }
}
