﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using PDA.API.Filters;
using PDA.API.WebModels;
using PDA.BL.Interfaces;
using PDA.Database.CustomExceptions;
using PDA.Database.EntityModels;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace PDA.API.Controllers.v1
{
    [VisitExceptionFilter]
    [ServiceFilter(typeof(AuthorizationTokenAttribute))]
    public class VisitController : ControllerBase
    {
        private readonly IVisitService _visitService;
        private readonly IMapper _mapper;
        public VisitController(IVisitService visitService, IMapper mapper)
        {
            _visitService = visitService;
            _mapper = mapper;
        }
        protected string CurrentUserId => Request.Headers["User"];
        // GET: api/<controller>
        [HttpPut("visit/{id}/start")]
        public async Task<WebModels.VisitState> StartVisit(int id, [FromBody]DelayedTimeModel model)
        {
            var visit = await _visitService.StartVisit(id, model?.Time, CurrentUserId);
            return _mapper.Map<WebModels.VisitState>(visit);
        }

        // GET api/<controller>/5
        [HttpPut("visit/{id}/fail")]
        public async Task<WebModels.VisitState> FailVisit(int id, [FromBody]VisitFailPostBody visitFailPostBody)
        {
            if (visitFailPostBody.Images.Count > 5)
                throw new MaxImagesException();
            var visit = await _visitService.FailVisit(id, visitFailPostBody.DelayedFailTime, visitFailPostBody.Notetext, visitFailPostBody.Images, visitFailPostBody.EmergencyId, CurrentUserId);
            var visitState = _mapper.Map<WebModels.VisitState>(visit);
            visitState.Note.Images = visit.Note.NoteAttachments != null ? visit.Note.NoteAttachments.OrderBy(x => x.ImageOrder).Select(x => x.PhotoURL).ToList() : new List<string>();
            return visitState;
        }
        [HttpGet("visit/{visit_id}/state")]
        public async Task<WebModels.VisitState> GetVisitState(int visit_id)
        {
            var visit = await _visitService.GetVisitState(visit_id, CurrentUserId);
            var visitState = _mapper.Map<WebModels.VisitState>(visit);
            return visitState;
        }
        [HttpPut("visit/{id}/complete")]
        public async Task<VisitResponse> CompleteVisit(int id, [FromBody] VisitResult visitResult)
        {
            var note = visitResult.Note != null ? _mapper.Map<Note>(visitResult.Note) : null;
            var images = visitResult.Note != null ? visitResult.Note.Images : null;
            var goods = _mapper.Map<List<Database.EntityModels.CollectedGood>>(visitResult.Goods);
            var visit = await _visitService.CompleteVisit(id, visitResult.DelayedCompleteTime, note, CurrentUserId, images, goods,
                visitResult.CustomerName, visitResult.CustomerSignature, visitResult.ClearBags, visitResult.IsWhiteSacks);
            var visitResponse = new VisitResponse();
            visitResponse.Visit = _mapper.Map<VisitEntity>(visit);
            if (visit.Note != null)
            {
                visitResponse.Note = _mapper.Map<NoteEntity>(visit.Note);
                visitResponse.Note.Images = visit.Note.NoteAttachments != null ? visit.Note.NoteAttachments.OrderBy(x => x.ImageOrder).Select(x => x.PhotoURL).ToList() : new List<string>();
            }
            visitResponse.CollectedGoods = _mapper.Map<List<WebModels.CollectedGood>>(visit.CollectedGoods);
           
            return visitResponse;
        }

        [HttpPut("visit/add")]
        public async Task<VisitResponse> AddVisit([FromBody] AddVisitModel addVisitModel)
        {
            var visit = await _visitService.AddVisit(addVisitModel.ShiftId, addVisitModel.SiteId, addVisitModel.SuggestTime, CurrentUserId);
            var visitResponse = new VisitResponse
            {
                Visit = _mapper.Map<VisitEntity>(visit)
            };
            return visitResponse;
        }
    }
}
