﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using PDA.BL.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace PDA.API.Filters
{
    public class AuthorizationTokenAttribute: ActionFilterAttribute
    {
        private readonly ITokenService _tokenService;
        public AuthorizationTokenAttribute(ITokenService tokenService)
        { _tokenService = tokenService; }
        public override async Task OnActionExecutionAsync(ActionExecutingContext filterContext, ActionExecutionDelegate next)
        {
            if (!filterContext.HttpContext.Request.Headers.TryGetValue("User", out var headers))
            {
                filterContext.Result = new BadRequestObjectResult("User is not authorized") { StatusCode = 401 };
            }
            if (!filterContext.HttpContext.Request.Headers.TryGetValue("Authorization", out var header))
            {
                filterContext.Result = new BadRequestObjectResult("No token provided") { StatusCode = 401 };
            }
            var userId = headers.FirstOrDefault();
            if (string.IsNullOrEmpty(userId))
            {
                filterContext.Result = new BadRequestObjectResult("Login Failed") { StatusCode = 401 };
                return;
            }
            var userToken = header.FirstOrDefault();
            if (string.IsNullOrEmpty(userToken))
            {
                filterContext.Result = new BadRequestObjectResult("Login Failed") { StatusCode = 401 };
                return;
            }
            var accessToken = await _tokenService.GetAccessTokenId(userId);
            userToken = userToken.Split(" ")[1];
            if (accessToken == null || accessToken != userToken)
            {
                filterContext.Result = new BadRequestObjectResult("No valid token provided") { StatusCode = 401 };
                return;
            }
            await next();
        }
    }
}
