﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using PDA.API.Errors;
using PDA.Database.CustomExceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace PDA.API.Filters
{
    public class ShiftExceptionFilterAttribute : ExceptionFilterAttribute
    {
        public override Task OnExceptionAsync(ExceptionContext context)
        {
            IActionResult result;
            switch(context.Exception)
            {
                case ShiftNotFoundException se:
                    result = new BadRequestObjectResult(se.Message) { StatusCode = (int)ApiErrorCode.NotFound };
                    break;
                case ShiftNotAcceptableException sae:
                    result = new BadRequestObjectResult(sae.Message) { StatusCode = (int)ApiErrorCode.Forbidden };
                    break;
                case ShiftNotClosableException sce:
                    result = new BadRequestObjectResult(sce.Message) { StatusCode = (int)ApiErrorCode.Forbidden };
                    break;
                case ShiftNotRejectableException sre:
                    result = new BadRequestObjectResult(sre.Message) { StatusCode = (int)ApiErrorCode.Forbidden };
                    break;
                case ShiftNotUsersException sue:
                    result = new BadRequestObjectResult(sue.Message) { StatusCode = (int)ApiErrorCode.Forbidden };
                    break;
                default:
                    return base.OnExceptionAsync(context);
            }
            context.Result = result;
            return Task.CompletedTask;
        }
    }
}
