﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using PDA.API.Errors;
using PDA.Database.CustomExceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace PDA.API.Filters
{
    public class VisitExceptionFilterAttribute : ExceptionFilterAttribute
    {
        public override Task OnExceptionAsync(ExceptionContext context)
        {
            IActionResult result;
            switch (context.Exception)
            {
                case VisitNotFoundException ve:
                    result = new BadRequestObjectResult(ve.Message) { StatusCode = (int)ApiErrorCode.NotFound };
                    break;
                case VisitNotStartableException sae:
                    result = new BadRequestObjectResult(sae.Message) { StatusCode = (int)ApiErrorCode.Forbidden };
                    break;
                case VisitNotFailableException sce:
                    result = new BadRequestObjectResult(sce.Message) { StatusCode = (int)ApiErrorCode.Forbidden };
                    break;
                case VisitNotCompletableException sre:
                    result = new BadRequestObjectResult(sre.Message) { StatusCode = (int)ApiErrorCode.Forbidden };
                    break;
                case MaxImagesException mie:
                    result = new BadRequestObjectResult(mie.Message) { StatusCode = (int)ApiErrorCode.BadRequest };
                    break;
                default:
                    return base.OnExceptionAsync(context);
            }
            context.Result = result;
            return Task.CompletedTask;
        }
    }
}

