﻿using AutoMapper;
using PDA.API.CustomExtensions;
using PDA.API.WebModels;
using PDA.Database.EntityModels;
using PDA.Database.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace PDA.API.Mapping
{
    public class MappingProfile: Profile
    {
        
        public MappingProfile()
        {
            CreateMap<User, UserEntity>().ForMember(dest => dest.UserId, opt => opt.MapFrom(src => src.Id))
                .ForMember(dest => dest.RegistrationDate, opt => opt.MapFrom(src => src.RegistrationDate.ToString("yyyy-MM-dd")));
            CreateMap<Shift, ShiftEntity>().ForMember(dest => dest.DriverId, opt => opt.MapFrom(src => src.UserId))
                .ForMember(dest => dest.ExecutionDate, opt => opt.MapFrom(src => src.ExecutionDate.ToString("yyyy-MM-dd")))
                .ForMember(dest => dest.ScheduledTimeStart, opt => opt.MapFrom(src => src.ScheduledTimeStart.HasValue ? src.ScheduledTimeStart.Value.ToString("HH:mm:ss") : null))
                .ForMember(dest => dest.ScheduledDuration, opt => opt.MapFrom(src => src.Duration.HasValue ? src.Duration.Value.ToString("HH:mm:ss") : null))
                .ForMember(dest => dest.ActualTimeStart, opt => opt.MapFrom(src => src.ActualTimeStart.HasValue ? src.ActualTimeStart.Value.ToISOString() : null))
                .ForMember(dest => dest.ActualTimeClosed, opt => opt.MapFrom(src => src.ActualTimeClosed.HasValue ? src.ActualTimeClosed.Value.ToISOString() : null))
                .ForMember(dest => dest.State, opt => opt.MapFrom(src => Enum.GetName(typeof(ShiftStateEnum), src.StateId).ToLower()));
            CreateMap<Visit, VisitEntity>()
                .ForMember(dest => dest.SuggestTime, opt => opt.MapFrom(src => src.SuggestTime.ToString("HH:mm:ss")))
                .ForMember(dest => dest.CompleteTime, opt => opt.MapFrom(src => src.CompleteTime.HasValue ? src.CompleteTime.Value.ToISOString() : null))
                .ForMember(dest => dest.StartTime, opt => opt.MapFrom(src => src.StartTime.HasValue ? src.StartTime.Value.ToISOString() : null))
                .ForMember(dest => dest.State, opt => opt.MapFrom(src => Enum.GetName(typeof(VisitStateEnum), src.StateId).ToLower()))
                .ForMember(dest => dest.TotalPayedAmount, opt => opt.MapFrom(src => src.TotalPayedAmount.HasValue ? src.TotalPayedAmount.Value : 0))
                .ForMember(dest => dest.TotalPayedScale, opt => opt.MapFrom(src => src.TotalPayedScale.HasValue ? src.TotalPayedScale.Value : 0));
            CreateMap<Site, SiteEntity>();
            CreateMap<Visit, WebModels.VisitState>()
                .ForMember(dest => dest.State, opt => opt.MapFrom(src => Enum.GetName(typeof(VisitStateEnum), src.StateId).ToLower()))
                .ForMember(dest => dest.StartTime, opt => opt.MapFrom(src => src.StartTime.HasValue ? src.StartTime.Value.ToISOString() : null))
                .ForMember(dest => dest.CompleteTime, opt => opt.MapFrom(src => src.CompleteTime.HasValue ? src.CompleteTime.Value.ToISOString() : null));
            CreateMap<Visit, VisitResult>()
                .ForMember(dest => dest.Goods, opt => opt.MapFrom(src => src.CollectedGoods));
            CreateMap<Price, PricedGood>()
                .ForMember(dest => dest.PriceValue, opt => opt.MapFrom(src => src.Value))
                .ForMember(dest => dest.PriceScale, opt => opt.MapFrom(src => src.Scale))
                .ForMember(dest => dest.PriceTimeSet, opt => opt.MapFrom(src => src.TimeUpdate.ToISOString()))
                .ForMember(dest => dest.Name, opt => opt.MapFrom(src => src.GoodCategory.Name))
                .ForMember(dest => dest.EWCCode, opt => opt.MapFrom(src => src.GoodCategory.EWCCode.Id))
                .ForMember(dest => dest.UnitOfMeasure, opt => opt.MapFrom(src => src.GoodCategory.UnitOfMeasure))
                .ForMember(dest => dest.Order, opt => opt.MapFrom(src => src.GoodCategory.Order));
            CreateMap<Price, PricedGoodV2>()
                .ForMember(dest => dest.PriceValue, opt => opt.MapFrom(src => src.Value))
                .ForMember(dest => dest.PriceScale, opt => opt.MapFrom(src => src.Scale))
                .ForMember(dest => dest.PriceTimeSet, opt => opt.MapFrom(src => src.TimeUpdate.ToISOString()))
                .ForMember(dest => dest.Name, opt => opt.MapFrom(src => src.GoodCategory.Name))
                .ForMember(dest => dest.EWCCode, opt => opt.MapFrom(src => src.GoodCategory.EWCCode.Id))
                .ForMember(dest => dest.UnitOfMeasure, opt => opt.MapFrom(src => src.GoodCategory.UnitOfMeasure))
                .ForMember(dest => dest.Order, opt => opt.MapFrom(src => src.GoodCategory.Order));
            CreateMap<NotePostBody, Note>()
                   .ForMember(dest => dest.EmergencyId, opt => opt.MapFrom(src => src.OptEmergencyId.HasValue ? src.OptEmergencyId.Value : (int?)null))
                   .ForMember(dest => dest.TimeCreated, opt => opt.MapFrom(src => src.CreationTime.HasValue ? src.CreationTime.Value : DateTimeOffset.UtcNow)); 
            CreateMap<Note, NoteEntity>()
                .ForMember(dest => dest.AuthorId, opt => opt.MapFrom(src => src.UserId))
                .ForMember(dest => dest.TimeCreated, opt => opt.MapFrom(src => src.TimeCreated.ToISOString()))
                .ForMember(dest => dest.OptEmergencyId, opt => opt.MapFrom(src => src.EmergencyId.HasValue ? src.EmergencyId.Value : (int?)null));
            CreateMap<Database.EntityModels.CollectedGood, WebModels.CollectedGood>()
                .ForMember(dest => dest.Name, opt => opt.MapFrom(src => src.GoodCategory.Name))
                .ForMember(dest => dest.TotalAmount, opt => opt.MapFrom(src => src.Price.Value * src.Weight))
                .ForMember(dest => dest.TotalScale, opt => opt.MapFrom(src => src.Price.Scale));
            CreateMap<NoteEntity, Note>()
                .ForMember(dest => dest.TimeCreated, opt => opt.MapFrom(src => src.TimeCreated == string.Empty || src.TimeCreated == null
                ? DateTimeOffset.UtcNow : DateTimeOffset.Parse(src.TimeCreated)))
                .ForMember(dest => dest.EmergencyId, opt => opt.MapFrom(src => src.OptEmergencyId.HasValue ? src.OptEmergencyId.Value : (int?)null));
            CreateMap<Emergency, EmergencyEntity>();
            CreateMap<GoodVisitResult, Database.EntityModels.CollectedGood>();
            CreateMap<Database.EntityModels.EWCCode, WebModels.EWCCode>();


        }
    }
}
